/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file botkernel.cpp
 * @brief BotKernel implementation file
 */


#include "botkernel.h"
/// Thread function to manage timeouts
void * threadFunc (void *);

/**
 * The class constructor. Initialize private attributes.
 * Parse the configuration file and initialize bot parameters and objects.
 * Then load plugins
 * @param confFile Configuration file path (including name)
 * @post An object is contructed
 */
BotKernel::BotKernel(string confFile)
{
	this->author = "eponyme (Nicoleau Fabien)";
	this->version = VERSION;
	this->description = "trustyRC kernel";
	this->in_loop_plugins.clear();
	this->in_command_handler_plugins.clear();
   this->in_free_command_handler_plugins.clear();
	this->in_type_handler_plugins.clear();
	this->in_before_treatment_plugins.clear();
	this->in_all_msgs_plugins.clear();
   this->countDowns.clear();
	this->sendQueue.clear();

	this->conff=new ConfigurationFile(confFile);
	if(!this->conff->load()) {
		cerr << "Error loading configuration file (" + confFile+ "). Exiting..." <<endl;
		exit(0);
	}
	if (this->conff->getValue("kernel.verbose")=="1") {
      this->displayLicenceHeader() ;
		this->verbose=true;
   }
	else {
		this->verbose=false;
   }
   this->initDirs();
	this->myLog = new LogFile(this->datasDir+"syslog/"+"trustyrc-",this->verbose,(this->conff->getValue("kernel.logkeepfiles")=="1")?true:false,this->conff->getValue("kernel.logminlevel"),this->conff->getValue("kernel.logperiod"));
   if ( ! this->myLog->open() ) {
      cerr<< "Error opening log file. Nothing will be logged" << endl;
   }
   this->conff->addProtectedKey("remotecontrol.port");
   this->conff->addProtectedKey("remotecontrol.accounts");
   this->conff->addProtectedKey("remotecontrol.maxclients");
   this->conff->addProtectedKey("admin.sapass");
	this->sock=new Socket(); 
   time(&this->startTime);
	this->connected=false;
	this->turn=true;	
	this->loadPlugins(true);
   this->AEX.penality = 0;
   time(&this->AEX.last_decrease);
}

/**
 * The class destructor
 * Unload plugins, stop threads, and delete objects
 */
BotKernel::~BotKernel()
{
	this->myLog->log("Going to delete kernel objects ...",INFO);
	this->unloadMyPlugins(false);
	delete(this->sock);
	delete(this->conff);
	delete(this->myLog);
}

/**
 * Connect the bot to the IRC server.
 * Connection parameters are picked in the configuration file.
 */
void BotKernel::connect()
{
	this->turn=true;
	this->myLog->log("Trying to connect to " + this->conff->getValue("kernel.server"),INFO);
   while(!this->sock->connectSock(Tools::strToInt(this->conff->getValue("kernel.port")),this->conff->getValue("kernel.server"),this->conff->getValue("kernel.dedicatedIP")))
   {	
      this->myLog->log("Unable to connect, waiting "+this->conff->getValue("kernel.reco_wait")+" seconds ...",ERROR);
      sleep(Tools::strToInt(this->conff->getValue("kernel.reco_wait"))); 
   }
	this->send(IRCProtocol::identify(this->conff->getValue("kernel.password"),this->conff->getValue("kernel.ident"),this->conff->getValue("kernel.name"),this->conff->getValue("kernel.nick")));
	this->setConnected(true);
	this->myLog->log("Bot connected : " + this->conff->getValue("kernel.server")+":"+this->conff->getValue("kernel.port")+" (password:"+this->conff->getValue("kernel.password")+") "+this->conff->getValue("kernel.nick")+" "+this->conff->getValue("kernel.ident")+((this->conff->getValue("kernel.dedicatedIP")=="")?"":" through "+this->conff->getValue("kernel.dedicatedIP")),INFO);
	this->setNick(this->conff->getValue("kernel.nick"));
   time(&this->startOnline);
}

/**
 * Makes the bot reload the connection
 * All connections objects a re-initialised
 */
void BotKernel::reconnect()
{
	this->myLog->log("Bot going to reconnect",INFO);
   this->sock->closeSock();
	delete(this->sock); 
	this->sock=new Socket();
	this->setConnected(false);
	this->connect();
}

/**
 * Kernel process
 * Manage objects to receive, send and treat messages
 */
void BotKernel::run()
{
	time_t now;
   vector<CountDownFunction>::iterator it;
   list<string> queue;
	bool treatMsg = true;
   bool complete;
   unsigned int startIndex;
	Message m;
   string buffer;
   vector<string> messages;
   Tools::log(this->getDatasDir()+"trustyrc.pid",Tools::intToStr(getpid()),false,true) ;
   this->connect();
   queue.clear();
   while(this->turn)
   {
      if ((!queue.empty())&&(queue.front()[queue.front().length()-1]=='\n')){ // check if a message is present, and if it's complete
        buffer = queue.front();
        queue.pop_front();
      }
      else {
         buffer = this->sock->receive();
         if (!this->sock->getState()) {
            this->myLog->log("Disconnected (read error)",WARNING);
            this->setConnected(false);
         }
         else {
            if (buffer.length() > 2 )
            {
               messages = Tools::stringToVector(buffer,"\n",0);
               if ( buffer[buffer.length()-1] == '\n') { // check if last received message is complete
                  complete = true;
               }
               else {
                  complete = false;
               }
               if((!queue.empty())&&(queue.back()[queue.back().length()-1]!='\n')) { // check if last message stored in queue is not complete
                  queue.back() += messages[0]+'\n';
                  startIndex = 1;
               }
               else {
                  startIndex = 0 ;   
               }
               for(unsigned int i = startIndex; i < messages.size() ; i ++ ) {
                  queue.push_back(messages[i]+'\n');
               } 
               if (!complete) {
                  queue.back() = queue.back().substr(0,queue.back().length()-1);     
               }
               buffer = queue.front();
               queue.pop_front();
            }  
         }
      }
      if (buffer.length() > 2 )
      {
         // \r\n 
         if ( buffer[buffer.length()-2] == '\r' ) {
            buffer.erase(buffer.length()-2); 
         } // \n
         else { 
            buffer.erase(buffer.length()-1); 
         }
         if(this->verbose) {
            cout << buffer << endl;
         }
         m.setMessage(buffer);
		   treatMsg = true;
		   // IN_BEFORE_TREATMENT
		   for ( unsigned int i = 0 ; i < this->in_before_treatment_plugins.size() ; i ++ )
		   {
		      if(!this->executeFunction(&m,this->in_before_treatment_plugins[i])) {
		         treatMsg = false;
			   	break;
		      }
		   }  
		   if(treatMsg) {
	         this->msgTreatment(&m);
         }
      }
		// IN_LOOP & COUNTDOWN
		time(&now);
      it = this->countDowns.begin();
      while ( it != this->countDowns.end() ) {
         if ( now >= (*it).timestamp ) {
            if(this->executeFunction(&((*it).msg),(*it).function)) {
               this->countDowns.erase(it);
            }
            else {
               (*it).timestamp = now + (*it).count;
               it ++ ;
            }
         }
         else {
            it ++ ;
         }
      }
		for ( unsigned int i = 0 ; i < this->in_loop_plugins.size() ; i ++ )
		{
			if ( this->in_loop_plugins[i].highlightedWord == "") {
				this->executeFunction(NULL,this->in_loop_plugins[i]);
			}
			else {
				if ((now-this->in_loop_plugins[i].lastExec)>= Tools::strToInt(this->in_loop_plugins[i].highlightedWord) ) {
					this->executeFunction(NULL,this->in_loop_plugins[i]);
					time(&this->in_loop_plugins[i].lastExec);
				}
			}
		}
		if (!this->connected && this->turn)
		{
			this->myLog->log("Disconnected, waiting 10 seconds ...",INFO);
			sleep(10);
			this->reconnect();
		}
   }
   this->sock->closeSock();
}

/**
 * Stop the bot
 */ 
void BotKernel::stop()
{
	this->myLog->log("stopped !",INFO);
	this->turn = false;
}

/**
 * Load all the plugins mentionned in the configuration file.
 * @param checkDependancy Set this to true to check for plugin requirements
 */
void BotKernel::loadPlugins(bool checkDependancy)
{
	int plugOK=0,plugNOK=0;
	vector<string> plugins = Tools::stringToVector(this->conff->getValue("kernel.plugins"),",",0);
	for(unsigned int i =0;i < plugins.size() ; i ++ )
	{
		if(this->loadPlugin(plugins[i],checkDependancy)) {
			plugOK++;
		}
		else {
			plugNOK++;
		}
	}
	this->myLog->log(Tools::intToStr(plugOK)+" plugin(s) successfully loaded. "+Tools::intToStr(plugNOK)+" plugin(s) unsuccessfully loaded.",INFO);
}

/**
 * Load a plugin, to make all its functions usable by the bot
 * @post The kernel can use new functions
 * @param fileName Dynamic librairy containing the plugins. Just the name, not the path
 * @param checkDependancy Set this to true to check for plugin requirements
 * @return true if the plugin has correctly been loaded, else false
 */
bool BotKernel::loadPlugin(string fileName,bool checkDependancy)
{
	void*handle;
   char * error;
	Plugin* obj;
   const unsigned int NBPATHS = 3;
   vector<string> requirements;
	plugin_constructor constructor;
	plugin_destructor destructor;
	StructFunctionStorage func;
	vector<StructFunctionStorage> funcs;
	pPlugin plug;
   string paths[NBPATHS] = {this->datasDir+"../plugins",PLUGINSDIR,"plugins"};

   for(unsigned int i = 0 ; i < NBPATHS ; i ++ ) {
   	handle = dlopen((paths[i]+"/"+fileName).c_str(), RTLD_NOW);
   	if (handle==NULL) {
         continue;
      }
      constructor = (plugin_constructor)(intptr_t) dlsym(handle,("contruct_"+fileName.substr(0,fileName.find("."))).c_str()); //*(void **)(&constructor) = dlsym(handle,("contruct_"+fileName.substr(0,fileName.find("."))).c_str()); //constructor = (plugin_constructor) dlsym(handle,("contruct_"+fileName.substr(0,fileName.find("."))).c_str());
      if (constructor==NULL) {
		   this->myLog->log("Cannot load symbol create("+fileName+"): " + dlerror(),ERROR);
         return false;
      }
	   destructor = (plugin_destructor)(intptr_t) dlsym(handle, ("destroy_"+fileName.substr(0,fileName.find("."))).c_str()); //*(void **)(&destructor) = dlsym(handle, ("destroy_"+fileName.substr(0,fileName.find("."))).c_str()); // destructor = (plugin_destructor) dlsym(handle, ("destroy_"+fileName.substr(0,fileName.find("."))).c_str());
      if (destructor==NULL) {
		   this->myLog->log("Cannot load symbol destroy("+fileName+"): " + dlerror(),ERROR);
         return false;
      }	
	   obj = constructor(this);

   	plug.name = obj->getName();
   	plug.handle = handle;
   	plug.object = obj ;
   	plug.creator = constructor;
   	plug.destructor = destructor;

	   if ( !obj->checkMembers() ) {
	   	this->myLog->log("Attributs error("+fileName+")",ERROR);
		   plug.destructor(plug.object);
   		dlclose(plug.handle);
	   	return false;
   	}
   	if ( this->pluginLoaded(plug.name) ) {
   		this->myLog->log("Plugin \"" + plug.name + "\" already loaded",WARNING);
	   	plug.destructor(plug.object);
   		dlclose(plug.handle);
   		return false;
   	}
      if(checkDependancy) {
         requirements = plug.object->getRequirements() ;
         for(unsigned int i = 0 ; i < requirements.size() ; i ++ ) {
            if(!this->pluginLoaded(requirements[i]) ) {
               this->myLog->log("Cannot load "+plug.name+". "+requirements[i]+" is required and not loaded",ERROR);
		         plug.destructor(plug.object);
		         dlclose(plug.handle);
		         return false;
            }
         }
      }
      plug.object->setHandle(plug.handle);
	   this->myPlugins.push_back(plug);
	   funcs = obj->getFunctions();
	   for (unsigned int i = 0 ; i < funcs.size() ; i ++ ) {
		   func = funcs[i];
		   func.handle = handle;	
         func.function = (plugin_function)(intptr_t) dlsym(handle, func.symbole.c_str()); //*(void **)(&func.function) = dlsym(handle, func.symbole.c_str()); // func.function = (plugin_function) dlsym(handle, func.symbole.c_str());
		   if (func.function==NULL) {
			   this->myLog->log("Cannot load symbol "+func.symbole+" ("+fileName+"): " + dlerror(),WARNING);
		   }	
		   else {
            this->storeFunction(&func);
		   }
	   }
	   this->myLog->log("Plugin "+fileName+" successfully loaded",INFO);
	   return true;
   }
   error = dlerror();
   if ( error != NULL) {
      this->myLog->log("Cannot load library " +fileName+":" + error,ERROR);
   }
   return false;
}

/**
 * Unload a plugin. Functions provided by the plugin 
 * can no more be used by the kernel.
 * @param name Plugin name
 * @param checkDependancy Set this to true to check for plugin requirements
 * @return true if the plugin has correctly been unloaded, else false
 */
bool BotKernel::unloadPlugin(string name,bool checkDependancy)
{
	void * handle;
   vector<pPlugin>::iterator it = this->myPlugins.begin();
	vector<StructFunctionStorage>::iterator it2 ;
   if (checkDependancy) {
      for(unsigned int i = 0 ; i < this->myPlugins.size() ; i++ ) {
         if ( this->myPlugins[i].object->requires(name) ) {
            this->myLog->log("Cannot unload "+name+". "+myPlugins[i].name+" requires this plugin and is loaded",ERROR);
            return false;
         }
      }
   }
   while ( it != this->myPlugins.end() )
   {
      if ( (*it).name == name )
      {
			handle = (*it).handle;
			(*it).destructor((*it).object);
         this->myPlugins.erase(it);
			it2 = this->in_loop_plugins.begin();
			while ( it2 != this->in_loop_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_loop_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_command_handler_plugins.begin();
			while ( it2 != this->in_command_handler_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_command_handler_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_free_command_handler_plugins.begin();
			while ( it2 != this->in_free_command_handler_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_free_command_handler_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_type_handler_plugins.begin();
			while ( it2 != this->in_type_handler_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_type_handler_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_before_treatment_plugins.begin();
			while ( it2 != this->in_before_treatment_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_before_treatment_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_all_msgs_plugins.begin();
			while ( it2 != this->in_all_msgs_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_all_msgs_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->in_first_word_plugins.begin();
			while ( it2 != this->in_first_word_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->in_first_word_plugins.erase(it2);
				else 
					it2 ++;
			}
			it2 = this->out_all_msgs_plugins.begin();
			while ( it2 != this->out_all_msgs_plugins.end() ) {
				if ( (*it2).handle == handle )
					this->out_all_msgs_plugins.erase(it2);
				else 
					it2 ++;
			}
			this->myLog->log("Plugin "+name + " unloaded",INFO);
			dlclose(handle);
         return true;
      }
      else
         it ++ ;
   }
   return false;
}

/**
 * Unload all the loaded plugins.
 * Makes the bot run only with the kernel (no functionnalities)
 * @param checkDependancy Set this to true to check for plugin requirements
 */
void BotKernel::unloadMyPlugins(bool checkDependancy)
{
	while(this->myPlugins.size()>0)
	{
		this->unloadPlugin(this->myPlugins[0].name,checkDependancy);
	}
}

/** 
 * Check if a plugin is loaded.
 * The result is based on the plugin NAME (object attribute), not on the file name 
 * containing the plugin
 * @param name Plugin name
 * @return True if the given plugin is loaded, else false
 */
bool BotKernel::pluginLoaded(string name)
{
	for ( unsigned int i = 0 ; i < this->myPlugins.size() ; i ++ )
	{
		if (this->myPlugins[i].name == name )
			return true;
	}
	return false;
}

/** 
 * Check if a plugin is loaded.
 * The result is based on the plugin handle 
 * @param handle Plugin handle
 * @return True if the given plugin is loaded, else false
 */
bool BotKernel::pluginLoaded(void*handle)
{
	for ( unsigned int i = 0 ; i < this->myPlugins.size() ; i ++ )
	{
		if (this->myPlugins[i].handle == handle )
			return true;
	}
	return false;
}

/**
 * Store a function and return it's id
 * @param func Function to store
 * @return function'address. NULL if nothing inserted
 */
plugin_function BotKernel::storeFunction(StructFunctionStorage*func)
{
   switch (func->type) {
	   case IN_LOOP : this->in_loop_plugins.push_back(*func);    
		break;
		case IN_COMMAND_HANDLER : this->in_command_handler_plugins.push_back(*func);
		break;
      case IN_FREE_COMMAND_HANDLER : this->in_free_command_handler_plugins.push_back(*func);
      break;
		case IN_TYPE_HANDLER : this->in_type_handler_plugins.push_back(*func);
		break;
		case IN_BEFORE_TREATMENT : this->in_before_treatment_plugins.push_back(*func);
		break;
		case IN_ALL_MSGS : this->in_all_msgs_plugins.push_back(*func);
		break;
      case IN_FIRST_WORD : this->in_first_word_plugins.push_back(*func) ;
      break;
      case OUT_ALL_MSGS : this->out_all_msgs_plugins.push_back(*func) ;
      break;
	   default : this->myLog->log("Undefined func type for "+func->symbole,WARNING);
                return NULL;
	}
   return func->function ;
}

/**
 * Register a new function.
 * The function is added in the plugins queues and can be triggered
 * @param hlword higlited word
 * @param object plugin's object
 * @param type function type
 * @param symbole symbole to execute
 * @param function function to execute
 * @param lastExec last exec
 * @param timeout fuction timeout
 * @return a pointer on the function, NULL if the plugin does no exists
 */
plugin_function BotKernel::registerFunction(string hlword,Plugin*object,func_type type,string symbole,plugin_function function,time_t lastExec,unsigned int timeout)
{
   if (!this->pluginLoaded(object->getHandle())) {
      return NULL;
   }
   StructFunctionStorage store;
   store.handle = object->getHandle();
   store.highlightedWord = hlword;
   store.object = object;
   store.type = type;
   store.symbole = symbole;
   store.function = function;
   store.lastExec = lastExec;
   store.timeout = timeout;
   return this->storeFunction(&store);
}

/**
 * Unregister a function
 * @param func Function pointer
 */
void BotKernel::unregisterFunction(plugin_function func)
{
   vector<StructFunctionStorage>::iterator it ;
   vector<CountDownFunction>::iterator itCount;
   it = this->in_loop_plugins.begin();
   while ( it != this->in_loop_plugins.end() ) {
      if ( (*it).function == func ) { this->in_loop_plugins.erase(it); return; }
      it++;
	}
	it = this->in_command_handler_plugins.begin();
   while ( it != this->in_command_handler_plugins.end() ) {
      if ( (*it).function == func ) { this->in_command_handler_plugins.erase(it); return; }
      it++;
	}
	it = this->in_free_command_handler_plugins.begin();
   while ( it != this->in_free_command_handler_plugins.end() ) {
      if ( (*it).function == func ) { this->in_free_command_handler_plugins.erase(it); return; }
      it++;
	}
	it = this->in_type_handler_plugins.begin();
   while ( it != this->in_type_handler_plugins.end() ) {
      if ( (*it).function == func ) { this->in_type_handler_plugins.erase(it); return; }
      it++;
	}
	it = this->in_before_treatment_plugins.begin();
   while ( it != this->in_before_treatment_plugins.end() ) {
      if ( (*it).function == func ) { this->in_before_treatment_plugins.erase(it); return; }
      it++;
	}
	it = this->in_all_msgs_plugins.begin();
   while ( it != this->in_all_msgs_plugins.end() ) {
      if ( (*it).function == func ) { this->in_all_msgs_plugins.erase(it); return; }
      it++;
	}
	it = this->in_first_word_plugins.begin();
   while ( it != this->in_first_word_plugins.end() ) {
      if ( (*it).function == func ) { this->in_first_word_plugins.erase(it); return; }
      it++;
	}
	it = this->out_all_msgs_plugins.begin();
   while ( it != this->out_all_msgs_plugins.end() ) {
      if ( (*it).function == func ) { this->out_all_msgs_plugins.erase(it); return; }
      it++;
	}
   itCount = this->countDowns.begin();
   while ( itCount != this->countDowns.end() ) {
      if ( (*itCount).function.function == func ) { this->countDowns.erase(itCount); return; }
      itCount++;
	}
}

/**
 * Add a count down to the kernel.
 * Functions stored will be executed at the end of their count down
 * A countdown function return false if it must not be deleted after execution (re launch countdown)
 * @param p Plugin's object
 * @param f Function to execute
 * @param m Message
 * @param count  countdown
 * @param timeout Function timeout
 * @return function adress
 */
plugin_function BotKernel::addCountDown(Plugin*p,plugin_function f,Message* m,unsigned int count,unsigned int timeout)
{  
   string max = this->conff->getValue("kernel.maxcountdowns") ;
   if ( (max=="") || (max=="0") || (Tools::strToUnsignedInt(max)>=this->countDowns.size()) ) {
      time_t now;
      time(&now);
      StructFunctionStorage sfs ;
      CountDownFunction cdf;
      Message msg(m->getMessage());
      sfs.handle = p->getHandle();
      sfs.highlightedWord = "";
      sfs.object = p;
      sfs.type = COUNTDOWN;
      sfs.symbole = "countDown";
      sfs.function = f;
      sfs.lastExec = 0;
      sfs.timeout = timeout;
      cdf.function = sfs;
      cdf.msg = msg;
      cdf.count = count;
      cdf.timestamp=now+count;
      this->countDowns.push_back(cdf);
      return f;
   }
   else {
      this->myLog->log("Max count downs reached : "+Tools::intToStr(this->countDowns.size()),WARNING);
      return NULL;
   }
}

/**
 * Treat a message according to its type.
 * Each type of plugin is tested to watch if the message is concerned
 * @param msg Pointer to the message object
 */
void BotKernel::msgTreatment(Message* msg)
{
   // IN_FIRST_WORD
   for ( unsigned int i = 0 ; i < this->in_first_word_plugins.size() ; i ++ ) {
      if ( msg->getPart(0) == this->in_first_word_plugins[i].highlightedWord ) {
         this->executeFunction(msg,this->in_first_word_plugins[i]);      
      }
   }
	// IN_TYPE_HANDLER
	if ( msg->getSplit().size() >= 2 ) {
		for ( unsigned int i = 0 ; i < this->in_type_handler_plugins.size() ; i ++ ) {
			if ( msg->getPart(1) == this->in_type_handler_plugins[i].highlightedWord ) {
				this->executeFunction(msg,this->in_type_handler_plugins[i]);
         }
		}
	}
	// IN_COMMAND_HANDLER & IN_FREE_COMMAND_HANDLER
	if ( msg->nbParts() >= 4 ) {
		for ( unsigned int i = 0 ; i < this->in_command_handler_plugins.size() ; i ++ ) {
			if ( Tools::to_lower(msg->getPart(3)) == ":"+this->conff->getValue("kernel.command_prefix")+Tools::to_lower(this->in_command_handler_plugins[i].highlightedWord) ) {
				if(!this->executeFunction(msg,this->in_command_handler_plugins[i])) {
               return ;
            }
         }
		}
		for ( unsigned int i = 0 ; i < this->in_free_command_handler_plugins.size() ; i ++ ) {
			if ( msg->getPart(3) == ":"+this->in_free_command_handler_plugins[i].highlightedWord ) {
				if(!this->executeFunction(msg,this->in_free_command_handler_plugins[i])) {
               return;
            }
         }
		}  
	}
	// IN_ALL_MSGS
	for ( unsigned int i = 0 ; i < this->in_all_msgs_plugins.size() ; i ++ ) {
      this->executeFunction(msg,this->in_all_msgs_plugins[i]);
	}
}

/**
 * Execute a function, managing the timeout
 * @param m Message pointer
 * @param pfs Function storage
 * @return True if the message treatment must continue, else false
 */
bool BotKernel::executeFunction(Message*m,StructFunctionStorage pfs)
{
   //cout << "Execution de : " << pfs.symbole << endl;
   sem_t my_sem;
   sem_init (&my_sem, 0, 0);
   CPPThread th;
   ThreadParams tp;
   tp.function = &pfs;
   tp.b = this;
   tp.msg = m;
   tp.sem = &my_sem;
   struct timespec my_timeout;
   struct timeval now;
   gettimeofday(&now,NULL);
   my_timeout.tv_sec = now.tv_sec + pfs.timeout;
   my_timeout.tv_nsec = now.tv_usec * 1000;
   th.exec(threadFunc,&tp);
   int back = sem_timedwait(&my_sem,&my_timeout);
   if ((back == -1)&&(errno == ETIMEDOUT)) {
      this->myLog->log(pfs.symbole +" timed out",WARNING);
      th.terminate();
      if ((m!=NULL) && (pfs.type!=OUT_ALL_MSGS)) {
         this->send(IRCProtocol::sendNotice(m->getNickSender(),pfs.symbole +" timed out")) ;
      }
      return true;
   }
   else {
      th.join();
      return pfs.back;
   }
   return true;
}

/**
 * Send a string to the server
 * @param str String to send
 */
void BotKernel::send(string str)
{
   unsigned int diff=0;
   time_t now;
   Message msg;
   const unsigned int MAXCHARS = 450;
   string command;
   vector<string> messages;
   bool sended;
   messages.clear();
   if ( (str.length() > MAXCHARS) && (str.find("PRIVMSG ")!=string::npos) ){
      command = str.substr(0,str.find(":")+1);
      for ( unsigned int i = command.length(); i < str.length() ; i += MAXCHARS-command.length() ) {
         messages.push_back(command+str.substr(i,MAXCHARS-command.length()));
      }
   }
   else {
      messages.push_back(str);
   }
   for(unsigned int i = 0 ; i < messages.size();i ++ ) {
      // OUT_ALL_MSGS
      msg.setMessage(messages[i]);
   	for ( unsigned int j = 0 ; j < this->out_all_msgs_plugins.size() ; j ++ ) {
         if (!this->executeFunction(&msg,this->out_all_msgs_plugins[j]) ) {
            return;
         }
   	}
      sended = false;
      if ( this->conff->getValue("kernel.antief") == "1" ) {
         while (!sended)
         {
            time(&now);
            diff = now - this->AEX.last_decrease;
            if ( diff > 0 ) {
               this->AEX.penality -= diff;
               if (this->AEX.penality < 0 )
                  this->AEX.penality = 0 ;
               time(&this->AEX.last_decrease);
            }
            if (( this->AEX.penality + 2 ) < 10 ) {
               if ( !this->sock->sendStr(msg.getMessage()+"\n") ) {
                  this->myLog->log("Disconnected (write error)",WARNING);
                  this->setConnected(false);
               }
               sended = true;
               this->AEX.penality += 2 ;
            } 
         }
      }
      else {
         if ( !this->sock->sendStr(msg.getMessage()+"\n") ) {
            this->myLog->log("Disconnected (write error)",WARNING);
            this->setConnected(false);
         }   
      }
   }
}

/**
 * Send strings to the server
 * @param vec Vector containing strings to send
 */
void BotKernel::send(vector<string> vec)
{
	for ( unsigned int i = 0 ; i < vec.size() ; i ++ ) {
		this->send(vec[i]);
	}
}

/**
 * Display licence header
 */
void BotKernel::displayLicenceHeader() {
   cout << "You are running trustyRC "<<this->version<<" by "+this->author+". A C++ IRC robot" << endl;
   cout << "trustyRC Copyright (C) 2006-2009  Nicoleau Fabien" << endl;
   cout << "This program comes with ABSOLUTELY NO WARRANTY." << endl;
   cout << "This is free software, and you are welcome to redistribute it under certain conditions." << endl;
   cout << "Read COPYING file for details." << endl << endl;
}

/**
 * Create needed directories
 */
void BotKernel::initDirs() {
   char * pHOME = NULL;
   DIR*directory = NULL;
   pHOME = getenv("HOME");
   string confName = this->conff->getFilePath().substr(this->conff->getFilePath().rfind("/")+1,this->conff->getFilePath().rfind(".")-this->conff->getFilePath().rfind("/")-1);
   if ( pHOME != NULL ) {
      this->datasDir = (string)pHOME+"/.trustyrc/";
   }
   else {
      this->datasDir = "/tmp/trustyrc/";
      cerr<< "Wrong datasdir value. Using "+this->datasDir << endl;
   }
   directory = opendir(this->datasDir.c_str()) ;
   if (directory==NULL) {
      if (mkdir(this->datasDir.c_str(),0755) == -1 ) {
         cerr << "Unable to create datas dir (check write access?). Can't continue ..." << endl;
         exit(-1);
      }
   }
   closedir(directory);
   directory = opendir((this->datasDir+"plugins").c_str());
   if (directory==NULL) {
      if (mkdir((this->datasDir+"plugins").c_str(),0755) == -1 ) {
         cerr << "Unable to create alias dir (check write access.). Can't continue ..." << endl;
         exit(-1);
      }
   }
   closedir(directory);
   directory = opendir((this->datasDir+confName).c_str());
   if (directory==NULL) {
      if (mkdir((this->datasDir+confName).c_str(),0755) == -1 ) {
         cerr << "Unable to create alias dir (check write access.). Can't continue ..." << endl;
         exit(-1);
      }
   }
   closedir(directory);
   this->datasDir += (confName+"/") ;
   directory = opendir((this->datasDir+"syslog").c_str());
   if (directory==NULL) {
      if (mkdir((this->datasDir+"syslog").c_str(),0755) == -1 ) {
         cerr << "Unable to create syslog dir (check write access). Nothing will be logged ..." << endl;
      }
   }
   closedir(directory);
}

/**
 * Gives A pointer to the ConfigurationFile's kernel objet
 * @return A pointer to the ConfigurationFile's kernel objet
 */
ConfigurationFile* BotKernel::getCONFF()
{
	return this->conff;
}

/**
 * Get kernel author
 * @return Kernel's author
 */
string BotKernel::getAuthor()
{
	return this->author;
}

/**
 * Get kernel version
 * @return Kernel version
 */
string BotKernel::getVersion()
{
	return this->version;
}

/**
 * Get kernel description
 * @return Kernel description
 */
string BotKernel::getDescription()
{
	return this->description;
}

/**
 * Gives A pointer to the LogFile's kernel objet
 * @return A pointer to the LogFile's kernel objet
 */
LogFile* BotKernel::getSysLog()
{
	return this->myLog;
}

/**
 * Gives bot's nick
 * @return bot's nick
 */
string BotKernel::getNick()
{
	return this->nick;
}

/**
 * Set bot's nick
 * @param nick new nick
 */
void BotKernel::setNick(string nick)
{
   //this->conff->setValue("kernel.nick",nick);
	this->nick = nick;
}

/**
 * get the bot connexion state
 * @return true for connected, false for disconnected
 */
bool BotKernel::getConnected()
{
   return this->connected;
}

/**
 * Set the bot connexion state
 * This state is important to make the bot reset if disconnected
 * @param state true for connected, false for disconnected
 */
void BotKernel::setConnected(bool state)
{
	this->connected = state;
}

/**
 * Return a plugin informations
 * Informations are name, handle, constructor, destructor ... 
 * witch name is given by the "name" parameter
 * @param name Plugin name
 * @return Plugin storage pointer. NULL if the plugin is not loaded
 */
pPlugin* BotKernel::getPlugin(string name)
{
	if ( !this->pluginLoaded(name) )
	{
		return NULL;
	}
	else
	{
		for ( unsigned int i = 0 ; i < this->myPlugins.size() ; i ++ )
		{
			if ( this->myPlugins[i].name == name)
			{
				return &this->myPlugins[i] ;
			}
		}
		return NULL;
	}
	return NULL;
}

/**
 * Return the plugins list
 * @return A vector containing plugins names
 */
vector<string> BotKernel::getPluginsList()
{
   vector<string> back;
	for ( unsigned int i = 0 ; i < this->myPlugins.size() ; i ++ )
	{
      back.push_back(this->myPlugins[i].name);
	}
   return back; 
}

/**
 * Return power on timestamp
 * @return Power on timestamp
 */
time_t BotKernel::getStartTime()
{
   return this->startTime;
}

/**
 * Return connection timestamp
 * @return connection timestamp
 */
time_t BotKernel::getStartOnline()
{
   return this->startOnline;
}

/**
 * Get a pointer on countdowns
 * @return The pointer
 */
vector<CountDownFunction>* BotKernel::getCountDowns()
{
   return &this->countDowns ;
}

/**
 * Get bot's datas directory
 * @return bot's datas directory
*/
string BotKernel::getDatasDir() 
{
   return this->datasDir;
}

void* threadFunc (void*args) 
{
   ThreadParams* tp = (ThreadParams*) args;
   StructFunctionStorage* pfs = tp->function;
   pfs->back = pfs->function(tp->msg,pfs->object,tp->b);
   sem_post (tp->sem);
   return NULL;
}

