/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file botkernel.h
 * @brief BotKernel header file
 */

#ifndef BOTKERNEL_H
#define BOTKERNEL_H

// timeout ----
/*#include <unistd.h>
#include <signal.h>
#include <setjmp.h>*/
// ------------
#include "message.h"
#include "ircprotocol.h"
#include "configurationfile.h"
#include "logfile.h"
#include "tools.h"
#include "socket.h"
#include "plugin.h"
#include "cppthread.h"
#include <sys/time.h>
#include <sys/types.h>
#include <semaphore.h>
#include <errno.h>
#include <dlfcn.h>
#include <dirent.h>
#include <time.h>
#include <list>
using namespace std;

/// Anti excess-flood variables
typedef struct 
{
   time_t last_decrease;
   int penality  ;
}AntiExcessFlood;

/// Countdown information storage
typedef struct {  
   StructFunctionStorage function;
   Message msg;
   time_t timestamp;
   unsigned int count;
}CountDownFunction;

/// Thread information storage
typedef struct {  
   StructFunctionStorage* function;
   Message* msg;
   BotKernel* b;
   sem_t* sem;
}ThreadParams;

/** @brief Bot kernel class
 *
 * This class manages the bot kernel. It can load plugins, 
 * and uses all project class
 */
class BotKernel
{
   public:
		/// Constructor
      BotKernel(string);
		/// Destructor
      ~BotKernel();
      /// Register a function 
      plugin_function registerFunction(string,Plugin*,func_type,string,plugin_function,time_t,unsigned int);
      /// Unregister a finction
      void unregisterFunction(plugin_function);
      /// Add a countdown 
      plugin_function addCountDown(Plugin*,plugin_function,Message*,unsigned int,unsigned int);
		/// Launch kernel process
		void run();
		/// Stop kernel process
		void stop();
		/// Send a message
		void send(string);
		/// Send a list (in a vector) of messages
		void send(vector<string>);
		/// Return the kernel version
		string getVersion();
		/// Return the kernel description
		string getDescription();
		/// Return the kernel author
		string getAuthor();
		/// set the bot connection state
		void setConnected(bool);
		/// get the bot connection state
		bool getConnected();
		/// Return the SysLog object pointer
		LogFile* getSysLog();
		/// get bot's nick
      string getNick();
		/// set bot's nick
		void setNick(string);
		/// Load a plugin
		bool loadPlugin(string,bool);
		/// Unload a plugin
		bool unloadPlugin(string,bool);
		/// Send a plugin informations
		pPlugin* getPlugin(string);
      /// Send plugin list
      vector<string> getPluginsList();
      /// Return power on timestamp
      time_t getStartTime();
      /// Return connection timestamp
      time_t getStartOnline();
      /// get a pointer on countdowns
      vector<CountDownFunction>* getCountDowns();
		/// Return the configuration file object pointer
		ConfigurationFile* getCONFF();
      /// Return datasdir
      string getDatasDir();

   private:
		/// Bot connection state
		bool connected;
		/// Bot version
		string version;
		/// Bot description
		string description;
		/// Bot author
		string author;
		/// Contains messages to send
		list<string> sendQueue;
		/// True if the bot must process
		bool turn;
      /// bot's datas directory
      string datasDir;
		/// True if received messages must be displayed
		bool verbose;
		/// Timestamp representing the bot launch time
      time_t startTime;
		/// Timestamp representing the bot connection time
      time_t startOnline;
		/// SysLog pointer
		LogFile* myLog;
		/// ConfigurationFile pointer
		ConfigurationFile * conff;
		/// Socket pointer
		Socket* sock;
		/// "loop" plugins functions storage
		vector<StructFunctionStorage> in_loop_plugins;
		/// "command handler" plugins functions storage
		vector<StructFunctionStorage> in_free_command_handler_plugins;
		/// "free command handler" plugins functions storage
		vector<StructFunctionStorage> in_command_handler_plugins;
		/// "type" plugins functions storage
		vector<StructFunctionStorage> in_type_handler_plugins;
		/// "before traitement" plugins functions storage
		vector<StructFunctionStorage> in_before_treatment_plugins;
		/// "all messages" plugins functions storage
		vector<StructFunctionStorage> in_all_msgs_plugins;
		/// "First word check" plugins functions storage
		vector<StructFunctionStorage> in_first_word_plugins;
		/// "all outgoing messages" plugins functions storage
		vector<StructFunctionStorage> out_all_msgs_plugins;
		/// plugins object en headers
		vector<pPlugin> myPlugins;
      /// Countdowns storage
      vector<CountDownFunction> countDowns;
		/// Bot's nick
      string nick ;
      /// Display licence header
      void displayLicenceHeader();
      /// Create needed directories
      void initDirs() ;
      /// Execute a plugin function
      bool executeFunction(Message*,StructFunctionStorage);
      /// Store a plugin function
      plugin_function storeFunction(StructFunctionStorage*);
      /// Anti excess flood variables
      AntiExcessFlood AEX;
		/// Treat a received message
		void msgTreatment(Message*);
		/// Connect the bot
		void connect();
		/// Reconnect the bot
		void reconnect();
		/// Load all the plugins mentionned in the configuration file
		void loadPlugins(bool);
		/// Unload all the loaded plugins
		void unloadMyPlugins(bool);
		/// Tell if a plugin is loaded (based on the plugin's name)
		bool pluginLoaded(string);
		/// Tell if a plugin is loaded (based on the plugin's handle)
		bool pluginLoaded(void*);
};
#endif
