/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file channel.cpp
 * @brief Channel implementation file
 */

#include "channel.h"

/**
 * Class constructor
 * Initialize private attributes
 * @param name Channel name
 */
Channel::Channel(string name)
{
	this->topic = "";
	this->name = name;
   this->users.clear();
   for(unsigned int i=0;i<4;i++) {
      this->lastPart[i] = "";
   }
	time(&this->lastWhoUpdate);
}

/**
 * Destructor
 * Clear private attributes
 */
Channel::~Channel()
{
	this->truncateUsersList();
}

/**
 * Get channel name
 * @return Channel name
 */
string Channel::getName()
{
	return this->name;
}

/**
 * Add a user to the channel
 * It's then possible to follow status
 * @param nick User nick
 * @param host User host
 * @param ident User ident
 * @param status User status
 * @return true if operation ok, else false (already added ?)
 */
bool Channel::addUser(string nick,string host,string ident,string status)// nick host ident status
{
	string * buddy = new string[4];
	buddy[0]=nick;
	buddy[1]=host;
	buddy[2]=ident;
	buddy[3]=status;
	if ( this->getHostByNick(nick) == "" ) { //check if this nick is already registred
		this->users.push_back(buddy);
		return true;
	}
	else {
		return false;
	}
}

/**
 * Return a vector iterator according to arguments
 * @param comparator String to compair with the vector
 * @param index User's infos index
 * @return Iterator usable for a vector
 */
vector<string*>::iterator Channel::getIterator(string comparator,unsigned int index)
{
	if ( index > 3 )
		return this->users.end()+1;
	vector<string*>::iterator it = this->users.begin();
	while (it != this->users.end()) {
		if ( (*it)[index] == comparator ) {
			return it;
		}
		else
			it ++;
	}
	return this->users.end()+1;
}

/**
 * Del a user from the chan
 * according to a given nick
 * @param nick Nick to delete
 * @post lastPart has been updated 
 * @return true if the user ahs been deleted, else false
 */
bool Channel::delUserByNick(string nick)
{
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) {
         for(unsigned int i = 0 ; i < 4 ; i ++ ) {
            this->lastPart[i] = (*it)[i];
         }
			delete [] (*it) ;
         this->users.erase(it);
         return true;		
	}
	else
		return false;
}

/**
 * Del a user from the chan
 * according to a given host
 * @param host host's user to delete
 * @post lastPart has been updated 
 * @return true if the user ahs been deleted, else false
 */
bool Channel::delUserByHost(string host)
{
	vector<string*>::iterator it = this->getIterator(host,1) ;
	if(it != (this->users.end()+1 )) {
         for(unsigned int i = 0 ; i < 4 ; i ++ ) {
            this->lastPart[i] = (*it)[i];
         }
			delete [] (*it) ;
         this->users.erase(it);
         return true;		
	}
	else
		return false;
}

/**
 * Give the nick corresponding to a given host
 * @param host host from witch you want to get the nick
 * @return Nick corresponding to the given host (empty string if no match found)
 */
string Channel::getNickByHost(string host)
{
	vector<string*>::iterator it = this->getIterator(host,1) ;
	if(it != (this->users.end()+1 )) 
		return (*it)[0] ;		
	else
		return "";
}

/**
 * Return informations about a nick, in this order :
 * tab[0]=nick;
 * tab[1]=host;
 * tab[2]=ident;
 * tab[3]=status;
 * @param nick User nick
 * @return array with informations
 */
string* Channel::getInfosByNick(string nick)
{
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) {	
      return (*it);
	}
	else {
      return NULL;
   }
}

/**
 * Give the status corresponding to a given nick
 * @param nick Nick from witch you want to get the status
 * @return Status corresponding to the given nick (empty string if no match found)
 */
string Channel::getStatusByNick(string nick)
{
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) {
		return (*it)[3] ;		
	}
	else
		return "";
}

/**
 * Give the status corresponding to a given host
 * @param host host from witch you want to get the status
 * @return Status corresponding to the given host (empty string if no match found)
 */
string Channel::getStatusByHost(string host)
{
	vector<string*>::iterator it = this->getIterator(host,1) ;
	if(it != (this->users.end()+1 )) {
		return (*it)[3] ;		
	}
	else
		return "";
}

/**
 * Give the host corresponding to a given nick
 * @param nick nick from witch you want to get the host
 * @return Host corresponding to the given nick (empty string if no match found)
 */
string Channel::getHostByNick(string nick)
{
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) {
		return (*it)[1] ;		
	}
	else
		return "";
}

/**
 * Give the ident corresponding to a given nick
 * @param nick nick from witch you want to get the ident
 * @return Ident corresponding to the given nick (empty string if no match found)
 */
string Channel::getIdentByNick(string nick)
{
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) {
		return (*it)[2] ;		
	}
	else
		return "";
}

/**
 * Give the host corresponding to a given host
 * @param host host from witch you want to get the ident
 * @return Ident corresponding to the given host (empty string if no match found)
 */
string Channel::getIdentByHost(string host)
{
	vector<string*>::iterator it = this->getIterator(host,1) ;
	if(it != (this->users.end()+1 )) {
		return (*it)[2] ;		
	}
	else
		return "";
}

/**
 * Change a user nick to an other one
 * This one is found by the nick
 * @param old Nick to change
 * @param newnick New nick (instead of old one)
 * @return True if the nick has been changed, else false (not found)
 */
bool Channel::setNickByNick(string old,string newnick)
{
	vector<string*>::iterator it = this->getIterator(old,0) ;
	if(it != (this->users.end()+1 )) {
		(*it)[0] = newnick;
		return true ;		
	}
	else
		return false;
}

/**
 * Change a user nick to an other one
 * This one is found by the host
 * @param host User host for witch you want to change the nick
 * @param newnick New nick (instead of old one)
 * @return True if the nick has been changed, else false (not found)
 */
bool Channel::setNickByHost(string host,string newnick)
{
	vector<string*>::iterator it = this->getIterator(host,1) ;
	if(it != (this->users.end()+1 )) {
		(*it)[0] = newnick;
		return true ;		
	}
	else
		return false;
}

/**
 * Change a user status to an other one
 * This one is found by the nick
 * @param nick User nick for witch you want to change the status
 * @param sign + or - (+ for grant level, - for remove level)
 * @param mode user mode (v,o etc ...)
 * @return True if the nick has been changed, else false (not found)
 */
bool Channel::updateStatusByNick(string nick,char sign,char mode)
{
   string::size_type pos ;
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) 
	{
		if (sign == '-' )
		{
			pos = ((*it)[3]).find(mode) ;
			if (pos!=string::npos)
			{
				((*it)[3]).erase(pos,1);
			}
		}
		else if (sign == '+')
		{
			pos = ((*it)[3]).find(mode) ;
			if (pos==string::npos)
			{
				((*it)[3])+=mode;
			}
		}
		else {
			return false;
		}
		return true ;		
	}
	else
	{
		return false;
	}
}

/**
 * Check if a given nick owns a given access 
 * @param nick The nick witch you want to check access
 * @param access Access to test (o,v,etc ...)
 * @return true if the nick owns the access, else false
 */
bool Channel::checkNickAccess(string nick,char access)
{
   string::size_type pos ;
	vector<string*>::iterator it = this->getIterator(nick,0) ;
	if(it != (this->users.end()+1 )) 
	{
		pos = ((*it)[3]).find(access) ;
		if(pos == string::npos )
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	else {
		return false;
	}
}

/**
 * Clear users list
 */
void Channel::truncateUsersList() 
{
	vector<string*>::iterator it = this->users.begin();
	while (it != this->users.end()) {
		delete [] (*it) ;
		it++;
	}
	this->users.clear();
}

/**
 * Return users vector informations
 * Users a stored in a string tab like that :
 * tab[0]=nick;
 * tab[1]=host;
 * tab[2]=ident;
 * tab[3]=status;
 * @return users vector informations
 */
vector<string*> Channel::getUsers()
{
	return this->users;
}

/**
 * Get last WHO update timestamp
 * @return last update timestamp
 */
time_t Channel::getLastWhoUpdate()
{
	return this->lastWhoUpdate;
}

/**
 * Notify that a a WHO command has been sent for this channel
 */
void Channel::notifyWho()
{
	time(&this->lastWhoUpdate);
}

/**
 * get topic
 * @return topic
 */
string Channel::getTopic() 
{
	return this->topic;
}

/**
 * set topic
 * @param topic channel topic
 */
void Channel::setTopic(string topic)
{
	this->topic = topic;
}

/**
 * Returns informations about the last user who left the channel
 * @return Informations about the last user who left the channel
 */
string* Channel::getLastPartInfos() 
{
   return this->lastPart ;
}

/**
 * Tell if a nick is on the channel
 * @param nick Nick to check
 * @return True if the nick is on the channel, else false
 */
bool Channel::isOnChannel(string nick) {
   if ( this->getInfosByNick(nick) == NULL )
      return false;
   else
      return true;
}
