/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file configurationfile.cpp
 * @brief ConfigurationFile implementation file
 */

#include "configurationfile.h"
/**
 * The class constructor. Initialize private attributes
 * @param fname Configuration file path (including name)
 * @post An object is contructed
 */
ConfigurationFile::ConfigurationFile(string fname)
{
	this->file = fname ;
	this->config.clear();
   this->protectedKeys.clear();
}

/**
 * The class destructor
 */
ConfigurationFile::~ConfigurationFile()
{

}

/**
 * Read the configuration file and load it in a MAP containe
 * File format :  key=value
 * # is the comment char
 * @pre The file must exist and be readable
 * @post The file is parsed, and closed. The MAP contains keys and values
 * @return true if no errors appears, else false 
 */
bool ConfigurationFile::load()
{
	this->config.clear();
	vector<string> temp ;
	ifstream ifs(this->file.c_str());
   if ( ifs ) {
   	string line;
      while ( getline( ifs, line ) ) {
			if ( (line[0] != '#')&&(line!="") ) {
				if ( line.find("=") == string::npos) {
					return false;
				}
				else {
					temp = Tools::stringToVector(line,"=",0) ;
					if ( temp.size()>1) {
                  if ( this->getValue(temp[0]) == "") {
						   this->config[temp[0]] = temp[1];
                  }
                  else {
                     this->config[temp[0]] += (","+temp[1]);
                  }
					}
               else
               {
                  if ( this->getValue(temp[0]) == "") {
                     this->config[temp[0]] = "";
                  }
               }
				}
			}
      }
		ifs.close();
		return true;
	}
	else {
		return false;
	}
}

/**
 * Flush all the configuration keys (with values) in the configuration file
 * File format :  key=value
 * @pre The file must exist and be writeable
 * @post All the configuration values are saved into the configuration file
 * @return true if the configuration file could be open, else erturn false
 */
bool ConfigurationFile::flush()
{
	ofstream ofs(this->file.c_str(),ios_base::trunc);
	if(ofs) {
		for (map<string,string>::const_iterator iter = this->config.begin(); iter!=this->config.end();++iter)
 		{  
			ofs << iter->first<<"="<<iter->second<<endl;
		}
		ofs.flush();
		ofs.close();
		return true;
	}
	else {
		return false;
	}
}

/**
 * Returns the MAP containing the configuration
 * @pre The file must have been parsed
 * @return a map container, containing the configuration value (with keys)
 */
map<string,string> ConfigurationFile::getConfig()
{
	return this->config;
}

/**
 * Add a key to the protected list
 * @param key Key to protect
 */
void ConfigurationFile::addProtectedKey(string key)
{
   this->protectedKeys.push_back(key);
}

/**
 * Return the value associated to a given key
 * @pre The file must have been parsed
 * @param key The key from with you want to get the value
 * @param displayProtected If true, protected key/values will be displayed
 * @return The value associated to the given key. Empty string if the key doesn't exists
 */
string ConfigurationFile::getValue(string key,bool displayProtected) 
{  
   if(!displayProtected && Tools::isInVector(this->protectedKeys,key)) {
      return "";
   }
	map<string,string>::iterator fter = this->config.find(key);
	if ( fter != this->config.end() ) {
		return fter->second ;
	}
	else {
		return "";
	}
}

/**
 * Set a value to a given key. If the key exists, the value is replaced.
 * If the key does'nt exists, it's added to the map
 * @post The value associated to the given key i saved in the configuration. To save it in the file, use flush() method
 * @param key The key for with you want to set a value
 * @param value The value that you want to give to the given key
 */
void ConfigurationFile::setValue(string key,string value) {
	this->config[key] = value;
}

/**
 * Delete a key from the configuration file
 * @param key The key you want delete
 */
bool ConfigurationFile::delKey(string key) 
{
	map<string,string>::iterator fter = this->config.find(key);
	if ( fter != this->config.end() ) {
		this->config.erase(fter);
      return true;
	}
   else
   {
      return false;
   }
}

/**
 * Get file path
 * @return file path
 */
string ConfigurationFile::getFilePath() {
   return this->file;
}
