/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file cppthread.cpp
 * @brief CPPThread implementation file
 */

#include "cppthread.h"
#include <iostream>
using namespace std;

/**
 * The class constructor. Initialize private attributes
 * @post An object is contructed
 */
CPPThread::CPPThread() {
   this->ti.finished = false;
   this->ti.running = false;
   this->handle = new pthread_t;
}

/**
 * The class destructor
 */
CPPThread::~CPPThread() {
   this->terminate();
   pthread_join (*this->handle, NULL);
   delete this->handle;
}

/**
 * Prepare and launch a thread. 
 * Stores a function to execute, and arguments, 
 * then create a thread, and launch the threadStartup
 * @param myThreadProcess Function to execute
 * @param args Arguments to send to the threaded function
 * @return true if the thread has been launched, else false
 */
bool CPPThread::exec(threadProcess myThreadProcess,void*args) {
   if ( ! this->isRunning() ) {
      this->ti.process = myThreadProcess;
      this->ti.args = args;
      if (pthread_create (this->handle, NULL,CPPThread::threadStartup,&this->ti) < 0) {
         return false;
      }
      return true;
   }
   return false;
}

/**
 * Threaded function. In this function, the thread is prepared,
 * then the function to execute is launched.
 * @param arg thread information structure pointer
 * @post running and finished flags are updated
 */
void* CPPThread::threadStartup(void*arg) {
   threadInfos* ti = (threadInfos*)arg;
   ti->running = true;
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
   ti->process(ti->args);
   ti->running = false;
   ti->finished = true;
   pthread_exit(0);
}

/**
 * Cancel the thread
 * @return true if the thread has been canceled, else false
 */
bool CPPThread::terminate() {
   if ( this->isRunning() ) {
      if (pthread_cancel (*this->handle) != 0) {
         return false;
      }
      ti.running = false;
      return true;
   }
   return false;
}

/**
 * Give running state
 * @return True if the thread is running, else false
 */
bool CPPThread::isRunning() {
   return this->ti.running;
}

/**
 * Tell if the thread is finished.
 * The thread is concidered has finished if the exectuion has finished itself
 * (not canceled)
 * @return True if the thread is finished, else false
 */
bool CPPThread::isFinished() {
   return this->ti.finished;
}

/**
 * Wait for thread end, and then free memory
 */
void* CPPThread::join() {
   void* ret;
   pthread_join(*this->handle,&ret);
   return ret;
}

/**
 * Return pthread handle.
 * Use this handle to use pthread functions 
 * not provided by this class
 * @return Handle's pointer
 */
pthread_t* CPPThread::getHandle() {
   return this->handle;
}
