/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file ircprotocol.cpp
 * @brief IRCProtocol implementation file
 */

#include "ircprotocol.h"

/**
 * Class contructor
 */
IRCProtocol::IRCProtocol()
{
}

/**
 * Class destructor
 */
IRCProtocol::~IRCProtocol()
{
}

/**
 * Format strings to identify to an IRC server according to parameters
 * @param pass IRC server password (empty string if not required)
 * @param ident Connection Ident
 * @param name Connection Name
 * @param nick Connection Nick
 * @return Vector containing strings to send to the server for authentification
 */
vector<string> IRCProtocol::identify(string pass,string ident,string name,string nick) 
{
	vector<string> ret;
	if ( pass != "") 
		ret.push_back("PASS "+pass);
	ret.push_back(IRCProtocol::changeNick(nick));
   ret.push_back("USER " + ident + " 0 * :" + name);
	return ret;
}
	
/**
 * Format a string to quit an IRC server
 * @param reason Quit message. Can be empty
 * @return String containing the message to send to quit the IRC server
 */
string IRCProtocol::quitServer(string reason) 
{
   return("QUIT :"+reason);
}
	
/**
 * Format a string to join a channel
 * @param channel Channel to join
 * @return String containing the message to send to join the channel
 */
string IRCProtocol::joinChannel(string channel) 
{
   return("JOIN " + channel);
}
	
/**
 * Format a string to leave a channel
 * @param channel Channel to leave
 * @param reason Leave msg. Can be empty
 * @return String containing the message to send to leave the channel
 */
string IRCProtocol::leaveChannel(string channel,string reason) 
{
   return("PART " + channel + " :"+reason);
}
	
/**
 * Format a string to change the bot nickname
 * @param nick New nick
 * @return String containing the message to send to change nick
 */
string IRCProtocol::changeNick(string nick) 
{
   return("NICK " + nick);
}
	
/**
 * Format a string to ping the server
 * @param ping String that the server must reply to validate the ping
 * @return String containing the message to send to ping the server
 */
string IRCProtocol::ping(string ping)
{
   return("PING :" + ping);
}
	
/**
 * Format a string to pong the server
 * @param pong string ping answer
 * @return String containing the message to send to pong the server
 */
string IRCProtocol::pong(string pong) 
{
   return("PONG " + pong);
}
	
/**
 * Format a string to send a message to a channel or to a user
 * @param destination Message receiver (channel or user(pv query))
 * @param message Message to send
 * @return String containing the message to send to the server to send a message on a channel or to a user
 */
string IRCProtocol::sendMsg(string destination,string message) 
{
   return("PRIVMSG " + destination + " :" + message);
}

/**
 * Format a strings to send a messages to a channel or to a user
 * @param destination Messages receiver (channel or user(pv query))
 * @param messages Messages to send
 * @return Vector with strings containing the messages to send to the server to send messages on a channel or to a user
 */
vector<string> IRCProtocol::sendMsg(string destination,vector<string> messages) 
{
   vector<string> msgs;
   for ( unsigned int i = 0 ; i < messages.size(); i ++ )
      msgs.push_back("PRIVMSG " + destination + " :" + messages[i]);
   return(msgs);
}

/**
 * Send an action on a channel. 
 * An action is that kind of message : * trustyrc slaps toto 
 * @param channel Channel where to send the action
 * @param action Action to send
 * @return A string containing the formatted message
 */
string IRCProtocol::sendAction(string channel,string action)
{
   return "PRIVMSG "+channel+" :"+"\x01"+"ACTION "+action+"\x01";
}

/**
 * Format a string to change a topic
 * @param channel Channel witch topic will change
 * @param topic New topic
 * @return String containing the message to send to change a channel topic
 */	
string IRCProtocol::changeTopic(string channel,string topic) 
{
   return("TOPIC " + channel + " :" + topic);
}

/**
 * Construct strings to apply a mode on different people
 * @param channel Channel where apply modes
 * @param users_list Users on witch apply modes
 * @param mode Mode ti apply
 * @param sign Mode sign
 * @param limit Limit for size
 * @return Strings for modes
 */
vector<string> IRCProtocol::applyModes(string channel,vector<string> users_list,char sign,char mode,unsigned int limit)
{
   vector<string> ret;
	string query,users;
	unsigned int i = 0 ;
	query = "MODE "+channel+ " "+sign;
	while ( i < users_list.size() ) {
		query += mode;
		users += " "+users_list[i];
		if ( (i+1)%limit == 0 ) {
			query += users;
			ret.push_back(query);
			query = "MODE "+channel+ " "+sign;
			users = "";
		}
		i++;
	}
	if ( users_list.size()%limit != 0 ) {
		query += users;
		ret.push_back(query);
	}
	return ret;
}
	
/**
 * Format strings to op users on a channel
 * Obsolete : use applyModes instead
 * @param vectorNicks Vector containing string containing nicks to op
 * @param channel Channel where nicks must be opped
 * @return Strings containing instructions to op users
 */	
vector<string> IRCProtocol::op(vector<string> vectorNicks,string channel) 
{
	vector<string> ret;
	string query,nicks;
	unsigned int i = 0 ;
	query = "MODE "+channel+ " +";
	while ( i < vectorNicks.size() )
	{
		query += "o";
		nicks += " "+vectorNicks[i];
		if ( (i+1)%4 == 0 )
		{
			query += nicks;
			ret.push_back(query);
			query = "MODE "+channel+ " +";
			nicks = "";
		}
		i++;
	}
	if ( vectorNicks.size()%4 != 0 )
	{
		query += nicks;
		ret.push_back(query);
	}
	return ret;
}

/**
 * Construct a string to op a user on a channel
 * Obsolete : use applyModes instead
 * @param nick Nick to op
 * @param channel Channel where the nick must be opped
 * @return OP string
 */
string IRCProtocol::op(string nick,string channel) 
{
   return ("MODE "+channel+" +o "+nick) ;
}
	
/**
 * Format strings to unop users on a channel
 * Obsolete : use applyModes instead
 * @param vectorNicks Vector containing string containing nicks to unop
 * @param channel Channel where nicks must be unopped
 * @return Strings containing instructions to unop users
 */	
vector<string> IRCProtocol::unop(vector<string> vectorNicks,string channel) 
{
	vector<string> ret;
	string query,nicks;
	unsigned int i = 0 ;
	query = "MODE "+channel+ " -";
	while ( i < vectorNicks.size() )
	{
		query += "o";
		nicks += " "+vectorNicks[i];
		if ( (i+1)%4 == 0 )
		{
			query += nicks;
			ret.push_back(query);
			query = "MODE "+channel+ " -";
			nicks = "";
		}
		i++;
	}
	if ( vectorNicks.size()%4 != 0 )
	{
		query += nicks;
		ret.push_back(query);
	}
	return ret;
}

/**
 * Construct a string to unop a user on a channel
 * @param nick Nick to unop
 * @param channel Channel where the nick must be unopped
 * @return UNOP string
 */
string IRCProtocol::unop(string nick,string channel) 
{
   return ("MODE "+channel+" -o "+nick) ;
}

/**
 * Format a string to ban a mask on a channel
 * @param mask mask to ban
 * @param channel ban's channel
 * @return ban string
 */
string IRCProtocol::ban(string mask,string channel)
{
   return ("MODE "+channel+" +b "+mask);
}	

/** 
 * Format a string to unban a mask on a channel
 * @param mask Mask to unban
 * @param channel Channel where to unban the user
 * @return Unban string
 */
string IRCProtocol::unban(string mask,string channel)
{
   return ("MODE "+channel+" -b "+mask);
}

/**
 * Format strings to voice users on a channel
 * Obsolete : use applyModes instead
 * @param vectorNicks Vector containing string containing nicks to voice
 * @param channel Channel where nicks must be voiced
 * @return Strings containing instructions to voice users
 */	
vector<string> IRCProtocol::voice(vector<string> vectorNicks,string channel) 
{
	vector<string> ret;
	string query,nicks;
	unsigned int i = 0 ;
	query = "MODE "+channel+ " +";
	while ( i < vectorNicks.size() )
	{
		query += "v";
		nicks += " "+vectorNicks[i];
		if ( (i+1)%4 == 0 )
		{
			query += nicks;
			ret.push_back(query);
			query = "MODE "+channel+ " +";
			nicks = "";
		}
		i++;
	}
	if ( vectorNicks.size()%4 != 0 )
	{
		query += nicks;
		ret.push_back(query);
	}
	return ret;
}
	
/**
 * Construct a string to voice a user on a channel
 * @param nick Nick to voice
 * @param channel Channel where the nick must be voiced
 * @return voice string
 */
string IRCProtocol::voice(string nick,string channel) 
{
   return ("MODE "+channel+" +v "+nick) ;
}
	
/**
 * Format strings to unvoice users on a channel
 * Obsolete : use applyModes instead
 * @param vectorNicks Vector containing string containing nicks to unvoice
 * @param channel Channel where nicks must be unvoiced
 * @return Strings containing instructions to unvoice users
 */	
vector<string> IRCProtocol::unvoice(vector<string> vectorNicks,string channel) 
{
	vector<string> ret;
	string query,nicks;
	unsigned int i = 0 ;
	query = "MODE "+channel+ " -";
	while ( i < vectorNicks.size() )
	{
		query += "v";
		nicks += " "+vectorNicks[i];
		if ( (i+1)%4 == 0 )
		{
			query += nicks;
			ret.push_back(query);
			query = "MODE "+channel+ " -";
			nicks = "";
		}
		i++;
	}
	if ( vectorNicks.size()%4 != 0 )
	{
		query += nicks;
		ret.push_back(query);
	}
	return ret;
}

/**
 * Construct a string to unvoice a user on a channel
 * @param nick Nick to unvoice
 * @param channel Channel where the nick must be unvoiced
 * @return voice string
 */
string IRCProtocol::unvoice(string nick,string channel) 
{
   return ("MODE "+channel+" -v "+nick) ;
}
	
/**
 * Format a string to send a notice
 * @param destination Notice receiver (nick or channel)
 * @param notice Notice message
 * @return String containing the message to send the notice
 */	
string IRCProtocol::sendNotice(string destination,string notice) 
{
   return("NOTICE " + destination +" :" + notice );
}

/**
 * Format strings to send notices
 * @param destination Notice receiver (nick or channel)
 * @param notices Notices messages
 * @return Strings containing the messages to send the notices
 */	
vector<string> IRCProtocol::sendNotices(string destination,vector<string> notices) 
{
   vector<string> back;
   for ( unsigned int i = 0 ; i  < notices.size() ; i ++ )
      back.push_back("NOTICE " + destination +" :" + notices[i]);
   return(back);
}

/**
 * Format a string to send a WHO command
 * @param channel Channel for the WHO command
 * @param params Parameters for WHO command
 * @return The WHO command
 */
string IRCProtocol::who(string channel,string params)
{
   if ( params == "" )
      return "WHO "+channel;
   else
      return "WHO "+channel+" %"+params;
}
	
/**
 * Format a string to kick a user from a channel 
 * @param nick User nick to kick
 * @param chan Channel from witch one the user is kicked
 * @param reason Kick reason, can be empty
 * @return String containing the message to kick the user
 */
string IRCProtocol::kick(string nick,string chan,string reason) 
{
   return("KICK " + chan + " " + nick + " :"+reason);
}

/**
 * Format a string to send a INVITE command
 * @param channel Channel where invite the user
 * @param nick Nick to invite
 * @return String containing the invite command
 */
string IRCProtocol::invite(string channel,string nick)
{
   return "INVITE "+channel+" "+nick;
}
