/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file logfile.cpp
 * @brief LogFile implementation file
 */

#include "logfile.h"


/**
 * Constructor. Initialize object members
 * @param name Base filename
 * @param verbose Set to true if you want a verbose logging (displays messages in console)
 * @param keep Set to true if you want to keep a log file when a new one is generated (period change)
 * @param level Log level : NOTHING,ERROR,WARNING or INFO
 * @param format format to finish log file name. Uses strftime function jokers. 
 */
LogFile::LogFile(string name,bool verbose,bool keep,string level,string format) {
   this->setLogLevel(this->strToLogLevel(level));
   this->setVerbose(verbose);
   this->setKeepFiles(keep);
   this->setPeriodFormat(format);
   this->baseFileName = name;
   this->stream = new ofstream();   
}

/**
 * Destructor. Close the file and delete stream
 */
LogFile::~LogFile() {
   this->close();
   delete this->stream;
}

/**
 * Open the log file and initilize it. If the file does not exists, it's created, 
 * according to base filename and period format
 * @return true if log file has been opened, else false
 */
bool LogFile::open() {
   bool openned;
   this->period = this->systemPeriod();
   this->stream->open((this->baseFileName+this->period+".log").c_str(),ios::out|ios::app);
   openned = this->stream->is_open();
   if (openned) {
      this->beginLog();
   }
   return openned;
}

/**
 * Finish the log file and close it.
 */
void LogFile::close() {
   this->endLog();
   this->stream->flush();
   this->stream->close();
}

/**
 * Close the log file, then open it
 */
void LogFile::reopen() {
   this->close();
   this->open();
}

/**
 * Give system period (time) according to given format
 * @return System period
 */
string LogFile::systemPeriod() {
   time_t now ;
   time(&now);
   char timeFormat[50];
   strftime(timeFormat,50,this->getPeriodFormat().c_str(),localtime(&now));
   return timeFormat;
}

/** 
 * Log an event to the log file, and displays it if verbose is set to true.
 * If the log file is not present, it will be created. 
 * If the period has change (you log a file per month, and month has changed for example),
 * log file will be closed and a new one will be created, with a name according to the new period.
 * If keepFiles is set to true, the old file will be kept, else, this function will delete it.
 * @param line Line to log
 * @param ll log level used for this line. Can be NOTUSED (default, log level system is not used, the line will be logged without taking care about level), ERROR,WARNING,INFO
 * @return true if the line has been logged, else false
 */
bool LogFile::log(string line,log_level ll) {
   time_t now ;
   char timeFormat[50];
   string logline,pastperiod;
   /*if ( !this->checkFile() ) {
      this->reopen();
   }*/
   if((this->systemPeriod() != this->period)) {
      pastperiod = this->period ;
      this->reopen();
      if(!this->keepFiles) {
         remove((this->baseFileName+pastperiod+".log").c_str());
      }
   }
   if ( ll != NOTUSED ) {
      switch (this->level) {
         case NOTUSED : ;
         break;
         case NOTHING : return true ;
         break;
         case ERROR : ;
         break;
         case WARNING : if((ll!=ERROR)&&(ll!=WARNING)) return true ;
         break;
         case INFO : if(ll!=INFO) return true ;
         break;
         default : ;
      }
   }
   time(&now); 
   strftime(timeFormat,16,"%b %e %X",localtime(&now)); 
   logline = (string)timeFormat + " " +this->getLevelTag(ll)+ line ;
   if ( this->verbose ){
      cout << logline << endl;
   }
   (*this->stream) << logline << endl;
   return (*this->stream);
}

/**
 * Convert a string log level to a log_level format
 * @param ll log level to convert
 * @return log level converted
 */
log_level LogFile::strToLogLevel(string ll) {
   if(ll=="error")
      return ERROR;
   else if (ll=="warning")
      return WARNING;
   else if (ll=="info")
      return INFO;
   else if (ll=="nothing")
      return NOTHING;
   else 
      return NOTUSED;
}

/**
 * Set period format 
 * @param format period format
 */
void LogFile::setPeriodFormat(string format) {
   this->periodFormat = format;
}

/**
 * Get period format
 * @return Period format
 */
string LogFile::getPeriodFormat() {
   return this->periodFormat;
}

/**
 * Set log level 
 * @param ll log level
 */
void LogFile::setLogLevel(string ll) {
   this->level = this->strToLogLevel(ll) ;
}

/**
 * Set log level 
 * @param ll log level
 */
void LogFile::setLogLevel(log_level ll) {
   this->level = ll ;
}

/**
 * Get log level
 * @return actual log level
 */
log_level LogFile::getLogLevel() {
   return this->level;
}

/**
 * Set keepFiles state
 * @param state State to set
 */
void LogFile::setKeepFiles(bool state) {
   this->keepFiles = state ;
}

/**
 * Get keepFiles state
 * @return keepFiles state
 */
bool LogFile::getKeepFiles() {
   return this->keepFiles;
}

/**
 * Set verbose state
 * @param state Verbose state
 */
void LogFile::setVerbose(bool state) {
   this->verbose = state;
}

/** 
 * Get verbose state
 * @return verbose state
 */
bool LogFile::getVerbose() {
   return this->verbose;
}

/**
 * Check if the log file exists
 * @return True if file exists, else false
 */
bool LogFile::checkFile() {
   struct stat stats;
   if (stat((this->baseFileName+this->period+".log").c_str(),&stats)==0) 
      return true;
   else
      return false;
}

/**
 * Initialize log file
 */
void LogFile::beginLog() {
   time_t now ;
   time(&now);
   char timeFormat[24];
   strftime(timeFormat,25,"%a %b %e %X %Y",localtime(&now)); 
   (*this->stream) << "**** BEGIN LOGGING AT "<< timeFormat << endl << endl ;
}

/**
 * Finish log file
 */
void LogFile::endLog() {
   time_t now ;
   time(&now);
   char timeFormat[24];
   strftime(timeFormat,25,"%a %b %e %X %Y",localtime(&now)); 
   (*this->stream) << "**** ENDING LOGGING AT " << timeFormat << endl << endl ;
}

/**
 * Get a tag according to a given log level
 * @param ll Log level given
 * @return Level taf according to the log level
 */
string LogFile::getLevelTag(log_level ll) {
   switch(ll) {
      case NOTUSED : return "";
      break;
      case NOTHING : return "";
      break;
      case ERROR : return "{E} ";
      break;
      case WARNING : return "{W} ";
      break;
      case INFO : return "{I} " ;
      break;
      default : return "";
   }
}

