/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file message.cpp
 * @brief Message implementation file
 */

#include "message.h"


/**
 * Class constructor
 * Split different parts of the messaeg to make it easier to use
 * @param message IRC message
 */
Message::Message(string message)
{
   this->setMessage(message);
}

/**
 * Constructor
 * Used for unknow string that are set after whith setMessage()
 */
Message::Message()
{
   this->message= "";
   this->split.clear();
}

/**
 * Class destructor
 */
Message::~Message()
{

}

/**
 * Set the message string and split it
 * @param message IRC message
 */
void Message::setMessage(string message)
{
   this->message = message;
   this->split = Tools::stringToVector(message," ",0);
   string source = this->getSource() ;
   if ( source == "" )
   {
      this->pv = true;
   }
   else
   {
	   if ( source[0] != '#' )
		   this->pv = true;
	   else
		   this->pv = false;
   }
   time(&this->timestamp);
}

/**
 * Return parts number's message
 * @return parts number
 */
unsigned int Message::nbParts() {
   return this->split.size() ;
}

/**
 * Return different parts of the message
 * @return A vector containing string representing parts of the message
 */
vector<string> Message::getSplit()
{
	return this->split;
}

/**
 * Give the message sender (nick host ident)
 * @return message sender
 */
string Message::getSender()
{
   if(this->getPart(0).length() > 0 ) {
   	return this->split[0].substr(1);
   }
   else {
      return "";
   }
}

/**
 * Get message sender's nick
 * @return message sender's nick
 */
string Message::getNickSender()
{
   if(this->getPart(0).length() > 0 ) {
   	return(this->message.substr(1,this->message.find("!")-1));
   }
   else {
      return "";
   }
}

/**
 * Get message sender's host
 * @return message sender's host
 */
string Message::getHostSender()
{
   if(this->getPart(0).length() > 0 ) {
   	return(this->split[0].substr(this->message.find("@")+1));
   }
   else {
      return "";
   }
}

/**
 * Get message sender's ident
 * @return message sender's ident
 */
string Message::getIdentSender()
{
   if(this->getPart(0).length() > 0 ) {
   	return(this->split[0].substr(this->message.find("!")+1,this->split[0].find("@")-this->message.find("!")-1));
   }
   else {
      return "";
   }
}

/**
 * Tell if the message is private to the bot
 * @return True if it's a private message, else false
 */
bool Message::isPrivate()
{
	return this->pv;
}

/**
 * Tell if the message is public (on a channel)
 * @return True if it's a public message, else false
 */
bool Message::isPublic()
{
	return !this->isPrivate();
}

/**
 * Return a part of the message.
 * This part is the one pointed by the given index
 * @param index Index of the wanted message part (start to zero)
 * @return Part of the message. empty if index is superior to the parts message number
 */
string Message::getPart(unsigned int index)
{
	if ( this->split.size() > index)
	{
		return this->split[index];
	}
	else
	{
		return "";
	}
}

/**
 * Get the message source (channel or bot nick if private)
 * @return Message source
 */
string Message::getSource()
{
   return this->getPart(2);
}

/**
 * Get the raw message
 * @return The raw message
 */
string Message::getMessage()
{
	return this->message;
}

/**
 * Get elapsed time between message creation and now (in seconds)
 * @return Elapsed time between message creation and now
 */
time_t Message::getElapsedTime() 
{
   time_t now;
   time(&now);
   return (now - this->timestamp);
}
