/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file plugin.cpp
 * @brief Plugin implementation file
 */

#include "plugin.h"

/**
 * Class constructor
 * Initialize private attributes
 */
Plugin::Plugin()
{
	this->author = "";
	this->description = "";
	this->version = "";
	this->name = "";
   this->funcs.clear();
   this->requirements.clear();
   this->handle = NULL;
}

/**
 * Class destructor
 */
Plugin::~Plugin(){

}

/**
 * Get plugin author
 * @return plugin author
 */
string Plugin::getAuthor()
{
	return this->author;
}

/**
 * Get plugin description
 * @return plugin description
 */
string Plugin::getDescription()
{
	return this->description;
}

/**
 * Get plugin version
 * @return plugin version
 */
string Plugin::getVersion()
{
	return this->version;
}

/**
 * Get plugin name
 * @return plugin name
 */
string Plugin::getName()
{
	return this->name;
}

/**
 * Return shared functions
 * @return Shared functions
 */
vector<StructFunctionStorage> Plugin::getFunctions() 
{
   return this->funcs;
}

/**
 * Check if all attributes have benn completed
 * @return True if ok, else false
 */
bool Plugin::checkMembers()
{
	if ( this->getAuthor() == "" )
		return false;
	if ( this->getDescription() == "" )
		return false;
	if ( this->getVersion() == "" )
		return false;
	if ( this->getName() == "" )
		return false;
   if ( this->funcs.size() == 0 )
      return false;
	return true;
}

/**
 * Bind a plugin function. Called in plugin constructor, it initialise the function registration (before kernel).
 * Used after plugin constructor has no effect (use kernel -> registerFunction instead)
 * @param highlightedWord Highlighted word (command), or time between two executions in seconds ( for IN_LOOP plugins )
 * @param type Function type
 * @param symbole function name to execute
 * @param lastExec last time plugin was executed
 * @param timeout Timeout for function (in seconds)
 */
void Plugin::bindFunction(string highlightedWord,func_type type,string symbole,time_t lastExec,unsigned int timeout)
{
   StructFunctionStorage store;
   store.handle = NULL;
   store.highlightedWord = highlightedWord;
   store.object = this;
   store.type = type;
   store.symbole = symbole;
   store.function = NULL;
   store.lastExec = lastExec;
   store.timeout = timeout;
   this->funcs.push_back(store);
}

/**
 * Get plugin's handle
 * @return Plugin's handle
 */
void * Plugin::getHandle()
{
   return this->handle;
}

/**
 * Set plugin's handle
 * @param handle Plugin's handle
 */
void Plugin::setHandle(void* handle)
{
   this->handle = handle;
}

/**
 * Add a requirement for this plugin. It means that to load this plugin
 * the requirement will have to be loaded before.
 * @param plugin Plugin that is required
 */
void Plugin::addRequirement(string plugin) 
{
   this->requirements.push_back(plugin);
}

/**
 * Get requirements list.
 * @return Requirements list (vector)
 */
vector<string> Plugin::getRequirements() 
{
   return this->requirements;
}

/**
 * Tell if this plugin requires an other one.
 * @param plugin Plugin to test
 * @return true is this plugin requires the given one, else false
 */
bool Plugin::requires(string plugin)
{
   return Tools::isInVector(this->requirements,plugin);
}
