/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file plugin.h
 * @brief Plugin header file
 */

#ifndef PLUGIN_H
#define PLUGIN_H

#include "message.h"
#include <time.h>
#include <vector>
#include <string>
using namespace std;
class Plugin;
class BotKernel;

/// Plugin types
typedef enum {
	IN_LOOP,IN_COMMAND_HANDLER,IN_FREE_COMMAND_HANDLER,IN_TYPE_HANDLER,IN_BEFORE_TREATMENT,IN_ALL_MSGS,IN_FIRST_WORD,COUNTDOWN,OUT_ALL_MSGS
} func_type;

/// Plugin function prototype
typedef bool (*plugin_function)(Message*,Plugin *,BotKernel*);
/// Plugin object constructor prototype
typedef Plugin *(*plugin_constructor)(BotKernel*);
/// Plugin objet destructor prototype
typedef void (*plugin_destructor)(Plugin*);

/// Plugin object and header storage
typedef struct 
{
	string name;
	void * handle;
	Plugin* object;
	plugin_constructor creator;
	plugin_destructor destructor;
} pPlugin;

/// Plugin function storage
typedef struct 
{
	void * handle;
	string highlightedWord;
	Plugin* object;
	func_type type;
	string symbole;
	plugin_function function ;
	time_t lastExec;
	unsigned int timeout;
   bool back;
}StructFunctionStorage;

/** @brief Class that manage a plugin 
 *
 * This class is a plugin head witch all the plugin class 
 * must derive. It contains function to get functions from the plugin,
 * and check if all informations are given
 */

class Plugin {
protected:
	/// Plugin author
	string author;
	/// Plugin descritpion
	string description;
	/// Plugin version
	string version;
	/// Plugin name
	string name;
   /// PLugins functions
   vector<StructFunctionStorage> funcs;
   /// Stores plugin's handle
   void * handle;
   /// stores plugins names that are required for the plugin
   vector<string> requirements;
public:
	/// Constructor
   Plugin();
	/// Destructor
   virtual ~Plugin();
	/// Return plugin functions
	vector<StructFunctionStorage> getFunctions();
	/// Return author
	string getAuthor();
	/// Return description
	string getDescription();
	/// Return version
	string getVersion();
	/// Return name
	string getName();
	/// Check if all members are given
	bool checkMembers();
   /// Register a plugin function
   void bindFunction(string,func_type,string,time_t,unsigned int);
   /// Get plugin's handle
   void * getHandle();
   /// Set plugin's handle
   void setHandle(void*);
   /// Add a plugin that is required for the plugin
   void addRequirement(string);
   /// Get requirements list
   vector<string> getRequirements();
   /// Tell if this plugin requires an other one
   bool requires(string);
};
#endif
