/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file advertising.cpp
 * @brief Advertising implementation file
 */

#include "advertising.h"

extern "C" bool displayAdvertise(Message*,Plugin*,BotKernel*);

/**
 * Constructor
 */ 
Advertising::Advertising(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Display ads";
	this->version = VERSION;
	this->name = "advertising";
   this->bindFunction("addad",IN_COMMAND_HANDLER,"addad",0,10);
   this->bindFunction("delad",IN_COMMAND_HANDLER,"delad",0,10);
   this->bindFunction("adinfos",IN_COMMAND_HANDLER,"adinfos",0,10);
   this->bindFunction("listads",IN_COMMAND_HANDLER,"listads",0,10);
	this->bindFunction("60",IN_LOOP,"cleanList",0,10); 
   this->addRequirement("admin");

   this->doc = new TiXmlDocument(b->getDatasDir()+"advertising.xml");
   if ( this->doc->LoadFile() )
   {
      this->root = this->doc->FirstChild();
   }
   else
   {
      this->initFile();
   }
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_advertising").Element();
   for (elem=elem->FirstChildElement() ; elem!=0;elem=elem->NextSiblingElement() ) {
      this->launchAdvertise(b,elem->ValueStr().substr(2),Tools::strToInt(elem->Attribute("frequency")));
   }
}

/**
 * Initilaize the XML file by creating root and first childs (file empty structure)
 */
void Advertising::initFile()
{
	TiXmlElement born("trustyrc_advertising");
   this->doc->InsertEndChild(born);
   this->root = this->doc->FirstChild();
   this->doc->SaveFile();
}

/**
 * Launch an add
 * @param b Kernel pointer
 * @param id Ad's id
 * @param freq Ad's display frequency
 */
void Advertising::launchAdvertise(BotKernel*b,string id, unsigned int freq)
{
   Message m(id);
   b->addCountDown(this,displayAdvertise,&m,freq,10);
}

/**
 * Add an advertising to a channel, and save it int the XML file
 * @param channel Ad's channel
 * @param frequency Ad's frequency (in seconds)
 * @param until Number (in seconds) after witch the ad is outdated
 * @param by User who add the ad
 * @param text Ad's text
 * @return ad's timestamp used to access to it 
 */
time_t Advertising::addAdvertise(string channel,unsigned int frequency,unsigned int until,string by,string text)
{
   time_t now,back;
   time(&now);
   back = now;
   if (!this->adExists(Tools::intToStr(now)) )
   {
	   char date[17];
      TiXmlElement item("ad"+Tools::intToStr(now));
      item.SetAttribute("channel",channel);
      item.SetAttribute("frequency",frequency);
      item.SetAttribute("until",until);
	   strftime(date,18,"%y-%m-%d %X",localtime(&now));
	   item.SetAttribute("date",date);
      item.SetAttribute("by",by);
      TiXmlText textItem(text);
      item.InsertEndChild(textItem);
      this->root->InsertEndChild(item);
      this->doc->SaveFile();
      return back;
   }
   else {
      return 0;
   }
}

/**
 * Delete an advertise from the XML file
 * @param id Ad's id
 * @return true is ad deleted, else false
 */
bool Advertising::delAdvertise(string id)
{
   bool result;
   TiXmlNode * parent ;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild().FirstChild("ad"+id).Element() ;
   if (elem) {
      parent = elem->Parent();
      result = parent->RemoveChild(elem);
      this->doc->SaveFile();
      return result;
   }
   return false;
}

/**
 * Tell if an ad exists
 * @param id Ad's id
 * @return true if exists, else false
 */
bool Advertising::adExists(string id)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild().FirstChild("ad"+id).Element() ;
   if (elem) {
      return true;
   }
   return false;
}

/**
 * Get advertise infos.
 * Infos are :
 * - channel
 * - frequency
 * - until 
 * - date
 * - sender
 * - text
 * @param id Ad's id
 */
vector<string> Advertising::getAdvertiseInfos(string id)
{
   vector<string> infos;
   infos.clear();
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild().FirstChild("ad"+id).Element() ;
   if (elem) {
      infos.push_back(elem->Attribute("channel"));
      infos.push_back(elem->Attribute("frequency"));
      infos.push_back(elem->Attribute("until"));
      infos.push_back(elem->Attribute("date"));
      infos.push_back(elem->Attribute("by"));
      infos.push_back(elem->GetText());
   }
   return infos;
}

/**
 * Get advertises list 
 * @return A vector containing id's and texts
 */
vector<string> Advertising::getAdvertisesList() 
{
   char date[17];
   time_t futur;
   vector<string> list;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_advertising").Element();
   for (elem=elem->FirstChildElement() ; elem!=0;elem=elem->NextSiblingElement() ) {
      futur = Tools::strToInt(elem->ValueStr().substr(2))+Tools::strToInt(elem->Attribute("until")) ;
      strftime(date,18,"%y-%m-%d %X",localtime(&futur));
      list.push_back(elem->ValueStr().substr(2)+" : "+elem->GetText()+ " ("+(string)date+")");
   }
   return list;
}

/**
 * Delete outdated ads 
 */
void Advertising::deleteOutdatedAds() 
{
   time_t now;
   time(&now);
   TiXmlNode * parent ;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_advertising").Element();
   for (elem=elem->FirstChildElement() ; elem!=0;elem=elem->NextSiblingElement() ) {
      if ( (Tools::strToInt(elem->ValueStr().substr(2)) + Tools::strToInt(elem->Attribute("until"))) <= now ) {
         parent = elem->Parent();
         parent->RemoveChild(elem);
      }   
   }
   this->doc->SaveFile();
}

extern "C"
{
	Plugin *contruct_advertising(BotKernel*b)
	{
		return new Advertising(b);
	}
	void destroy_advertising(Plugin*p)
	{
		delete p;
	}
   bool displayAdvertise(Message*m,Plugin*p,BotKernel*b)
   {
      Advertising*pAd = (Advertising*) p;
      vector<string> infos = pAd->getAdvertiseInfos(m->getMessage());
      if ( infos.size() > 0 ) {
         b->send(IRCProtocol::sendMsg(infos[0],infos[5]));
         return false;
      }
      return true;
   }
	bool addad (Message*m,Plugin*p,BotKernel*b)
	{  
      unsigned int timestamp;
      Message msg;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Advertising*pAd = (Advertising*) p;
      if ((ppAdm!=NULL) && m->isPrivate() && (m->nbParts()>=8) ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            timestamp = pAd->addAdvertise(m->getPart(4),Tools::strtimeToSeconds(m->getPart(5)),Tools::strtimeToSeconds(m->getPart(6)),m->getSender(),Tools::vectorToString(m->getSplit()," ",7)) ;
            if (timestamp> 0) { 
               msg.setMessage(Tools::intToStr(timestamp)); 
               if(b->addCountDown(p,displayAdvertise,&msg,Tools::strtimeToSeconds(m->getPart(5)),10)==NULL) {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"ERROR : the ad is registred but couldn't be launched")) ;
               }
               else {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"Advertise added and launched"));
               }
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Error, try again in a few seconds"));
            }
         }
      }
		return true;
	}
	bool delad (Message*m,Plugin*p,BotKernel*b)
	{
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Advertising*pAd = (Advertising*) p;
      if ((ppAdm!=NULL) && m->isPrivate() && (m->nbParts()==5) ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            if ( pAd->delAdvertise(m->getPart(4)) ) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Advertise deleted"));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"ERROR : no advertise deleted"));
            }
         }
      }
		return true;
	}
	bool adinfos (Message*m,Plugin*p,BotKernel*b)
	{
	   char date[17];
      time_t futur;
      vector<string> infos;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Advertising*pAd = (Advertising*) p;
      if ((ppAdm!=NULL) && m->isPrivate() && (m->nbParts()==5) ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            infos = pAd->getAdvertiseInfos(m->getPart(4));
            if ( infos.size() > 0 ) {
               futur = Tools::strToInt(m->getPart(4))+Tools::strToInt(infos[2]) ;
               strftime(date,18,"%y-%m-%d %X",localtime(&futur));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"channel: "+infos[0]+" frequency : "+infos[1]+" from "+infos[4]+" on "+infos[3]+" to "+(string) date));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"text : "+infos[5]));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"ERROR : advertise not found"));
            }
         }
      }
		return true;
	}
	bool listads (Message*m,Plugin*p,BotKernel*b)
	{
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Advertising*pAd = (Advertising*) p;
      if ((ppAdm!=NULL) && m->isPrivate() ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            b->send(IRCProtocol::sendNotices(m->getNickSender(),pAd->getAdvertisesList()));
         }
      }
		return true;
	}
   bool cleanList(Message*m,Plugin*p,BotKernel*b)
   {
      Advertising*pAd = (Advertising*) p;
      pAd->deleteOutdatedAds();
      return true;
   }
}
