/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file bzrh.cpp
 * @brief BZRH implementation file
 */

#include "bzrh.h"

/**
 * Constructor
 */ 
BZRH::BZRH(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugin that allows queries to bugzilla.redhat.com";
	this->version = VERSION;
	this->name = "bzrh";
   this->bindFunction("bzsearch",IN_COMMAND_HANDLER,"bzsearch",0,120);
   this->bindFunction("bug",IN_COMMAND_HANDLER,"bug",0,60);
   this->bindFunction("",IN_ALL_MSGS,"checkBug",0,60);
}

/**
 * Search for bugs on bugzilla
 * @param pattern Search pattern
 * @param max max results
 * @return vector containing results
 */
vector<string> BZRH::searchBugs(string pattern,string max) 
{
   vector<string>results ;
   string bug_search = Tools::urlencode(pattern);
   const string URL_BZRH = "https://bugzilla.redhat.com/";
   const string BEGIN_BUGS = "<tr class=\"bz_bugitem";
   const string END_BUGS = "</table>";
   const string BEGIN_URL = "href=\"";
   const string BEGIN_DESC = "<td ";
   string URL = URL_BZRH+"buglist.cgi?quicksearch="+bug_search;
   CURL *curl;  
   CURLcode result;
   string::size_type pos;
   string tmp;
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init();  
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, BZRH::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);  
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
      if (result == CURLE_OK) { 
         pos = buffer.find(BEGIN_BUGS);
         if ( pos == string::npos ) {
            results.clear();
            results.push_back("Zarro Boogs found."); 
            return results;
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(END_BUGS);
         if ( pos == string::npos ) {
            results.clear();
            results.push_back("* Parse error (1) *"); 
            return results;
         }
         buffer = buffer.substr(0,pos);
         while(pos != string::npos ) {
            pos = buffer.find(BEGIN_URL);
            if ( pos == string::npos ) {
               results.clear();
               results.push_back("* Parse error (2) *"); 
               return results;
            }
            buffer = buffer.substr(pos+BEGIN_URL.length());
            tmp=URL_BZRH+buffer.substr(0,buffer.find("\">"));   
            for(unsigned int i = 0 ; i < 7 ; i ++ ) {
               pos = buffer.find(BEGIN_DESC);
               if ( pos == string::npos ) {
                  results.clear();
                  results.push_back("* Parse error (3) *"); 
                  return results;
               }
               buffer=buffer.substr(pos+BEGIN_DESC.length());
            }
            results.push_back(tmp+" : "+Tools::cleanHTML(buffer.substr(1,buffer.find("\n")-1)));
            pos= buffer.find(BEGIN_URL);
         }
         if ( (max=="") || (max=="0") || (Tools::strToUnsignedInt(max)>=results.size()) ) {
            return results;
         }
         else {
            tmp = "Too many bugs ("+Tools::intToStr(results.size())+") ! Max is "+max+". Use bz's webiste : "+URL ;
            results.clear();
            results.push_back(tmp); 
            return results;
         }
      }  
      else {   
         results.clear();
         results.push_back("* ERROR, try again later please *");
         return results;
      }  
   }
   results.clear();
   results.push_back("curl init error");
   return results;
}

/**
 * Retrieve informations about a bug
 * @param bug Bug number
 * @param displayNotFound Tell if "bug not found must be displayed
 * @return Bug's infos
 */
string BZRH::getBugInfos(string bug,bool displayNotFound) 
{
   string bug_number;
   if (bug[0] == '#' ) {
      bug_number = bug.substr(1);
   }
   else {
      bug_number =bug;
   }
   const string URL = "https://bugzilla.redhat.com/show_bug.cgi?format=multiple&id="+Tools::urlencode(bug_number);
   const string SHOW_BUG_URL = "http://bugzilla.redhat.com/show_bug.cgi?id=";
   const string SUMMARY = "<th>Summary:</th>";
   const string BEGIN_SUMMARY = "<td colspan=3>";
   const string ASSIGNEE = "Assignee:</th>";
   const string BEGIN_ASSIGNEE = "<td>";
   const string STATUS = "<th>Status:</th>";
   const string BEGIN_STATUS = "<td>";
   const string SEVERITY = "<th>Severity:</th>";
   const string BEGIN_SEVERITY = ">";
   const string PRIORITY = "<th>Priority:</th>";
   const string BEGIN_PRIORITY = ">";
   string summary,assignee,status,severity,priority;
   string::size_type pos;
   CURL *curl;  
   CURLcode result;  
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init(); 
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,BZRH::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);  
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
      if (result == CURLE_OK) {  
         pos = buffer.find(SUMMARY);
         if ( pos == string::npos ) {
            if (displayNotFound)
               return "Bug not found";
            else 
               return "";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_SUMMARY) +BEGIN_SUMMARY.length();
         summary = buffer.substr(pos, buffer.find("</td>") - pos );
         pos = buffer.find(ASSIGNEE);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_ASSIGNEE) +BEGIN_ASSIGNEE.length(); 
         assignee = buffer.substr(pos, buffer.find("</td>") - pos );
         pos = buffer.find(STATUS);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos);
         pos = buffer.find(BEGIN_STATUS) + BEGIN_STATUS.length();
         buffer = buffer.substr(pos);
         status = buffer.substr(0, buffer.find("\n"));
         pos = buffer.find(SEVERITY);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos+SEVERITY.length());
         pos = buffer.find(BEGIN_SEVERITY) + BEGIN_SEVERITY.length();
         buffer = buffer.substr(pos);
         severity = buffer.substr(0, buffer.find("\n"));
         pos = buffer.find(PRIORITY);
         if ( pos == string::npos ) {
            return "parse error";
         }
         buffer = buffer.substr(pos+PRIORITY.length());
         pos = buffer.find(BEGIN_PRIORITY) + BEGIN_PRIORITY.length();
         buffer = buffer.substr(pos);
         priority = buffer.substr(0, buffer.find("\n"));
         return "bug "+SHOW_BUG_URL+bug_number+" : "+severity+", "+priority+", "+status+", "+Tools::cleanHTML(assignee)+", "+Tools::cleanHTML(summary);
      }  
      else {   
         return "* ERROR, try again later please *"; 
      }  
   }
   return "curl init error"; 
}

int BZRH::writer(char *data, size_t size, size_t nmemb,string *buffer)  
{  
   int result = 0;  
   if (buffer != NULL) {  
      buffer->append(data, size * nmemb);  
      result = size * nmemb;  
   }   
   return result;  
}

extern "C"
{
	Plugin *contruct_bzrh(BotKernel*b)
	{
		return new BZRH(b);
	}
	void destroy_bzrh(Plugin*p)
	{
		delete p;
	}
	bool bzsearch (Message*m,Plugin*p,BotKernel*b)
	{
      BZRH*bz= (BZRH*)p;
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),bz->searchBugs(Tools::vectorToString(m->getSplit()," ",4), b->getCONFF()->getValue(p->getName()+".maxbugs"))));
      }
		return true;
	}
	bool bug (Message*m,Plugin*p,BotKernel*b)
	{
      BZRH*bz= (BZRH*)p;
      if ( m->isPublic() && (m->nbParts() == 5) && (m->getPart(4)!="") ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(4),true)));
      }
		return true;
	}
   bool checkBug(Message*m,Plugin*p,BotKernel*b)
   {
      const string BUG = "bug";
      const string URL1 = "https://bugzilla.redhat.com/show_bug.cgi?id=";
      const string URL2 = "http://bugzilla.redhat.com/show_bug.cgi?id=";
      BZRH*bz= (BZRH*)p;
      if (m->isPublic() && (b->getCONFF()->getValue(p->getName()+".autoreact")=="1")) {
         if ( m->getPart(3).find(":"+BUG) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(4),false)));
         }
         else if (m->getPart(3).find(":"+URL1) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(3).substr(URL1.length()+1),false)));
         }
         else if (m->getPart(3).find(":"+URL2) == 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(3).substr(URL2.length()+1),false)));
         }
         for (unsigned int i = 4 ; i < m->nbParts() ; i ++ ) {
            if ( m->getPart(i).find(BUG) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i+1),false)));
            }
            else if (m->getPart(i).find(URL1) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i).substr(URL1.length()),false)));
            }
            else if (m->getPart(i).find(URL2) == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),bz->getBugInfos(m->getPart(i).substr(URL2.length()),false)));
            }
         }  
      }
      return true;   
   }
}
