/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file fedoraproject.cpp
 * @brief FedoraProject implementation file
 */

#include "fedoraproject.h"

/**
 * Constructor
 */ 
FedoraProject::FedoraProject(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugin in connection with fedora project";
	this->version = VERSION;
	this->name = "fedoraproject";
   this->bindFunction("whoowns",IN_COMMAND_HANDLER,"whoowns",0,60);
   this->bindFunction("fas",IN_COMMAND_HANDLER,"fas",0,60);
	this->bindFunction("reloadfas",IN_COMMAND_HANDLER,"reloadfas",0,60);
   this->addRequirement("admin");

	if (!this->loadFasFile(b->getDatasDir()+"fas.txt")) {
		b->getSysLog()->log("Unable to load fas file",WARNING);
	}
}

/**
 * Tells who owns a package using https://admin.fedoraproject.org/pkgdb/packages/name/ webpage
 * @param name Package name
 * @return Package's owner
 */
string FedoraProject::whoowns(string name) 
{
   const string URL = "https://admin.fedoraproject.org/pkgdb/packages/name/"+Tools::urlencode(name);
   const string LAST_OWNER = "<span class=\"ownerName\"><a href=\"";
   const string END_OWNER = "</a>";
   CURL *curl;  
   CURLcode result;  
   string::size_type pos;
   char errorBuffer[CURL_ERROR_SIZE];  
   string buffer;  
   curl = curl_easy_init();  
   if (curl) {  
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorBuffer);  
      curl_easy_setopt(curl, CURLOPT_URL, URL.c_str());  
      curl_easy_setopt(curl, CURLOPT_HEADER, 0);  
      curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);  
      curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, FedoraProject::writer);  
      curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer); 
      result = curl_easy_perform(curl);  
      curl_easy_cleanup(curl);  
      if (result == CURLE_OK) {  
         pos = buffer.rfind(LAST_OWNER);
         if ( pos == string::npos ){
            return (name+" : package not found");
         }
         else {
            buffer = buffer.substr(pos+LAST_OWNER.length());
            buffer = buffer.substr(buffer.find(">")+1);
            return (name+"'s owner : "+buffer.substr(0,buffer.find(END_OWNER)));
         }
      }  
      else {  
         return (name+" : package not found");
      }  
   } 
   return "curl init error";
}

/**
 * Load FAS file that contains FAS users informations
 * @return true is load is OK, else false
 */
bool FedoraProject::loadFasFile(string file) {
	this->usersInfos.clear();
	vector<string> temp,tempinfos;
	ifstream ifs(file.c_str());
   if ( ifs ) {
   	string line;
      while ( getline( ifs, line ) ) {
			temp = Tools::stringToVector(line,",",0) ;
			tempinfos.clear();
			tempinfos.push_back(temp[1]);
			tempinfos.push_back(temp[2]);
			tempinfos.push_back(temp[3]);
			this->usersInfos[temp[0]] = tempinfos;
      }
		ifs.close();
		return true;
	}
	else {
		return false;
	}
}

/**
 * Get FAS user infos
 * Infos are (see https://admin.fedoraproject.org/accounts/group/dump/cla_done) :
 * - mail
 * - real name
 * - status
 * @return FAS user infos
 */
vector<string> FedoraProject::getFasUserInfos(string nick) {
	vector<string> notFound;
	notFound.push_back("user not found");
	map<string,vector<string> >::iterator fter = this->usersInfos.find(nick);
	if ( fter != this->usersInfos.end() ) {
		return fter->second ;
	}
	else {
		return notFound;
	}
}

int FedoraProject::writer(char *data, size_t size, size_t nmemb,string *buffer)  
{  
   int result = 0;  
   if (buffer != NULL) {  
      buffer->append(data, size * nmemb);  
      result = size * nmemb;  
   }   
   return result;  
}

extern "C"
{
	Plugin *contruct_fedoraproject(BotKernel*b)
	{
		return new FedoraProject(b);
	}
	void destroy_fedoraproject(Plugin*p)
	{
		delete p;
	}
	bool whoowns (Message*m,Plugin*p,BotKernel*b)
	{
      FedoraProject * fp = (FedoraProject*)p;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),fp->whoowns(m->getPart(4))));
      }
		return true;
	}
	bool fas (Message*m,Plugin*p,BotKernel*b)
	{
      FedoraProject * fp = (FedoraProject*)p;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),Tools::vectorToString(fp->getFasUserInfos(m->getPart(4)),",",0)));
      }
		return true;
	}
	bool reloadfas(Message*m,Plugin*p,BotKernel*b) 
	{
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
		FedoraProject * fp = (FedoraProject*)p;
      if (ppAdm!=NULL) {
			adm = (Admin*)ppAdm->object ;
      	if ( adm->isSuperAdmin( m->getSender()) && m->isPrivate()) {
				if (fp->loadFasFile(b->getDatasDir()+"fas.txt")) {
					b->getSysLog()->log("FAS file reloaded by "+m->getSender(),INFO);
					b->send(IRCProtocol::sendNotice(m->getNickSender(),"FAS file reloaded"));
				}
				else {
					b->getSysLog()->log("Unable to load fas file (by "+m->getSender()+")",WARNING);
					b->send(IRCProtocol::sendNotice(m->getNickSender(),"Unable to load fas file"));
				}
      	}
		}
		return true;
	}
}
