/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file gameserver.cpp
 * @brief GameServer implementation file
 */

#include "gameserver.h"

/**
 * Constructor
 */ 
GameServer::GameServer(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Provides tools to query game servers";
	this->version = VERSION;
	this->name = "gameserver";
	this->bindFunction("q3",IN_COMMAND_HANDLER,"q3",0,10);
	this->bindFunction("warsow",IN_COMMAND_HANDLER,"warsow",0,10);
	this->bindFunction("wsw",IN_COMMAND_HANDLER,"warsow",0,10);
	this->bindFunction("hl",IN_COMMAND_HANDLER,"hl",0,10);
	this->bindFunction("cs",IN_COMMAND_HANDLER,"hl",0,10);
}

/**
 * Convert a string to a long (seems fucked)
 * /!\ DOESN'T WORK /!\
 * @param str String to convert
 * @return Number corresponding to the string
 */
long GameServer::strToLong(string str) 
{
   char buffer[4];
   long result;
   for(unsigned int i = 0 ; i < 4 ; i ++ )
   {
      buffer[i] = str[i] ;
   }
   result= strtoll(buffer,NULL,10);
   return result;
}

/**
 * Get a string in HL1 protocol
 * Read datas until a 0x00 char is found
 * @param index Index to read datas
 * @param datas Datas to read
 * @post The index is updated
 * @return The string
 */
string GameServer::getHLstring(unsigned int * index,char*datas) {
   string buffer = "";
   while(datas[*index] != '\x00')
   {
      buffer += datas[(*index)];
      (*index)++;
   }
   (*index)++;
   return buffer;
}

/**
 * Get a byte in HL1 protocol
 * Read only one byte
 * @param index Index to read datas
 * @param datas Datas to read
 * @post The index is updated
 * @return The byte
 */
char GameServer::getHLbyte(unsigned int*index,char*datas) {
   (*index)++;
   return datas[(*index)-1];
}

/**
 * Get a long in HL1 protocol
 * @param index Index to read datas
 * @param datas Datas to read
 * @post The index is updated
 * @return The long data
 */
string GameServer::getHLlong(unsigned int*index,char*datas) {
   string buffer = "";
   for(unsigned int i = 0 ; i < 4 ; i ++ ) {
      buffer[i] = datas[(*index)+i];
   }
   (*index)+=4;
   return buffer;
}

/**
 * Get players list in HL1 protocol
 * @param players players list to fill
 * @param datas Datas to read
 * @post Players list is updated
 * @return True if protocol OK, else false
 */
bool GameServer::getHL1Players(vector<string>*players,char*datas) {
   players->clear();
   unsigned int index = 4 ;
   if ( getHLbyte(&index,datas) != '\x44' )
      return false;
   int nbPlayers = (int)getHLbyte(&index,datas) ;
   for(int i = 0 ; i < nbPlayers ; i ++ ) {
      index++;
      players->push_back("\""+getHLstring(&index,datas)+"\"");   
      index += 8;
   }
   return true;
}

/**
 * Get challenge in HL1 protocol
 * @param challenge string that will contain the challenge
 * @param datas Datas to read
 * @post Challenge is updated
 * @return True if protocol OK, else false
 */
bool GameServer::getHL1Challenge(string*challenge,char*datas) {
   unsigned int index=4;
   if ( getHLbyte(&index,datas) != '\x41' )
      return false;
   *challenge = getHLlong(&index,datas) ;
   return true;
}

/**
 * Get HL server's info in HL1 protocol
 * @param settings Map that will contain settings
 * @param datas Datas to read
 * @post Settings are set
 * @return True if protocol OK, else false
 */
bool GameServer::getHL1Infos(map<string,string>* settings,char*datas) {
   unsigned int index = 5;
   if(datas[4] == '\x6d') { // HL1 support
      (*settings)["ip"] = getHLstring(&index,datas);
      (*settings)["hostname"] = getHLstring(&index,datas);
      (*settings)["map"] = getHLstring(&index,datas);
      (*settings)["gamedir"] = getHLstring(&index,datas);
      (*settings)["gamedesc"] = getHLstring(&index,datas);
      (*settings)["players"] = getHLbyte(&index,datas);
      (*settings)["maxplayers"] = getHLbyte(&index,datas);
      (*settings)["version"] = getHLbyte(&index,datas);
      (*settings)["dedicated"] = getHLbyte(&index,datas);
      (*settings)["os"] = getHLbyte(&index,datas);
      (*settings)["pass"] = getHLbyte(&index,datas);
      (*settings)["ismod"] = getHLbyte(&index,datas);
      (*settings)["urlinfo"] = getHLstring(&index,datas);
      (*settings)["urldl"] = getHLstring(&index,datas);
      (*settings)["unused"] = getHLstring(&index,datas);
      (*settings)["modversion"] = getHLlong(&index,datas);
      (*settings)["modsize"] = getHLlong(&index,datas);
      (*settings)["svonly"] = getHLbyte(&index,datas);
      (*settings)["cidll"] = getHLbyte(&index,datas);
      (*settings)["secure"] = getHLbyte(&index,datas);
      (*settings)["numbots"] = getHLbyte(&index,datas);
      return true;
   }
   else {
      return false;
   }
}

/**
 * Get a Q3 gametype according to a number
 * @param number gametype number
 * @return Gametype string
 */
string GameServer::getQ3GameType(string number)
{
   if ( number == "0" )
      return "FFA";
   else if ( number == "1" )
      return "1v1";
   else if ( number == "2" )
      return "CB TDM";
   else if ( number == "3" )
      return "TDM";
   else if ( number == "4" )
      return "CTF" ;
   else if ( number == "5" )
      return "CA" ;
   else if ( number == "6" )
      return "FF TDM" ;
   else
      return "unknown";
}

/**
 * Parse datas to extract settings and players from a warsow server
 * @param settings map that will contain settings
 * @param players vector that will contain players
 * @param datas Datas to read
 * @post players and settings are set
 * @return True if protocol OK, else false
 */
bool GameServer::parseWSWinfos(map<string,string>* settings,vector<string>* players,char* datas)
{
   unsigned int index,rindex;
   vector<string> split = Tools::stringToVector(datas,"\x0a",1);
   if(split.size()>= 1 )
   {
      vector<string>sets = Tools::stringToVector(split[0],"\\",0);
      for ( unsigned int i = 1 ; i < sets.size() ; i +=2 ) {
         (*settings)[sets[i]] = sets[i+1]  ;
      }
      for (unsigned int i = 1 ; i < (split.size() -1); i ++ )  {
         index = split[i].find("\"") ;
         rindex = split[i].rfind("\"");
         (*players).push_back(Tools::parseQ3Colors(split[i].substr(index,rindex-index+1)));
      }
      return true;
   }
   return false;
}

/**
 * Parse datas to extract settings and players from a Q3 server
 * @param settings map that will contain settings
 * @param players vector that will contain players
 * @param datas Datas to read
 * @post players and settings are set
 * @return True if protocol OK, else false
 */
bool GameServer::parseQ3infos(map<string,string>* settings,vector<string>* players,char* datas)
{
   vector<string> split = Tools::stringToVector(datas,"\x0a",1);
   if(split.size()>= 1 )
   {
      vector<string>sets = Tools::stringToVector(split[0],"\\",0);
      for ( unsigned int i = 1 ; i < sets.size() ; i +=2 ) {
         (*settings)[sets[i]] = sets[i+1]  ;
      }
      for (unsigned int i = 1 ; i < split.size() ; i ++ )  {
         (*players).push_back(Tools::parseQ3Colors(split[i].substr(split[i].find("\""))));
      }
      return true;
   }
   return false;
}

/**
 * Send a UDP query to a server
 * @param ip Server' ip
 * @param port Server's posrt
 * @param sock Pointer to a socket
 * @param query Query to send
 * @post The socket is opened
 * @return True if send OK, else false
 */
bool GameServer::sendQuery(string ip,string port,int* sock,string query) 
{
   sockaddr_in sockname;
   sockname.sin_family=AF_INET; 
	sockname.sin_addr.s_addr=inet_addr(ip.c_str());
   sockname.sin_port=htons(Tools::strToInt(port.c_str()));
   (*sock) = socket(AF_INET,SOCK_DGRAM,0);
   if ( (*sock) <0 ) {
      return false;
   }
   else {
      if (sendto((*sock),query.c_str(),strlen(query.c_str()),0,(struct sockaddr*)&sockname,sizeof(sockname))<0) {
         return false;
      } 
      else {
         return true;
      }
   }
}

/**
 * Get a UDP result from a server
 * @pre A query has been sended
 * @param sock Socket used for communication
 * @param buffer Buffer that will contain result (answer)
 * @return A string containing "0" is everything is OK, else contain the error that occured
 */
string GameServer::getResult(int sock,char*buffer) 
{
   int nbChars;
   fd_set readfds;
   struct timeval tv;
   tv.tv_sec = 3;
   tv.tv_usec = 0;
   FD_ZERO(&readfds);
   FD_SET(sock, &readfds);
   if (select(sock+1, &readfds, NULL, NULL, &tv) >=0 ) {
      if (FD_ISSET(sock, &readfds)) { 
	      nbChars=recv(sock,buffer,MAX_CHARS,0);
         if ((nbChars > -1) && (nbChars <= MAX_CHARS) ) {
            buffer[nbChars] = '\0';
            return "0";
         }
         else {
            return "Read error" ;
         }
      }
      else {
         return "No response" ;
      }
   }
   else {   
      return "Socket down" ;
   }
}

extern "C"
{
	Plugin *contruct_gameserver(BotKernel*b)
	{
		return new GameServer(b);
	}
	void destroy_gameserver(Plugin*p)
	{
		delete p;
	}
	bool q3 (Message*m,Plugin*p,BotKernel*b)
	{
      GameServer * gs = (GameServer*) p;
      char buffer[MAX_CHARS];
      map<string,string> settings;
      vector<string> players,split;   
      string result,query,message;
      int sock;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         split = Tools::stringToVector(m->getPart(4),":",0);
		   if ( split.size() == 2 ) {
            query = "\xff\xff\xff\xffgetstatus";
            if (gs->sendQuery(split[0],split[1],&sock,query)) {
               result = gs->getResult(sock,buffer);
               if ( result == "0" ) {
                  if (gs->parseQ3infos(&settings,&players,buffer) ) {
                     message =  ""+Tools::parseQ3Colors(settings["sv_hostname"]) + " => Q3 " + gs->getQ3GameType(settings["g_gametype"]) + " (" + settings["gameversion"] +") on " +settings["mapname"] +" Players:"+Tools::intToStr(players.size())+ "/"+settings["sv_maxclients"] + " ("+Tools::vectorToString(players,",",0)+") blue:" + settings["Score_Blue"]+" red:"+settings["Score_Red"]+ " Time Left:"+settings["Score_Time"]+ " pb:"+settings["sv_punkbuster"]+ " pv:"+settings["g_needpass"]+"";
                  }
                  else {
                     message = "* Parse error *" ;
                  }  
               }
               else {
                  message = result ;
               } 
            }
            else {
               message = "* Error, unable to send querry *";
            }
            b->send( IRCProtocol::sendMsg(m->getSource(),message) ) ;
            close(sock);           
         }
      }
      return true;
	}
	bool warsow (Message*m,Plugin*p,BotKernel*b)
	{
      GameServer * gs = (GameServer*) p;
      char buffer[MAX_CHARS];
      map<string,string> settings;
      vector<string> players,split;   
      string result,query,message;
      int sock;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         split = Tools::stringToVector(m->getPart(4),":",0);
		   if ( split.size() == 2 ) {
            query = "\xff\xff\xff\xffgetinfo";
            if (gs->sendQuery(split[0],split[1],&sock,query)) {
               result = gs->getResult(sock,buffer);
               if ( result == "0" ) {
                  if (gs->parseWSWinfos(&settings,&players,buffer) ) {
                     message =  ""+Tools::parseQ3Colors(settings["sv_hostname"]) + " => WSW " + settings["g_gametype"] + " (" + settings["version"] +") on " +settings["mapname"] +" Players:"+Tools::intToStr(players.size())+ "/"+settings["sv_maxclients"] + " ("+Tools::vectorToString(players,",",0)+") " + settings["g_match_score"]+ " Time Left:"+settings["g_match_time"]+ " pure:"+settings["sv_pure"]+ " pv:"+settings["g_needpass"]+"";
                  }
                  else {
                     message = "* Parse error *" ;
                  }  
               }
               else {
                  message = result ;
               } 
            }
            else {
               message = "* Error, unable to send querry *";
            }
            b->send( IRCProtocol::sendMsg(m->getSource(),message) ) ;
            close(sock);           
         }
      }
      return true;
	}
	bool hl (Message*m,Plugin*p,BotKernel*b)
	{
      GameServer * gs = (GameServer*) p;
      char buffer[MAX_CHARS];
      map<string,string> settings;
      vector<string> players,split;   
      string result,query,message,challenge;
      int sock;
      if ( m->isPublic() && (m->nbParts() == 5) ) {
         split = Tools::stringToVector(m->getPart(4),":",0);
		   if ( split.size() == 2 ) {
            query = "\xFF\xFF\xFF\xFF\x54\x53\x6F\x75\x72\x63\x65\x20\x45\x6E\x67\x69\x6E\x65\x20\x51\x75\x65\x72\x79\x00";
            if (gs->sendQuery(split[0],split[1],&sock,query)) {
               result = gs->getResult(sock,buffer);
               if ( result == "0" ) {
                  if (!gs->getHL1Infos(&settings,buffer) ) {
                     message = "Error, only HL1 servers are supported" ;
                  }  
                  else {
                     query = "\xFF\xFF\xFF\xFF\x57";
                     if (gs->sendQuery(split[0],split[1],&sock,query)) {
                        result = gs->getResult(sock,buffer);
                        if ( result == "0" ) {
                           if ( !gs->getHL1Challenge(&challenge,buffer) ) {
                              message =  "* Parse error *" ;
                           }
                           else {
                              query = "\xFF\xFF\xFF\xFF\x55";
                              for (unsigned int i = 0 ; i < 4 ; i ++ ) {
                                 query += challenge[i] ;
                              }
                              if (gs->sendQuery(split[0],split[1],&sock,query)) {
                                 result = gs->getResult(sock,buffer);
                                 if ( result == "0" ) {
                                    if (gs->getHL1Players(&players,buffer)) {            
                                       message =  ""+settings["hostname"]+ " => HL1 " + settings["gamedesc"] + " on " +settings["map"] +" Players:"+Tools::intToStr((int)settings["players"][0])+ "/"+Tools::intToStr((int)settings["maxplayers"][0])+ " ("+Tools::vectorToString(players,",",0)+")  pure:"+Tools::intToStr((int)settings["secure"][0])+ " pv:"+Tools::intToStr((int)settings["pass"][0])+"";
                                    }
                                    else {
                                       message =  "* Parse error *";
                                    }
                                 }
                                 else {
                                    message = result ;
                                 } 
                              }
                              else {
                                 message = "* Unable to send query *";
                              }
                           }
                        }
                        else {
                           message = result ;
                        } 
                     }
                     else {
                        message = "* Error, unable to send querry *";
                     }
                  }
               }
               else {
                  message = result ;
               } 
            }
            else {
               message = "* Error, unable to send querry *";
            }
            b->send( IRCProtocol::sendMsg(m->getSource(),message) ) ;
            close(sock);           
         }
      }
      return true;
	}
}
