/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file ignore.cpp
 * @brief Ignore implementation file
 */

#include "ignore.h"

/**
 * Constructor
 */ 
Ignore::Ignore(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Ignore manager";
	this->version = VERSION;
	this->name = "ignore";
	this->bindFunction("addignore",IN_COMMAND_HANDLER,"addIgnore",0,10);
	this->bindFunction("delignore",IN_COMMAND_HANDLER,"delIgnore",0,10);
	this->bindFunction("ignorelist",IN_COMMAND_HANDLER,"ignoreList",0,10);
	this->bindFunction("isignored",IN_COMMAND_HANDLER,"isIgnored",0,10);
	this->bindFunction("35",IN_LOOP,"purifyList",0,30);
	this->bindFunction("",IN_BEFORE_TREATMENT,"testIgnoredUser",0,10);
   this->addRequirement("admin");

   this->doc = new TiXmlDocument(b->getDatasDir()+"ignore.xml");
   if ( this->doc->LoadFile() )
   {
      this->root = this->doc->FirstChild();
   }
   else
   {
      this->initFile();
   }
}

/**
 * Initilaize the XML file by creating root and first childs (file empty structure)
 */
void Ignore::initFile()
{
	TiXmlElement born("trustyrc_ignore_list");
   this->doc->InsertEndChild(born);
   this->root = this->doc->FirstChild();
   this->doc->SaveFile();
}

/**
 * Add a host to the ignore list
 * @param mask mask to ignore
 * @param by User mask that add the ignore
 * @param duration Ignore duration (in seconsd)
 */
void Ignore::addIgnore(string mask,string by,unsigned int duration)
{
   time_t now;
	time(&now);
	char date[17];
   TiXmlElement item("ignore");
   item.SetAttribute("mask",Tools::to_lower(mask));
   item.SetAttribute("timestamp",now);
	strftime(date,18,"%y-%m-%d %X",localtime(&now));
	item.SetAttribute("date",date);
   item.SetAttribute("duration",duration);
   item.SetAttribute("by",by);
   this->root->InsertEndChild(item);
   this->doc->SaveFile();
}

/**
 * Del a host from the ignore list
 * @param index Ignore index
 */
bool Ignore::delIgnore(unsigned int index)
{
   bool result;
   TiXmlHandle docHandle (this->doc);
   TiXmlNode * parent ;
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_ignore_list").Child(index).Element() ;
   if (elem)
   {
      parent = elem->Parent();
      result = parent->RemoveChild(elem);
      this->doc->SaveFile();
      return result;
   }
   return false;
}

/**
 * Check if a host is ignored
 * @param host Host to check
 * @return true if ignored, else false
 */
bool Ignore::isIgnored(string host)
{
	TiXmlElement * elem = this->root->FirstChildElement();
 	if ( elem != NULL) {
     	while (elem != 0) {
			if (Tools::ircMaskMatch(Tools::to_lower(host),Tools::to_lower(elem->Attribute("mask")))) {
				return true;
			}
        	elem = elem->NextSiblingElement();
     	}
		return false;
	}
   return false;
}

/**
 * Give the ignore list
 * @return A vector containing ignored hosts
 */
vector<string> Ignore::getIgnoreList()
{
	char date[17];
   time_t futur;
   string endIgnore = "";
	vector<string> list ;
   TiXmlElement * elem = this->root->FirstChildElement();
   unsigned int i = 0 ;
   while ( elem != NULL )
   {
      futur = Tools::strToInt(elem->Attribute("timestamp"))+Tools::strToInt(elem->Attribute("duration")) ;
      if ( futur > Tools::strToInt(elem->Attribute("timestamp")) ) {
         strftime(date,18,"%y-%m-%d %X",localtime(&futur));
         endIgnore = " to " +string(date) ;
      }
      else {
         endIgnore = " (permanent)" ;
      }
		list.push_back("#"+Tools::intToStr(i)+" "+(string)elem->Attribute("mask")+" on "+(string)elem->Attribute("date")+endIgnore+" "+(string)elem->Attribute("by"));
		elem = elem->NextSiblingElement();
      i++;
   }  
	return list;
}

/**
 * Clear ignore list from outdated ignores
 */
void Ignore::purifyList()
{
   time_t now;
   time(&now);
	TiXmlElement * elem;
   elem = this->root->FirstChildElement();
   while ( elem != NULL )
   {
      if (((string)elem->Attribute("duration")!="0") && ( (Tools::strToInt(elem->Attribute("timestamp"))+Tools::strToInt(elem->Attribute("duration"))) <= now ))
      {
         this->root->RemoveChild(elem);
      }
      elem = elem->NextSiblingElement();
   }   
   this->doc->SaveFile();
}

extern "C"
{
	Plugin *contruct_ignore(BotKernel*b)
	{
		return new Ignore(b);
	}
	void destroy_ignore(Plugin*p)
	{
		delete p;
	}
   bool isIgnored (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Ignore* im = (Ignore*)p;
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }    
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5 )
         {
            if ( adm == NULL )
            {
               exec = true;
            }
            else
            {
               if (adm->isSuperAdmin( m->getSender()))
               { 
                  exec = true;
               }
               else
               {
                  exec = false;
               }
            }
            if ( exec )
            {
               if(im->isIgnored(m->getPart(4)))
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"YES"));
               }
               else
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"NO"));
               }
            }
         }
      }
		return true;
   }
   bool addIgnore (Message*m,Plugin*p,BotKernel*b)
   {
      string time;
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Ignore* im = (Ignore*)p;
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }    
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6)
         {
            if ( adm == NULL )
            {
               exec = true;
            }
            else
            {
               if (adm->isSuperAdmin( m->getSender()))
               { 
                  exec = true;
               }
               else
               {
                  exec = false;
               }
            }
            if ( exec )
            {
			      if ( m->getPart(5).length()>8 )
               {
				      time = m->getPart(5).substr(0,8);
               }
			      else
               {
                  time = m->getPart(5);
				      im->addIgnore(m->getPart(4),m->getSender(),Tools::strtimeToSeconds(time));
				      b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" ignored"));
                  b->getSysLog()->log(m->getPart(4) + " ignored by "+m->getSender(),INFO);
			      }
            }
         }
      }
		return true;
   }
   bool delIgnore (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Ignore* im = (Ignore*)p;
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }    
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5)
         {
            if ( adm == NULL )
            {
               exec = true;
            }
            else
            {
               if (adm->isSuperAdmin( m->getSender()))
               { 
                  exec = true;
               }
               else
               {
                  exec = false;
               }
            }
            if ( exec )
            {
				   if ( im->delIgnore(Tools::strToInt(m->getPart(4))) )
               {
				      b->send(IRCProtocol::sendNotice(m->getNickSender(),"#"+m->getPart(4)+" unignored"));
                  b->getSysLog()->log("#"+m->getPart(4) + " unignored by "+m->getSender(),INFO);
               }
            }
         }
      }
		return true;
   }
   bool ignoreList (Message*m,Plugin*p,BotKernel*b)
   {
      bool exec = false;
      string buffer;
      pPlugin * pp = b->getPlugin("admin");
      Ignore* im = (Ignore*)p;
      Admin * adm ;
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
      }
      else 
      {
         adm = NULL;  
      }    
      if(m->isPrivate())
      {
         if ( adm == NULL )
         {
            exec = true;
         }
         else
         {
            if (adm->isSuperAdmin( m->getSender()))
            { 
               exec = true;
            }
            else
            {
               exec = false;
            }
         }
         if ( exec )
         {
			   b->send(IRCProtocol::sendNotices(m->getNickSender(),im->getIgnoreList()));
         }
      }
		return true;
   }
   bool purifyList (Message*m,Plugin*p,BotKernel*b)
   {
      Ignore* im = (Ignore*)p;
      im->purifyList();
		return true;
   }
	bool testIgnoredUser (Message*m,Plugin*p,BotKernel*b)
	{
      Ignore* im = (Ignore*)p;
	   if ((m->getPart(1) == "PRIVMSG")&&(im->isIgnored(m->getSender()) ) ) {
         return false;
		}
      return true;
	}
}
