/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file lamoule.cpp
 * @brief Lamoule implementation file
 */

#include "lamoule.h"

/**
 * Constructor
 */ 
Lamoule::Lamoule(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "ladder lamoule";
	this->version = VERSION;
	this->name = "lamoule";
	this->bindFunction("lamoule",IN_COMMAND_HANDLER,"lamoule",0,10);
	this->bindFunction("topshot",IN_COMMAND_HANDLER,"topshot",0,10);
	this->bindFunction("top5",IN_COMMAND_HANDLER,"top5",0,10);
	this->bindFunction("player",IN_COMMAND_HANDLER,"player",0,10);
	this->bindFunction("deleteplayer",IN_COMMAND_HANDLER,"deleteplayer",0,10);
	this->bindFunction("toptotal",IN_COMMAND_HANDLER,"toptotal",0,10);
	this->bindFunction("nextscore",IN_COMMAND_HANDLER,"nextscore",0,10);
	this->bindFunction("increase",IN_COMMAND_HANDLER,"increase",0,10);
	this->bindFunction("3600",IN_LOOP,"purifyFile",0,10);
   this->addRequirement("admin");
   this->addRequirement("usersinfos");

   this->nextScore = 0 ;
   this->MAX_SCORE=1000;
   this->FIRST_FLOOR = 250 ;
   this->SECOND_FLOOR  = 800 ;
   this->doc = new TiXmlDocument(b->getDatasDir()+"lamoule.xml");
   if ( this->doc->LoadFile() ) {
      this->root = this->doc->FirstChild();
   }
   else {
      this->initFile();
   }
}

/**
 * Initilaize the XML file by creating root and first childs
 */
void Lamoule::initFile()
{
   TiXmlElement born("trustyrc_lamoule_ladder");
   this->doc->InsertEndChild(born);
   this->root = this->doc->FirstChild();
   TiXmlElement ladderNode( "ladder" );
   this->root->InsertEndChild(ladderNode);

   TiXmlElement topNode("topshot" );
	topNode.SetAttribute( "nick","" );
	topNode.SetAttribute( "score", "0" );
   topNode.SetAttribute( "date", "" );
   this->root->InsertEndChild(topNode);
   
   this->doc->SaveFile();
}

/**
 * Get the lamoule's topshot 
 * @return A vector containing the topshot informations (nick,score,date) (Check the vector size before using it ! (empty if a problem append))
 */
vector<string> Lamoule::getTopShot()
{
   vector<string> back;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement*elem = docHandle.FirstChild("trustyrc_lamoule_ladder").FirstChild("topshot").Element();
   if ( elem ) {
      back.push_back(elem->Attribute("nick"));
      back.push_back(elem->Attribute("score"));
      back.push_back(elem->Attribute("date"));
   }
   return back;
}

/**
 * Set the lamoule's topshot
 * @param nick Topshoter's nick
 * @param score Topshot's score
 * @param date Topshot's date
 */
void Lamoule::setTopShot(string nick,string score,string date)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement*elem = docHandle.FirstChild("trustyrc_lamoule_ladder").FirstChild("topshot").Element();
   if ( elem ) {
      elem->SetAttribute("nick",nick);
      elem->SetAttribute("score",score);
      elem->SetAttribute("date",date);
      this->doc->SaveFile();
   }
}

/** 
 * Add a player in the ladder
 * @param nick Nick to add
 * @param initialScore Initial player's score
 */
void Lamoule::addPlayer(string nick,unsigned int initialScore)
{
   time_t now;
   time(&now);
   TiXmlElement newNode("player");
   newNode.SetAttribute("nick",nick);
   newNode.SetAttribute("total",Tools::intToStr(initialScore));
   newNode.SetAttribute("nbLamoule","1");
   newNode.SetAttribute("lastLamoule",Tools::intToStr(int(now)));
   this->root->FirstChild("ladder")->InsertEndChild(newNode);
   this->doc->SaveFile();
}

/**
 * Generate a random score.
 * If "nextscore" is set, then it's returned
 * @return generated score
 */
int Lamoule::generateScore() {
   int score;
   if ( this->nextScore != 0 ) {
      score = this->nextScore;
      this->nextScore = 0 ;
   }
   else {
      score = Tools::random(0,this->MAX_SCORE);
      if ( score >= this->SECOND_FLOOR ) {
         score = Tools::random(0,score) ;
         if ( score >= this->SECOND_FLOOR ) {
            score = Tools::random(0,score) ;
         }
      }
      else if ((score >= this->FIRST_FLOOR) && (score < this->SECOND_FLOOR)) {
         score = Tools::random(0,score) ;
      }
   }
   return score;
}

/**
 * Increase score for a player
 * @param nick Player's nick
 * @param score Player's score
 * @param diffAttempts Time between two attempts
 * @param checkTop Tell if topshot must be checked
 * @return 'null char' if the player plays too fast, 't' if tophsot, else 'o'
 */
char Lamoule::increaseScore(string nick,int score,unsigned int diffAttempts,bool checkTop)
{
   time_t now;
   time(&now);
	char timeFormat[17];
   bool found = false;
   for(TiXmlElement*child = this->root->FirstChild("ladder")->FirstChildElement(); child!=0; child=child->NextSiblingElement()) {
      if ( Tools::to_lower(child->Attribute("nick")) == Tools::to_lower(nick) ) {
         found = true ;
         if (difftime(now,Tools::strToInt(child->Attribute("lastLamoule"))) > diffAttempts) {
            child->SetAttribute("total",(int)(Tools::strToDouble(child->Attribute("total"))+score));
            child->SetAttribute("nbLamoule",Tools::strToInt(child->Attribute("nbLamoule"))+1);
            child->SetAttribute("lastLamoule",now);
            this->doc->SaveFile();         
         } 
         else {
            return '\0';
         }
         break;
      }
   }
   if (!found) {
      this->addPlayer(nick,score);
   }
   if ( checkTop ) {
      if ( score > Tools::strToInt(this->getTopShot()[1]) ) {
         strftime(timeFormat,18,"%y-%m-%d %X",localtime(&now));
         this->setTopShot(nick,Tools::intToStr(score),timeFormat);
         return 't';
      }
   }
   return 'o';
}

/**
 * Sort a vector elements depending on scores or average
 * @param criterion Sort criterion
 * @param min_attempts Minimum attempts to be sorted
 * @post The vector is sorted
 * @return A vecotr containing nodes sorted
 */
vector<TiXmlElement *> Lamoule::sort(sort_criterion criterion,int min_attempts)
{
   vector<TiXmlElement *> vecBack;
   vector<TiXmlElement*>::iterator it;
   vecBack.clear();
   bool inserted;
   double tempXML,tempVEC;
   for(TiXmlElement*child = this->root->FirstChild("ladder")->FirstChildElement(); child!=0; child=child->NextSiblingElement()) {
      inserted = false;
      if ( Tools::strToInt(child->Attribute("nbLamoule")) >= min_attempts ) {
         if ( vecBack.empty() ) {
            vecBack.push_back(child);   
         }
         else {
            for (it = vecBack.begin();it != vecBack.end();it++) {
               if (criterion == AVERAGE) {
                  tempXML = Tools::strToDouble(child->Attribute("total")) / Tools::strToDouble(child->Attribute("nbLamoule")) ;
                  tempVEC = Tools::strToDouble((*it)->Attribute("total")) / Tools::strToDouble((*it)->Attribute("nbLamoule")) ;
                  if ( (tempXML >= tempVEC)  ) {
                     vecBack.insert(it,child);
                     inserted = true;
                     break;
                  }
               }
               else if ( criterion == TOTAL ) {
                  tempXML = Tools::strToDouble(child->Attribute("total")) ;
                  tempVEC = Tools::strToDouble((*it)->Attribute("total")) ;
                  if ( (tempXML >= tempVEC) ) {
                     vecBack.insert(it,child);
                     inserted = true;
                     break;
                  }
               }
            }
            if ( ! inserted ) {
               vecBack.push_back(child);                  
            }         
         }
      }
   }
   return vecBack;
}

/**
 * Get top 5 depending on score or average
 * @param criterion Sort criterion
 * @param min_attempts Minimal attempts number to appear in ladder
 * @return A vector with top 5 players
 */
vector<string> Lamoule::get5first(sort_criterion criterion,int min_attempts)
{
   vector<string> back;
   vector<TiXmlElement *> vecElems ;
   unsigned int i;
   vecElems.clear();
   back.clear();
   string nbPlayers;
   vecElems = this->sort(criterion,min_attempts);
	nbPlayers = Tools::intToStr(vecElems.size());
   i = 0 ;
   while ((i < 5) && (i < vecElems.size()))
   {  
      back.push_back("("+Tools::intToStr(i+1)+"/"+nbPlayers+") " + ""+string(vecElems[i]->Attribute("nick")) + " => average:" + Tools::doubleToStr(Tools::strToDouble(string(vecElems[i]->Attribute("total")))/Tools::strToDouble(string(vecElems[i]->Attribute("nbLamoule")))) + " total:" + string(vecElems[i]->Attribute("total"))+"" );
      i++;
   }
   return back ;
}

/**
 * Get informations about a player 
 * - total
 * - nb lamoule
 * - average
 * - reset time
 * - rank
 * @param nick Player's nick
 * @param min_attempts Minimal attempts number to have a rank
 * @return A vector containing informations
 */
vector<string> Lamoule::getInfosPlayer(string nick,int min_attempts)
{
   vector<TiXmlElement *> vecElems ;
   vector<string> back;
   vecElems.clear();
	bool found = false;
   vecElems = this->sort(AVERAGE,min_attempts);
   unsigned int i ;
   for (i=0  ; i < vecElems.size() ; i ++ ) {
		if ( Tools::to_lower(vecElems[i]->Attribute("nick")) == Tools::to_lower(nick) )
		{
			found = true;
			break;
		}
   }
   for(TiXmlElement*child = this->root->FirstChild("ladder")->FirstChildElement(); child!=0; child=child->NextSiblingElement()) {
      if ( Tools::to_lower(child->Attribute("nick")) == Tools::to_lower(nick) ) {
         back.push_back(child->Attribute("total"));
         back.push_back(child->Attribute("nbLamoule")) ;
         back.push_back(Tools::doubleToStr(double(Tools::strToDouble(child->Attribute("total")) / Tools::strToDouble(child->Attribute("nbLamoule")))));
         back.push_back(child->Attribute("lastLamoule")) ;
         if (found) {
            back.push_back(Tools::intToStr(i+1));
         }
         else {
            back.push_back("-");
         }
         break;
      }
   }
   return back ;
}

/**
 * Delete a player from the ladder
 * @param nick Nick to delete
 * @return true i the nick has been deleted, else false
 */
bool Lamoule::deletePlayer(string nick)
{
   for(TiXmlElement*child = this->root->FirstChild("ladder")->FirstChildElement(); child!=0; child=child->NextSiblingElement()) {
      if ( Tools::to_lower(child->Attribute("nick")) == Tools::to_lower(nick) ) {
         child->Parent()->RemoveChild(child) ;
         this->doc->SaveFile();
         return true;
      }
   }
   return false;
}

/**
 * Purify the XML file by deleting player who didn't play since a while
 * @param reset_time Time (in seconds) after witch one a player is deleted
 */
void Lamoule::purifyFile(int reset_time) 
{
   time_t now;
   time(&now);
   for(TiXmlElement*child = this->root->FirstChild("ladder")->FirstChildElement(); child!=0; child=child->NextSiblingElement()) {
      if ( difftime(now,Tools::strToInt(child->Attribute("lastLamoule"))) >= reset_time ) {
         child->Parent()->RemoveChild(child) ;
      }
   }
   this->doc->SaveFile();
}

/**
 * Set next score
 * @param score Next score
 */
bool Lamoule::setNextScore(int score) 
{
   if ( score < this->MAX_SCORE ) {
      this->nextScore = score;
      return true;
   }
   else {
      return false;
   }
}

extern "C"
{
	Plugin *contruct_lamoule(BotKernel*b)
	{
		return new Lamoule(b);
	}
	void destroy_lamoule(Plugin*p)
	{
		delete p;
	}
	bool lamoule (Message*m,Plugin*p,BotKernel*b)
	{
      vector<string*> list,chanUsers;
      map<string,Channel*>::iterator fter ;
      map<string,Channel*>* users;
      string nick="";
      pPlugin * ppUI = b->getPlugin("usersinfos");
      Lamoule*lm=  (Lamoule*)p;
      UsersInfos*ui = NULL;
      int rdm,rdm2 ;
      string message,bonus;
      char back;
      if (m->isPublic()) {
         rdm = lm->generateScore() ;
         string lamoule = "la mou" ;
         for (int i = 0; i < (rdm/10) ; i ++)  {
            lamoule += "u" ;
         }
         lamoule += "le !!!" ;
         rdm2 = Tools::random(0,50) ;
         switch (rdm2 ) {
            case 1 : if ( ppUI != NULL ) {
                        ui=(UsersInfos*)ppUI->object;
                        users = ui->getUsers();
                        fter = users->find(m->getSource());
                        if ( fter != users->end() ) {
                           chanUsers = ((Channel*)fter->second)->getUsers();
                           if ( chanUsers.size() > 0 )  {
                              nick = chanUsers[Tools::random(0,chanUsers.size()-1)][0];
                           }
                           else {
                              nick = m->getNickSender();
                           }
                        }
                     }
                     if ( nick == m->getNickSender() )  {
                        bonus = " ("+nick+" cratered. : 0)";
                        rdm = 0;
                     }
                     else {
                        bonus = " (railed by "+nick+" : -100)" ;
                        rdm -= 100 ;
                        if ( rdm < 0 ) {
                           rdm = 0 ;
                        }
                     }
            break;
            case 2 : if ( ppUI != NULL ) {
                        ui=(UsersInfos*)ppUI->object;
                        users = ui->getUsers();
                        fter = users->find(m->getSource());
                        if ( fter != users->end() ) {
                           chanUsers = ((Channel*)fter->second)->getUsers();
                           if ( chanUsers.size() > 0 )  {
                              nick = chanUsers[Tools::random(0,chanUsers.size()-1)][0];
                           }
                           else {
                              nick = m->getNickSender();
                           }
                        }
                     }
                     if ( nick == m->getNickSender() )  {
                        bonus = " ("+nick+" killed himself. : 0)";
                     }
                     else {
                        bonus = " (pummeled by "+nick+" : 0)";
                     }   
                     rdm = 0 ;
            break ;
            case 3 : bonus = " (MH taken !!! : +100)" ;
                     rdm += 100 ;
                     if ( rdm >= 1000 ) {
                        rdm = 950 ;
                     }
            break;
            case 4 : bonus = " (YA taken !!! : +50)" ;
                     rdm += 50 ;
                     if ( rdm >= 1000 ) {
                        rdm = 950 ;
                     }
            break;
            case 5 : bonus = " (RA taken !!! : +100)" ;
                     rdm += 100 ;
                     if ( rdm >= 1000 ) {
                        rdm = 950 ;
                     }
            break;
            default : bonus = "";
         }  
         back = lm->increaseScore(m->getNickSender(),rdm,Tools::strToInt(b->getCONFF()->getValue(p->getName()+".diff_attempts")),true);
         if (back!='\0') {
            message = lamoule+", "+ Tools::intToStr(rdm) + " pour " +m->getNickSender()+""+bonus;
            if ( back == 't' ) {
               message += " (TOPSHOT !!!)";
            }
            b->send(IRCProtocol::sendMsg(m->getSource(),message));
         }    
         else  {  
             b->send(IRCProtocol::sendNotice(m->getNickSender(),"A lamoule every "+b->getCONFF()->getValue(p->getName()+".diff_attempts")+" seconds " + m->getNickSender() +""));
         }
      }
		return true;
	}
	bool topshot (Message*m,Plugin*p,BotKernel*b) {
      Lamoule*lm=  (Lamoule*)p;
      vector<string> back;
      if (m->isPublic()) {
         back = lm->getTopShot();
         b->send(IRCProtocol::sendMsg(m->getSource(),"* Topshot by " + back[0] + " : " + back[1] + " on " +back[2] +" *"));
      }
      return true;
   }
	bool nextscore (Message*m,Plugin*p,BotKernel*b) {
      pPlugin * ppAdm = b->getPlugin("admin");
      Lamoule*lm=  (Lamoule*)p;
      Admin*adm = NULL;
      if (ppAdm!=NULL) {
         adm = (Admin*)ppAdm->object ;
         if (m->isPublic() && adm->isSuperAdmin(m->getSender()) && (m->nbParts()==5)) {
            if (lm->setNextScore(Tools::strToInt(m->getPart(4))) ) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Done." ));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Not done." ));
            }
         }
      }
      return true;
   }
	bool increase (Message*m,Plugin*p,BotKernel*b) {
      pPlugin * ppAdm = b->getPlugin("admin");
      Lamoule*lm=  (Lamoule*)p;
      Admin*adm = NULL;
      if (ppAdm!=NULL) {
         adm = (Admin*)ppAdm->object ;
         if (m->isPublic() && adm->isSuperAdmin(m->getSender()) && (m->nbParts()==6)) {
            if (lm->increaseScore(m->getPart(4),Tools::strToInt(m->getPart(5)),0,false) == 'o' ) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"done." ));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"not done." ));
            }
         }
      }
      return true;
   }
	bool player (Message*m,Plugin*p,BotKernel*b) {
      time_t now;
      time(&now);
      vector<string> infos;
      Lamoule*lm=  (Lamoule*)p;
      if ( m->isPublic() && (m->nbParts()==5) ) {
         infos = lm->getInfosPlayer(m->getPart(4),Tools::strToInt(b->getCONFF()->getValue(p->getName()+".min_attempts"))) ;
         if ( infos.size() > 0 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* "+m->getPart(4)+" => total:" + infos[0]+ " shots:" + infos[1]+ " average:"+infos[2]+ " rank:"+infos[4]+ " reset:"+ Tools::doubleToStr((Tools::strToDouble(b->getCONFF()->getValue(p->getName()+".reset_time")) - difftime(now,Tools::strToInt(infos[3])))/3600) + "H *") ) ;
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Non existent nick *") ) ;
         }
      }
      return true;
   }
	bool deleteplayer (Message*m,Plugin*p,BotKernel*b) {
      pPlugin * ppAdm = b->getPlugin("admin");
      Lamoule*lm=  (Lamoule*)p;
      Admin*adm = NULL;
      if (ppAdm!=NULL) {
         adm = (Admin*)ppAdm->object ;
         if ( m->isPublic() && (m->nbParts()==5) && adm->isSuperAdmin(m->getSender()) ) {
            if (lm->deletePlayer(m->getPart(4)) ) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Done." ));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Not done." ));
            }
         }
      }
      return true;
   }
	bool top5 (Message*m,Plugin*p,BotKernel*b) {
      vector<string> back;
      Lamoule*lm=  (Lamoule*)p;
      if (m->isPublic())
      {
         back =lm->get5first(AVERAGE,Tools::strToInt(b->getCONFF()->getValue(p->getName()+".min_attempts")));
         for (unsigned int i = 0 ; i < back.size() ; i ++ ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),back[i] ));
         }
      }
      return true;
   }
	bool toptotal (Message*m,Plugin*p,BotKernel*b) {
      vector<string> back;
      Lamoule*lm=  (Lamoule*)p;
      if (m->isPublic())
      {
         back =lm->get5first(TOTAL,Tools::strToInt(b->getCONFF()->getValue(p->getName()+".min_attempts")));
         for (unsigned int i = 0 ; i < back.size() ; i ++ ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),back[i] ));
         }
      }
      return true;
   }
   bool purifyFile (Message*m,Plugin*p,BotKernel*b) {
      Lamoule*lm=  (Lamoule*)p;
      lm->purifyFile(Tools::strToInt(b->getCONFF()->getValue(p->getName()+".reset_time")));
      return true;
   }
}
