/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file logfactory.cpp
 * @brief LogFactory implementation file
 */

#include "logfactory.h"

/**
 * Constructor
 */ 
LogFactory::LogFactory(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Channels logging manager";
	this->version = VERSION;
	this->name = "logfactory";
   this->bindFunction("greplog",IN_COMMAND_HANDLER,"greplog",0,25); 
   this->bindFunction("lastseen",IN_COMMAND_HANDLER,"lastseen",0,25); 
	this->bindFunction("JOIN",IN_TYPE_HANDLER,"joinHandler",0,10);
	this->bindFunction("PART",IN_TYPE_HANDLER,"partHandler",0,10);
	this->bindFunction("QUIT",IN_TYPE_HANDLER,"quitHandler",0,10);
	this->bindFunction("NICK",IN_TYPE_HANDLER,"nickHandler",0,10);
	this->bindFunction("TOPIC",IN_TYPE_HANDLER,"topicHandler",0,10);
	this->bindFunction("KICK",IN_TYPE_HANDLER,"kickHandler",0,10);
	this->bindFunction("MODE",IN_TYPE_HANDLER,"modeHandler",0,10);
	this->bindFunction("PRIVMSG",IN_TYPE_HANDLER,"privmsgHandler",0,10);
	this->bindFunction("332",IN_TYPE_HANDLER,"topicJoin",0,10);
	this->bindFunction("333",IN_TYPE_HANDLER,"topicInfos",0,10);
	this->bindFunction("240",IN_LOOP,"cleanLogs",0,30);
   this->bindFunction("",OUT_ALL_MSGS,"sendHandler",0,10);
   this->addRequirement("usersinfos");
   this->logs = new map<string,LogFile*>;
   this->kernel = b;
   DIR*directory = opendir((this->kernel->getDatasDir()+"logs").c_str());
   if (directory == NULL ) {
      if (mkdir((this->kernel->getDatasDir()+"logs").c_str(),0755) == -1 ) {
         this->kernel->getSysLog()->log("Unable to create logs dir (check write access?). Nothing will be logged ...",WARNING);
      }
   } 
}

/**
 * Destructor
 */
LogFactory::~LogFactory() 
{
   this->destroyLogs();
   delete this->logs;
}

/**
 * Destroy log files
 */
void LogFactory::destroyLogs() {
   for(map<string,LogFile*>::const_iterator iter = this->logs->begin();iter!=this->logs->end();iter++) {
      delete iter->second;
   }   
}

/**
 * Tell if a given channel has to be logged
 * @param channel Channel to test
 * @return true if the channel has to be logged, else false
 */
bool LogFactory::hasToBeLogged(string channel) 
{
   return Tools::isInVector(Tools::stringToVector(this->kernel->getCONFF()->getValue(this->name+".log"),","),channel) ; 
}

/**
 * Return channels that are logged and where the bot is present
 * @return A vector containg a pointer on Channel objects
 */
vector<Channel*> LogFactory::getLoggedChannels() {
   vector<Channel*> results;
   map<string,Channel*>::iterator fter ;
   pPlugin * ppUser = this->kernel->getPlugin("usersinfos");
   UsersInfos*ui = (UsersInfos*)ppUser->object ;
   map<string,Channel*>* users = ui->getUsers();
   vector<string> channels = Tools::stringToVector(this->kernel->getCONFF()->getValue(this->name+".log"),",") ;
   for (unsigned int i = 0; i < channels.size(); i ++ ) {
      fter = users->find(channels[i]);
      if(fter != users->end() ) {
         results.push_back(fter->second);
      }
   }
   return results;
}

/**
 * As configuration can change during process,
 * a log file can be still openned while it has not to be logged anymore.
 * This method will close log where the bot is no more present, 
 * or that have not to be logged anymore. 
 * Case : the bot join a channel, a log file is openned. Then the configuration change, and the channel 
 * has no more to be logged. The bot leave the channel. As it's no more in configuration file, the file is not closed.
 * This method will ensure that no useless logs are openned.
 * Call it in a timer.
 */
void LogFactory::cleanLogs() { 
   vector<string> logged,joined;
   pPlugin * ppUser = this->kernel->getPlugin("usersinfos");
   UsersInfos*ui = (UsersInfos*)ppUser->object ;
   map<string,Channel*>* users = ui->getUsers();
   for(map<string,Channel*>::const_iterator iter = users->begin();iter!=users->end();iter++) {
      joined.push_back(iter->first);
   } 
   for(map<string,LogFile*>::const_iterator iter = this->logs->begin();iter!=this->logs->end();iter++) {
      logged.push_back(iter->first);
   } 
   for (unsigned int i = 0 ; i < logged.size() ; i ++ ) {
      if ( logged[i] == "private" ) {
         if ( !hasToBeLogged(logged[i]) ) {
            this->closeLog(logged[i]) ;
         }
      }
      else {
         if ( (!Tools::isInVector(joined,logged[i])) || (!hasToBeLogged(logged[i])) ) {
            this->closeLog(logged[i]) ;
         }
      }
   }
}

/**
 * Log an event in a logfile. If the log file is not opened, 
 * this function will try to do it
 * @param channel Channel where the event occured
 * @param event Event to log
 * @return true if event has been logged
 */
bool LogFactory::log(string channel,string event) {
   LogFile* lf = (*this->logs)[channel] ;
   if ( lf == NULL ) {
      if(this->newLog(channel)) {
         lf = (*this->logs)[channel] ;
      }
   }
   if ( lf != NULL ) {
      return lf->log(event);
   }
   return false;
}

/**
 * Open a new log for a channel and store it
 * @param channel Channel to log
 * @return true is the log file has been opened, else false
 */
bool LogFactory::newLog(string channel) {
   LogFile* newFile = new LogFile(this->kernel->getDatasDir()+"logs/"+channel+"-",false,true,"","%Y");
   (*this->logs)[channel] = newFile;
   return newFile->open();
}

/**
 * Close a log and delete its object
 * @param channel Channel for witch we close log
 */
void LogFactory::closeLog(string channel) {
   LogFile* lf = (*this->logs)[channel];
   if ( lf != NULL ) {
      lf->close();
      delete(lf);
      this->logs->erase(channel); 
   }
}

extern "C"
{
	Plugin* contruct_logfactory(BotKernel*b)
	{
		return new LogFactory(b);
	}
	void destroy_logfactory(Plugin*p)
	{
		delete p;
	}
	bool greplog (Message*m,Plugin*p,BotKernel*b)
	{
      string command;
      if(m->isPublic()&&(m->nbParts()>=5) ) {
         if ( ((LogFactory*)p)->hasToBeLogged(m->getSource()) ) {
            command = "grep \""+Tools::vectorToString(m->getSplit()," ",4)+"\" $(ls -tr "+b->getDatasDir()+"logs/"+ m->getSource() +"-*.log)|sed -e '$!{h;d;}' -e x > "+b->getDatasDir()+"greplog.log" ;
         }
         else {
            command = "grep \""+Tools::vectorToString(m->getSplit()," ",4)+"\" $(ls -tr "+b->getDatasDir()+"logs/"+ m->getSource() +"-*.log)|tail -1 > "+b->getDatasDir()+"greplog.log" ;
         }
         if (system (command.c_str()) != -1 ) {
            std::ifstream greplog((b->getDatasDir()+"greplog.log").c_str() );
            if ( greplog ) {
               string line; 
               while ( getline( greplog, line ) ) {
                  b->send(IRCProtocol::sendMsg(m->getSource(),line));
               }
               greplog.close();
            }
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"Execution error"));
         }
      }
		return true;
	}
	bool lastseen (Message*m,Plugin*p,BotKernel*b)
	{
      string command;
      if(m->isPublic()&&(m->nbParts()==5) ) {
         if ( ((LogFactory*)p)->hasToBeLogged(m->getSource()) ) {
            command = "grep \""+m->getPart(4)+"\" $(ls -tr "+b->getDatasDir()+"logs/"+ m->getSource() +"-*.log)|sed -e '$!{h;d;}' -e x > "+b->getDatasDir()+"greplog.log" ;
         }
         else {
            command = "grep \""+m->getPart(4)+"\" $(ls -tr "+b->getDatasDir()+"logs/"+ m->getSource() +"-*.log)|tail -1 > "+b->getDatasDir()+"greplog.log" ;
         }
         if (system (command.c_str()) != -1 ) {
            std::ifstream greplog((b->getDatasDir()+"greplog.log").c_str() );
            if ( greplog ) {
               string line; 
               while ( getline( greplog, line ) ) {
                  b->send(IRCProtocol::sendMsg(m->getSource(),line));
               }
               greplog.close();
            }
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"Execution error"));
         }
      }
		return true;
	}
	bool joinHandler (Message*m,Plugin*p,BotKernel*b) 
	{
      string channel;
      if (m->getSource()[0] == ':' ) {
         channel = m->getSource().substr(1);
      }
      else {
         channel = m->getSource() ;
      }
      if ( ((LogFactory*)p)->hasToBeLogged(channel)) {
         if ( m->getNickSender() == b->getNick() ) {
            ((LogFactory*)p)->log(channel,"* Now talking on "+ channel );
         }
         else {
            ((LogFactory*)p)->log(channel,"* " + m->getNickSender() + " (" + m->getSender() + ") has joined " + channel);
         }
      }
		return true;
	}
	bool topicJoin (Message*m,Plugin*p,BotKernel*b)
	{
      if ( ((LogFactory*)p)->hasToBeLogged(m->getPart(3))) {
         ((LogFactory*)p)->log(m->getPart(3),"* Topic for " + m->getPart(3) + " is: " + Tools::vectorToString(m->getSplit()," ",4).substr(1));
      }
		return true;
	}
	bool topicInfos (Message*m,Plugin*p,BotKernel*b)
	{
      time_t time;
      if ( ((LogFactory*)p)->hasToBeLogged(m->getPart(3))) {
         time =  Tools::strToUnsignedInt(m->getPart(5)) ;
         ((LogFactory*)p)->log(m->getPart(3),"* Topic for " + m->getPart(3) + " set by " + m->getPart(4) + " at " + ((string)ctime(&time)).substr(0,24)) ;
      }
		return true;
	}
	bool partHandler (Message*m,Plugin*p,BotKernel*b) 
	{
      if ( ((LogFactory*)p)->hasToBeLogged(m->getSource())) {
         if ( m->getNickSender() == b->getNick() ) {
            ((LogFactory*)p)->log(m->getSource(),"* You have left channel " + m->getSource());
         }
         else {
            ((LogFactory*)p)->log(m->getSource(),"* "+m->getNickSender()+" ("+m->getSender()+") has left "+m->getSource()+((m->nbParts()>3)?" ("+Tools::vectorToString(m->getSplit()," ",3).substr(1)+")":""));
         }
      }
		return true;
	}
	bool quitHandler (Message*m,Plugin*p,BotKernel*b)
	{
      vector<string>* channels;
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = (UsersInfos*)ppUser->object ;
      channels = ui->getLastQuitChannels() ;
      for(unsigned int i = 0 ; i < channels->size() ; i ++ ) {
         if (((LogFactory*)p)->hasToBeLogged(channels->at(i))) {
            ((LogFactory*)p)->log(channels->at(i),"* "+m->getNickSender()+" has quit"+" ("+Tools::vectorToString(m->getSplit()," ",2).substr(1)+")");
         } 
      }
		return true;
	}
	bool nickHandler (Message*m,Plugin*p,BotKernel*b) 
	{
      bool isBot;
      vector<Channel*> channels ;
      string event;
      if( m->getPart(2).substr(1) == b->getNick() ) { // as setNick command would have changed nick before this function is called, we use new nick to check
         isBot = true;
         event = "* You are now known as "+m->getPart(2).substr(1);
      }
      else {
         isBot = false;
         event = "* "+m->getNickSender()+" is now known as "+m->getPart(2).substr(1); 
      }
      channels = ((LogFactory*)p)->getLoggedChannels() ;
      for (unsigned int i = 0 ; i < channels.size() ; i ++ ) {
         if (isBot) {
            ((LogFactory*)p)->log(channels[i]->getName(),event);
         }
         else {
            if (channels[i]->isOnChannel(m->getPart(2).substr(1)) ) {
               ((LogFactory*)p)->log(channels[i]->getName(),event);
            }  
         }
      }
		return true;
	}
	bool topicHandler (Message*m,Plugin*p,BotKernel*b)
	{
      if ( ((LogFactory*)p)->hasToBeLogged(m->getSource())) {
         ((LogFactory*)p)->log(m->getSource(),"* "+m->getNickSender()+" has changed the topic to: "+Tools::vectorToString(m->getSplit()," ",3).substr(1));
      }
		return true;
	}
	bool kickHandler (Message*m,Plugin*p,BotKernel*b)
	{
      if ( ((LogFactory*)p)->hasToBeLogged(m->getSource())) {
         if ( m->getPart(3) == b->getNick() ) {
            ((LogFactory*)p)->log(m->getSource(),"* You have been kicked from "+m->getSource()+" by "+m->getNickSender()+" ("+Tools::vectorToString(m->getSplit()," ",4).substr(1)+")");
         }
         else {
            ((LogFactory*)p)->log(m->getSource(),"* "+m->getNickSender()+" has kicked "+m->getPart(3)+" from "+m->getSource()+" ("+Tools::vectorToString(m->getSplit()," ",4).substr(1)+")");
         }
      }
		return true;
	}
	bool modeHandler (Message*m,Plugin*p,BotKernel*b)
	{
      string modeAction = "";
     	string modes = m->getPart(3) ;
  		char sign = '\0';
  		unsigned int nicksIndex = 4 ;
      if ( ((LogFactory*)p)->hasToBeLogged(m->getSource())) {
   		for (unsigned int i = 0 ; i < modes.length() ; i ++ ) {
   			if ((modes[i]=='+')||(modes[i]=='-')) {
   				sign = modes[i];
   			}
   			else {
               switch(modes[i]) {
                  case 'o': modeAction = (string)((sign=='+')?"gives":"removes")+" channel operator status "+(string)((sign=='+')?"to ":"from ")+m->getPart(nicksIndex);
                  break;
                  case 'v': modeAction = (string)((sign=='+')?"gives":"removes")+" voice "+(string)((sign=='+')?"to ":"from ")+m->getPart(nicksIndex);
                  break;
                  case 'b': modeAction = (string)((sign=='+')?"sets":"removes")+" ban on "+m->getPart(nicksIndex);
                  break;
                  default : modeAction = (string)"sets mode "+sign+modes[i]+" "+m->getSource() ;
               }
               ((LogFactory*)p)->log(m->getSource(),"* "+m->getNickSender()+" "+modeAction);
   				nicksIndex++;
   			}
   		}
      }
		return true;
	}
	bool privmsgHandler (Message*m,Plugin*p,BotKernel*b)
	{
      string actionMsg;
      if (m->isPublic() ) {
         if ( ((LogFactory*)p)->hasToBeLogged(m->getSource())) { 
            if ( m->getPart(3)==(":"+(string)"\x01"+"ACTION") ) {
               actionMsg = Tools::vectorToString(m->getSplit()," ",4) ;
               ((LogFactory*)p)->log(m->getSource(),"* "+m->getNickSender()+" "+actionMsg.substr(0,actionMsg.length()-1));
            }
            else {
               ((LogFactory*)p)->log(m->getSource(),"<"+m->getNickSender()+"> "+Tools::vectorToString(m->getSplit()," ",3).substr(1));
            }
         } 
      }
      else {
         if ( ((LogFactory*)p)->hasToBeLogged("private")) { 
            ((LogFactory*)p)->log("private","<"+m->getNickSender()+"> "+Tools::vectorToString(m->getSplit()," ",3).substr(1));
         }
      }
		return true;
	}
   bool cleanLogs (Message*m,Plugin*p,BotKernel*b) 
   {
      ((LogFactory*)p)->cleanLogs();
      return true;
   }
	bool sendHandler (Message*m,Plugin*p,BotKernel*b) 
	{
      string actionMsg;
      if (m->getPart(0) == "PRIVMSG") {
         if ( m->getPart(1)[0] == '#' ) { // public msg
            if ( ((LogFactory*)p)->hasToBeLogged(m->getPart(1))) { 
               if ( m->getPart(2) == (":"+(string)"\x01"+"ACTION") ) {
                  actionMsg = Tools::vectorToString(m->getSplit()," ",3) ;
                  ((LogFactory*)p)->log(m->getPart(1),"* "+b->getNick()+" "+actionMsg.substr(0,actionMsg.length()-1));
               }
               else {
                  ((LogFactory*)p)->log(m->getPart(1),"<"+b->getNick()+"> "+Tools::vectorToString(m->getSplit()," ",2).substr(1));
               }
            } 
         }
      }
      // Following lines are useless cause if the bot disconnects, then object will be deleted, and logs will be closed
      /*else if (m->getPart(0) == "QUIT") {
         ((LogFactory*)p)->destroyLogs();
      }*/
		return true;
	}
}
