/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file quotes.cpp
 * @brief Quotes implementation file
 */ 

#include "quotes.h"

/**
 * Constructor
 */ 
Quotes::Quotes(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Quote storage and access";
	this->version = VERSION;
	this->name = "quotes";
	this->bindFunction("quote",IN_COMMAND_HANDLER,"quote",0,10);
	this->bindFunction("addquote",IN_COMMAND_HANDLER,"addQuote",0,10);
	this->bindFunction("delquote",IN_COMMAND_HANDLER,"delQuote",0,10);
	this->bindFunction("searchquote",IN_COMMAND_HANDLER,"searchQuote",0,10);
	this->bindFunction("quoteinfos",IN_COMMAND_HANDLER,"quoteInfos",0,10);
	this->bindFunction("lastquote",IN_COMMAND_HANDLER,"lastQuote",0,10);
   this->addRequirement("admin");

   this->doc = new TiXmlDocument(b->getDatasDir()+"quotes.xml");
   if ( this->doc->LoadFile() ) {
      this->root = this->doc->FirstChild();
   }
   else {
      TiXmlElement born("trustyrc_quotes");
      this->doc->InsertEndChild(born);
      this->root = this->doc->FirstChild();
      this->doc->SaveFile();
   }
   this->nbQuotes = this->getNbChilds(this->root);
}

/**
 * Return child number for a node
 * @param node Node that we want child's number
 * @return Node's child number
 */
unsigned int Quotes::getNbChilds(TiXmlNode*node)
{
   TiXmlNode * child = node->FirstChild();
   if ( child != NULL) {
      int i = 0 ;
      while (child != 0)
      {
         child = child->NextSibling();
         i += 1 ; 
      }
      return i ;
   }
   else {
      return 0;
   }
}

/**
 * Add a quote in the XML tree
 * @param host Quoter's host
 * @param quote Quote's text
 */
void Quotes::addQuote(string host,string quote)
{
   time_t now;
   time(&now);
	char timeFormat[17];
	strftime(timeFormat,18,"%y-%m-%d %X",localtime(&now));
   TiXmlElement item( "quote" );
   item.SetAttribute( "from", host );
   item.SetAttribute( "date", timeFormat );
   TiXmlText text(quote);
   item.InsertEndChild(text);
   this->root->InsertEndChild(item);
   this->doc->SaveFile();
   this->nbQuotes ++ ;
}

/**
 * Return a quote
 * @param index Quote's index (start at 1)
 * @return Quote's text
 */
string Quotes::getQuote(unsigned int index)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild().Child(index-1).ToElement() ;
   if ( this->nbQuotes > 0 ) {
      if(elem && (index>=1)) {
         return("["+Tools::intToStr(index)+"/"+Tools::intToStr(this->nbQuotes)+"] => "+elem->GetText());
      }
      else {
         return "* Inexistent quote. There are "+Tools::intToStr(this->nbQuotes)+" quotes (from 1 to "+Tools::intToStr(this->nbQuotes)+") *";
      }
   }
   else {
      return "* No recorded quotes *";
   }
}

/**
 * Return a random quote
 * @return Random quote text
 */
string Quotes::getRandomQuote() 
{
   if ( this->nbQuotes > 0 ) {
      return(this->getQuote(Tools::random(1,this->nbQuotes)));
   }
   else {
      return "* No recorded quotes *";
   }
}

/**
 * Return quotes a quote matching to a given pattern an quotes numbers matching too
 * @param pattern Pattern used for search
 * @return A quote and all quotes numbers matching
 */
vector<string> Quotes::searchQuote(string pattern)
{
   vector<string> answer;
   TiXmlHandle docHandle (this->doc);
	string quotes_nums = "";
	string buffer = "";
   unsigned int i = 1;
   if (pattern.length() >= 3) {
	   for(TiXmlElement*child = docHandle.FirstChild().FirstChild().ToElement(); child!=0; child=child->NextSiblingElement()) {
         buffer = child->GetText();
         if ( Tools::to_lower(buffer).find(Tools::to_lower(pattern),0) != string::npos ) {
            quotes_nums += Tools::intToStr(i)+" ";
            answer.push_back("["+Tools::intToStr(i)+"/"+Tools::intToStr(this->nbQuotes)+"] => " + buffer) ;
         }
         i++;
	   }
      if (answer.size() > 0 ) {
         buffer = answer[Tools::random(0,answer.size()-1)];
         answer.clear();
         answer.push_back(buffer);
         answer.push_back("* other matches => " + quotes_nums+ " *");
      }
      else {
         answer.push_back("* No match found *");
      }
   }
   else {
      answer.push_back("* pattern must contain at least of 3 chars *");      
   }
   return answer;
}

/**
 * Delete a quote
 * @param index Quote index (start at 1)
 * @return True if deleted, else false
 */
bool Quotes::delQuote(unsigned int index)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement*elem = docHandle.FirstChild().Child(index-1).ToElement() ;
   if (elem) {
      this->root->RemoveChild(elem);
      this->nbQuotes --;
      this->doc->SaveFile();
      return true;
   }
   else {
      return false;
   }
}

/**
 * Return last inserted quote
 * @return last inserted quote text
 */
string Quotes::getLastQuote()
{
   return this->getQuote(this->nbQuotes);
}

/**
 * Return informations (date en quoter) about a quote
 * @param index Quote index (start at 1)
 * @return Quote's informations
 */
string Quotes::quoteInfos(unsigned int index)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement*elem = docHandle.FirstChild().Child(index-1).ToElement() ;
   if (elem) {
      return ("date:"+(string)elem->Attribute("date")+" from:"+(string)elem->Attribute("from"));
   }
   else {
      return "* Inexistent quote *";
   }
}

extern "C"
{
	Plugin *contruct_quotes(BotKernel*b)
	{
		return new Quotes(b);
	}
	void destroy_quotes(Plugin*p)
	{
		delete p;
	}
	bool quote (Message*m,Plugin*p,BotKernel*b) 
   {
      Quotes*q = (Quotes*)p;
      if (m->isPublic() ) {
         if (m->nbParts() == 4 ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),q->getRandomQuote()));
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),q->getQuote(Tools::strToInt(m->getPart(4)))));
         }
      }
      return true;
   }
	bool addQuote (Message*m,Plugin*p,BotKernel*b) 
   {
      Quotes*q = (Quotes*)p;
      if (m->isPublic() && (m->nbParts()>4) ) {
         q->addQuote(m->getSender(),Tools::vectorToString(m->getSplit()," ",4));
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"* quote added *"));
      }
      return true;
   }
	bool delQuote (Message*m,Plugin*p,BotKernel*b) 
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Quotes*q = (Quotes*)p;
      if ((ppAdm!=NULL) && m->isPublic() && (m->nbParts()==5) ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            if (q->delQuote(Tools::strToInt(m->getPart(4)))) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"* Quote deleted *"));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"* Error *"));
            }
         }
      }
      return true;
   }
	bool searchQuote (Message*m,Plugin*p,BotKernel*b) 
   {
      Quotes*q = (Quotes*)p;
      if ( m->isPublic() && (m->nbParts() > 4)) {
         b->send(IRCProtocol::sendMsg(m->getSource(),q->searchQuote(Tools::vectorToString(m->getSplit()," ",4))));
      }
      return true;
   }
	bool quoteInfos (Message*m,Plugin*p,BotKernel*b) 
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Quotes*q = (Quotes*)p;
      if ((ppAdm!=NULL) && m->isPublic() && (m->nbParts()==5) ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            b->send(IRCProtocol::sendNotice(m->getNickSender(),q->quoteInfos(Tools::strToInt(m->getPart(4)))));
         }
      }
      return true;
   }
	bool lastQuote (Message*m,Plugin*p,BotKernel*b) 
   {
      Quotes*q = (Quotes*)p;
      if (m->isPublic() ) {
         b->send(IRCProtocol::sendMsg(m->getSource(),q->getLastQuote()));
      }
      return true;
   }
}
