/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file remotecontrol.cpp
 * @brief RemoteControl implementation file
 */

#include "remotecontrol.h"

extern "C" void * myThread (void *);

/**
 * Constructor
 */ 
RemoteControl::RemoteControl(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugin that allows remote control via a TCP communication";
	this->version = VERSION;
	this->name = "remotecontrol"; 

   this->MYPORT = Tools::strToUnsignedInt(b->getCONFF()->getValue(this->getName()+".port")) ; 
   this->MAXCLIENTS = Tools::strToUnsignedInt(b->getCONFF()->getValue(this->getName()+".maxclients")) ; 
   this->clients = new int[this->MAXCLIENTS];
	this->pt = new CPPThread();
	if (!pt->exec(myThread,(void*)b)) {
      b->getSysLog()->log("Unable to launch tcp thread",WARNING);
   } 
   // Just a trick to avoid load error. This function will be unregistered at first call
   this->bindFunction("PING",IN_FIRST_WORD,"myUselessFunction",0,5);
}

/**
 * Destructor
 */
RemoteControl::~RemoteControl()
{
   for ( unsigned int i = 0 ; i < this->MAXCLIENTS ; i ++ ) {
      if ( this->clients[i] != 0 ) {
         close(this->clients[i]);
      }
   }
   close(this->sockfd);
   delete [] this->clients;
   delete(this->pt);
}

/**
 * TCP server to receive connections.
 * This function will wait for a connection and talk to the kernel
 * to execute command requested by the user connected
 * @param b Pointer on the kernel
 * @pre This function must be called in a threaded function to don't block the bot
 * @return NOTHING. Juste a NULL pointer
 */
void* RemoteControl::tcpServer(BotKernel*b) 
{
   int  new_fd; // socket for new client
   int highest = 0 ;
   int reuse_addr = 1;
   char buffer[RemoteControl::MAXDATASIZE] ;
   int slot;
   int numbytes;
   struct sockaddr_in my_addr,their_addr;
   socklen_t sin_size;
   struct timeval tv;
   fd_set readfds;

   if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
      b->getSysLog()->log("remotecontrol - socket : "+(string)strerror(errno),ERROR);
      return NULL;
   }
   if (setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &reuse_addr,sizeof(reuse_addr)) == -1) {
      b->getSysLog()->log("remotecontrol - setsockopt : "+(string)strerror(errno),ERROR);
      return NULL;
   }

   my_addr.sin_family = AF_INET;         /* host byte order */
   my_addr.sin_port = htons(this->MYPORT);     /* short, network byte order */
   my_addr.sin_addr.s_addr = INADDR_ANY; /* auto-remplissage avec mon IP */
   bzero(&(my_addr.sin_zero), 8);        /* zero pour le reste de struct */
   if (bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr)) == -1) {
      b->getSysLog()->log("remotecontrol - bind : "+(string)strerror(errno),ERROR);
      return NULL;
   }
   if (listen(sockfd, RemoteControl::BACKLOG) == -1) {
      b->getSysLog()->log("remotecontrol - listen : "+(string)strerror(errno),ERROR);
      return NULL;
   }
   bzero(clients,this->MAXCLIENTS*sizeof(int));
   highest = sockfd ;
   while(true) {
      sin_size = sizeof(struct sockaddr_in);      
      this->setSocketList(&tv,&readfds);
      if (select(highest+1, &readfds, NULL, NULL, &tv) >=0 ) {
         if (FD_ISSET(sockfd, &readfds)) {
            if ((new_fd = accept(sockfd, (struct sockaddr *)&their_addr, &sin_size)) == -1) {
               b->getSysLog()->log("accept : "+(string)strerror(errno),ERROR);
               continue;
            }  
            if ( (slot = this->manageNewConnection(new_fd)) >= 0 ) {
               if ( new_fd > highest ) {
                  highest = new_fd ;
               }
               b->getSysLog()->log("remotecontrol : connexion received from "+(string)inet_ntoa(their_addr.sin_addr)+" (slot "+Tools::intToStr(slot)+")",INFO);
               send(new_fd,"Welcome on trustyRC remote control. Enter your password :\n",58,MSG_NOSIGNAL); // Just for test. In the futur, the client will have to auth as a bot admin
            }     
            else {
               send(new_fd, "Sorry, no free slot! bye\n",25,MSG_NOSIGNAL); 
               close(new_fd);   
            }
         }
         for ( unsigned int i = 0 ; i < this->MAXCLIENTS ; i ++ ) {
            if ( FD_ISSET(clients[i],&readfds) ) {
               if ( (numbytes=recv(clients[i],buffer,RemoteControl::MAXDATASIZE,0)) <= 0 ) {
                  b->getSysLog()->log("remotecontrol : connexion lost from slot "+Tools::intToStr(i),INFO);  
                  close(clients[i]);
                  clients[i] = 0 ;
               }
               else {
                  buffer[numbytes] = '\0';
                  cout << "Received from slot " << i << " : " << buffer;  // to delete
               }
            }
         }
      } 
      else {
         b->getSysLog()->log("remotecontrol : select error",WARNING);
         return NULL;
      }
   }
   return NULL;
}

/**
 * Construct a socket list to be used by select.
 * Sockets are added to fd list
 * @param tv timeval structure
 * @param readfds fd_set will store sockets
 */
void RemoteControl::setSocketList(struct timeval*tv,fd_set*readfds) 
{
   tv->tv_sec = 0;
   tv->tv_usec = 250000; // 0.25 seconds
   FD_ZERO(readfds);
   for ( unsigned int i = 0 ; i < this->MAXCLIENTS ; i ++ ) {
      if ( this->clients[i] != 0 ) {
         FD_SET(this->clients[i],readfds);
      } 
   }
   FD_SET(this->sockfd,readfds);
}

/**
 * Manage new connections. If there is enough room
 * connection will be stored, else, connexion have to be closed
 * @param sock Incomming client socket
 * @return Client slot (-1 if no room)
 */
int RemoteControl::manageNewConnection(int sock)
{
   for(unsigned int i = 0 ; i < this->MAXCLIENTS ; i ++ ) {
      if ( this->clients[i] == 0 ) {
         this->clients[i] = sock ;
         return i;
      }
   }
   return -1;
}

extern "C"
{
	Plugin *contruct_remotecontrol(BotKernel*b)
	{
		return new RemoteControl(b);
	}
	void destroy_remotecontrol(Plugin*p)
	{
		delete p;
	}
   void * myThread(void * arg)
   {
      BotKernel* b = (BotKernel*)arg;
      // wait for plugin loaded
      while (b->getPlugin("remotecontrol")==NULL) {
         usleep(10);
      }
      RemoteControl* rc = (RemoteControl*)b->getPlugin("remotecontrol")->object ;
      rc->tcpServer(b);
	   return NULL;
   }
	bool myUselessFunction (Message*m,Plugin*p,BotKernel*b)
	{
      b->unregisterFunction(myUselessFunction);
		return true;
	}
}
