/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file survey.cpp
 * @brief Survey implementation file
 */

#include "survey.h"

/**
 * Constructor
 */ 
Survey::Survey(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "survey plugin";
	this->version = VERSION;
	this->name = "survey";
   this->surveys.clear();
   this->bindFunction("survey",IN_COMMAND_HANDLER,"launchSurvey",0,10);
   this->bindFunction("cancelsurvey",IN_COMMAND_HANDLER,"stopSurvey",0,10);
   this->addRequirement("admin");
}

/**
 * Launch a survey on a channel
 * @param channel Channel where launch the survey
 * @param question Survey's question
 * @param time Survey length (in seconds)
 * @param answers Possible answers
 * @return true is the survey has been launched, else false
 */
bool Survey::launchSurvey(string channel,string question,unsigned int time,vector<string> answers)
{
   if (!this->surveyRunning(channel)) {
      struct_survey newSurvey;
      newSurvey.channel = channel;
      newSurvey.question = question;
      newSurvey.time = time ;
      newSurvey.answers = answers;
      newSurvey.results.clear();
      newSurvey.functions.clear();
      newSurvey.countDown = NULL;
      for ( unsigned int i = 0 ; i < newSurvey.answers.size() ; i ++ ) {
         newSurvey.results.push_back(0) ;
      }
      newSurvey.voters.clear();
      this->surveys.push_back(newSurvey);
      return true;
   }
   return false;
}

/**
 * Finish the survey
 * @param channel Channel where finish the survey
 * @return A vector containing results
 */
vector<string> Survey::finishSurvey(string channel)
{
   vector<string> back;
   vector<struct_survey>::iterator it = this->surveys.begin();
   while ( it != this->surveys.end() ) {
      if ( (*it).channel == channel ) { 
         for ( unsigned int i = 0 ; i < (*it).answers.size() ; i ++ ) {
            back.push_back("* "+(*it).answers[i]+" : "+Tools::intToStr((*it).results[i]));
         }
         this->surveys.erase(it); 
         return back; 
      }
      it++;
   }
   return back;
}

/**
 * Register a vote for a user
 * @param channel Channel where the user votes
 * @param nick User's nick
 * @param answer User's answer
 * @return true if the vote has been registered, else false
 */
bool Survey::vote(string channel,string nick,string answer)
{
   int id=-1;
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         if (!Tools::isInVector(surveys[i].voters,nick)) {
            id = this->getAnswerId(this->surveys[i].answers,answer);
            if ( id>=0 ) {
               this->surveys[i].results[id] ++ ;
               this->surveys[i].voters.push_back(nick) ;
               return true;
            }
            return false;
         }
         return false;
      }
   }
   return false;
}

/**
 * Stop a survey on a channel
 * @param channel Channel where stop the survey
 * @return true if the survey has been stopped, else false
 */ 
bool Survey::stopSurvey(string channel)
{
   vector<struct_survey>::iterator it = this->surveys.begin();
   while ( it != this->surveys.end() ) {
      if ( (*it).channel == channel ) { 
         this->surveys.erase(it); 
         return true; 
      }
      it++;
   }
   return false;
}

/**
 * Test if a survey is running on a channel
 * @param channel Channel where test if a survey is running
 * @return true if a channel is running, else false
 */
bool Survey::surveyRunning(string channel) {
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         return true;
      }
   }
   return false;
}

/**
 * Get survey's functions
 * @param channel Channel's survey
 * @return A vector containing functions
 */
vector<plugin_function> Survey::getSurveyFunctions(string channel) {
   vector<plugin_function>  back;
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         return this->surveys[i].functions ;
      }
   }
   return back;
}

/**
 * Set survey's functions
 * @param channel Channel's survey
 * @param functions A vector functions
 * @return True if "set" is OK, else false
 */
bool Survey::setSurveyFunctions(string channel,vector<plugin_function> functions)
{
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         this->surveys[i].functions = functions;
         return true;
      }
   }
   return false;
}

/**
 * Get survey's countdown
 * @param channel Channel's survey
 * @return countdown pointer
 */
plugin_function Survey::getCountDown(string channel) 
{
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         return this->surveys[i].countDown ;
      }
   }
   return NULL;
}

/**
 * Set survey's countdown
 * @param channel Channel's survey
 * @param function Countdown function
 * @return True if "set" is OK, else false
 */
bool Survey::setCountDown(string channel,plugin_function function)
{
   for ( unsigned int i = 0 ; i < this->surveys.size() ; i ++ ) {
      if ( this->surveys[i].channel == channel ) {
         this->surveys[i].countDown = function;
         return true;
      }
   }
   return false;
}

/**
 * Get an answer's id
 * @param answers Answers list
 * @param answer Answer to check
 * @return Answer's id (-1 if not present)
 */
int Survey::getAnswerId(vector<string>answers,string answer)
{
   for (unsigned int i = 0 ; i < answers.size() ; i ++ ) {
      if (answers[i] == answer ) {
         return i;
      }
   }
   return -1;
}

extern "C"
{
	Plugin *contruct_survey(BotKernel*b)
	{
		return new Survey(b);
	}
	void destroy_survey(Plugin*p)
	{
		delete p;
	}
   bool stopSurvey (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Survey * sv = (Survey*) p;
      if ((ppAdm!=NULL) && m->isPublic()) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            vector<plugin_function> functions = sv->getSurveyFunctions(m->getSource());
            for(unsigned int i = 0 ; i < functions.size() ; i ++ ) {
               b->unregisterFunction(functions[i]);
            }
            b->unregisterFunction(sv->getCountDown(m->getSource()));
            if ( sv->stopSurvey(m->getSource()) ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),"* Survey canceled "));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"* No survey to cancel *"));
            }
         }
      }
      return true;
   }
	bool endSurvey (Message*m,Plugin*p,BotKernel*b)
   {
      Survey * sv = (Survey*) p;
      vector<plugin_function> functions = sv->getSurveyFunctions(m->getSource());
      for(unsigned int i = 0 ; i < functions.size() ; i ++ ) {
         b->unregisterFunction(functions[i]);
      }
      b->send(IRCProtocol::sendMsg(m->getSource(),"* Survey finished !, results :"));
      b->send(IRCProtocol::sendMsg(m->getSource(),sv->finishSurvey(m->getSource())));
      return true;
   }
	bool vote (Message*m,Plugin*p,BotKernel*b)
	{
      Survey * sv = (Survey*) p;
      if ( m->isPublic() ) {
         if (!sv->vote(m->getSource(),m->getNickSender(),m->getPart(3).substr(b->getCONFF()->getValue("kernel.command_prefix").length()+1))) {
            b->send(IRCProtocol::sendNotice(m->getNickSender(),"ERROR, already voted ?, wrong channel ?")) ;
         }
      }
		return true;
	}
	bool launchSurvey (Message*m,Plugin*p,BotKernel*b)
	{
      Survey * sv = (Survey*) p;
		ConfigurationFile * cf = b->getCONFF();
      vector<plugin_function> functions;
      vector<string> answers,tmpanswers;
      unsigned int duration;
      string prefix = b->getCONFF()->getValue("kernel.command_prefix") ;
      if(m->isPublic()) {
         vector<string> elems = Tools::stringToVector(m->getMessage(),">",0);
         if ( elems.size() == 4 ) {
            answers.clear();
            tmpanswers = Tools::stringToVector(elems[2]," ",0);
            for (unsigned int i = 0 ; i < tmpanswers.size();i++ ) {
               if ( (!Tools::isInVector(answers,tmpanswers[i])) && (tmpanswers[i]!="")) {
                  answers.push_back(tmpanswers[i]);
               }
            }
            duration = Tools::strtimeToSeconds(elems[3]) ;
            if((cf->getValue(p->getName()+".maxduration")=="")||(cf->getValue(p->getName()+".maxduration")=="0")||(Tools::strToUnsignedInt(cf->getValue(p->getName()+".maxduration"))>=duration)) {
               if((cf->getValue(p->getName()+".maxanswers")=="")||(cf->getValue(p->getName()+".maxanswers")=="0")||(Tools::strToUnsignedInt(cf->getValue(p->getName()+".maxanswers"))>=answers.size())) {
                  if ( sv->launchSurvey(m->getSource(),elems[1],Tools::strtimeToSeconds(elems[3]),answers) ) {
                     for (unsigned int i = 0 ; i < answers.size(); i ++ ) {
                        functions.push_back(b->registerFunction(answers[i],p,IN_COMMAND_HANDLER,"surveyAnswer",vote,0,10));
                     }
                     plugin_function func = b->addCountDown(p,endSurvey,m,duration,10);
                     if ( func == NULL ) {
                        sv->stopSurvey(m->getSource());
                        b->send(IRCProtocol::sendMsg(m->getSource(),"Error, too many count downs are running"));
                     }
                     else {
                        b->send(IRCProtocol::sendMsg(m->getSource(),"* Survey launched ("+Tools::intToStr(duration)+" seconds) : "+elems[1]));
                        b->send(IRCProtocol::sendMsg(m->getSource(),"* Possible answers are : "+prefix+Tools::vectorToString(answers," "+prefix,0)));
                        sv->setSurveyFunctions(m->getSource(),functions);
                        sv->setCountDown(m->getSource(),func);  
                     }
                  }
                  else {
                     b->send(IRCProtocol::sendNotice(m->getNickSender(),"Error while launching survey. One already launched ?")) ;
                  }
               }
               else {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"Error : a survey can't have more thant "+cf->getValue(p->getName()+".maxanswers")+" answers.")) ;      
               }
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Error : duration can't exceed "+cf->getValue(p->getName()+".maxduration")+" seconds.")) ;
            }
         }
      }
		return true;
	}
}
