/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file usersinfos.cpp
 * @brief UsersInfos implementation file
 */

#include "usersinfos.h"

/**
 * Constructor
 */ 
UsersInfos::UsersInfos(BotKernel*b)
{
	this->author = "Nicoleau Fabien";
	this->description = "Follow users modes";
	this->version = VERSION;
	this->name = "usersinfos";
	this->bindFunction("JOIN",IN_TYPE_HANDLER,"onJoin",0,10);
	this->bindFunction("PART",IN_TYPE_HANDLER,"onPart",0,10);
	this->bindFunction("QUIT",IN_TYPE_HANDLER,"onQuit",0,10);
	this->bindFunction("KICK",IN_TYPE_HANDLER,"onKick",0,10);
	this->bindFunction("MODE",IN_TYPE_HANDLER,"mode",0,10);
	this->bindFunction("NICK",IN_TYPE_HANDLER,"nick",0,10);
	this->bindFunction("352",IN_TYPE_HANDLER,"event352",0,10);
	this->bindFunction("005",IN_TYPE_HANDLER,"event005",0,10);
	this->bindFunction("240",IN_LOOP,"reloadUsers",0,10);
	//this->bindFunction("test",IN_COMMAND_HANDLER,"test",0,10);
   this->lastQuitChannels.clear();
}

/**
 * Destructor
 */
UsersInfos::~UsersInfos()
{
	for (map<string,Channel*>::const_iterator iter = this->users.begin();iter!=this->users.end();++iter)
	{ 
		delete ((Channel*)iter->second);
	}
}

/**
 * Add a prefixe translation for a mode
 * Examples :
 * @ for o
 * % for h
 * + for v
 * @param mode Mode for witch you give a prefixe
 * @param prefixe Prefixe corresponding to the given mode
 */
void UsersInfos::addPrefixe(char mode,char prefixe)
{
	string translation = "";
	translation += mode;
	translation += prefixe;
	this->prefixes.push_back(translation);
}

/**
 * Get a prefix correspondig to a mode
 * Examples :
 * @ for o
 * % for h
 * + for v
 * @param mode Mode for witch you want the prefix
 * @return Prefix corresponding to the given mode
 */
char UsersInfos::getPrefixe(char mode)
{
	for(unsigned int i = 0 ; i < this->prefixes.size();i++)
	{
		if (this->prefixes[i][0]==mode)
		{
			return this->prefixes[i][1];
			break;
		}
	}
	return '\0';
}

/**
 * Get all prefixes
 * @return all prefixes in a string
 */
string UsersInfos::getPrefixes() {
   string prefixes = "";
	for(unsigned int i = 0 ; i < this->prefixes.size();i++)
	{
      prefixes += this->prefixes[i] ;
	}
   return prefixes;
}

/**
 * Tell if a user has the given mode on a given channel
 * @param mode Mode to test for the user
 * @param channel Channel to check
 * @param nick Nick to check
 * @return True if the user has the mode, else false 
 */
bool UsersInfos::hasMode(string channel,string nick,char mode)
{
   string status = "";
   map<string,Channel*>::iterator fter = this->users.find(channel);
   if ( fter == this->users.end() ) {
      return false;
   }
   else {
      status = ((Channel*)(fter->second))->getStatusByNick(nick);
      if ( status.find(this->getPrefixe(mode)) != string::npos )
         return true;
      else
         return false;
   }
}

/**
 * Get channels where the last user who quitted was present
 * return Channels where the last user who quitted was present
 */
vector<string>* UsersInfos::getLastQuitChannels()
{
   return &this->lastQuitChannels;
}

/**
 * Gives A pointer to the users's kernel attribute
 * @return A pointer to the users's storage attribute
 */
map<string,Channel*>* UsersInfos::getUsers()
{
	return &this->users;
}

extern "C"
{
	Plugin *contruct_usersinfos(BotKernel*b)
	{
		return new UsersInfos(b);
	}
	void destroy_usersinfos(Plugin*p)
	{
		delete p;
	}
	bool reloadUsers(Message*m,Plugin*p,BotKernel*b)
	{
		time_t minus;
		time(&minus);
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		if (users->size()>0)
		{
			map<string,Channel*>::const_iterator toUpdate = users->begin();
			for (map<string,Channel*>::const_iterator iter = users->begin();iter!=users->end();++iter)
			{ 
				if (((Channel*)iter->second)->getLastWhoUpdate()<minus)
				{
					minus = ((Channel*)iter->second)->getLastWhoUpdate() ;
					toUpdate = iter;
				}
			}
			((Channel*)toUpdate->second)->truncateUsersList();
			b->send(IRCProtocol::who(((Channel*)toUpdate->second)->getName(),""));
			((Channel*)toUpdate->second)->notifyWho();
		}
		return true;
	}

	bool onJoin(Message*m,Plugin*p,BotKernel*b)
	{
      string channel="";
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		map<string,Channel*>::iterator fter = users->find(m->getSource());
		if(m->getNickSender()==b->getNick())
		{
			if ( fter != users->end() ) {
				delete ((Channel*)fter->second);
				users->erase(fter);
			}
         channel = m->getSource() ;
         if (channel[0] == ':') { // Unreal IRCD
            channel=channel.erase(0,1);
         }
			(*users)[channel] = new Channel(channel);
			b->send(IRCProtocol::who(channel,""));
		}
		else
		{
			if ( fter == users->end() ) {
				(*users)[m->getSource()] = new Channel(m->getSource());
				b->send(IRCProtocol::who(m->getSource(),""));
			}
			else
			{
				((Channel*)fter->second)->addUser(m->getNickSender(),m->getHostSender(),m->getIdentSender(),"");
			}
		}
		return true;
	}

	bool onPart(Message*m,Plugin*p,BotKernel*b)
	{
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		map<string,Channel*>::iterator fter = users->find(m->getSource());
		if ( fter != users->end() ) {
			if(m->getNickSender()!=b->getNick())
			{
				((Channel*)fter->second)->delUserByNick(m->getNickSender());
			}
			else
			{
				delete ((Channel*)fter->second);
				users->erase(fter);
			}
		}
		return true;
	}

	bool onQuit(Message*m,Plugin*p,BotKernel*b)
	{
      UsersInfos* ui = (UsersInfos*)p;
      ui->getLastQuitChannels()->clear();
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		for (map<string,Channel*>::const_iterator iter = users->begin();iter!=users->end();++iter)
		{ 
			if (((Channel*)iter->second)->delUserByNick(m->getNickSender())) {
            ui->getLastQuitChannels()->push_back(((Channel*)iter->second)->getName());
         }
		}
		return true;
	}

	bool onKick(Message*m,Plugin*p,BotKernel*b)
	{
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		map<string,Channel*>::iterator fter = users->find(m->getSource());
		if ( fter != users->end() ) {
			if(m->getPart(3)!=b->getNick())
			{
				((Channel*)fter->second)->delUserByNick(m->getPart(3));
			}
			else
			{
				delete ((Channel*)fter->second);
				users->erase(fter);
			}
		}
		return true;
	}

	bool mode(Message*m,Plugin*p,BotKernel*b)
	{	
		UsersInfos*obj = (UsersInfos*)p;
		if ( m->getSplit().size() > 4 )
		{
			map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
			map<string,Channel*>::iterator fter = users->find(m->getSource());
			if ( fter != users->end() ) {
				string modes = m->getPart(3) ;
				char sign = '\0';
				unsigned int nicksIndex = 4 ;
				for (unsigned int i = 0 ; i < modes.length() ; i ++ )
				{
					if ((modes[i]=='+')||(modes[i]=='-'))
					{
						sign = modes[i];
					}
					else
					{
						((Channel*)fter->second)->updateStatusByNick(m->getPart(nicksIndex),sign,obj->getPrefixe(modes[i]));
						nicksIndex++;
					}
				}
			}
		}
      return true;
	}

	bool nick(Message*m,Plugin*p,BotKernel*b)
	{
		map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		for (map<string,Channel*>::const_iterator iter = users->begin();iter!=users->end();++iter)
		{ 
			((Channel*)iter->second)->setNickByNick(m->getNickSender(),m->getSource().substr(1)); // source is new nick
		}
		return true;
	}

	bool event352(Message*m,Plugin*p,BotKernel*b)
	{
	   map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		map<string,Channel*>::iterator fter = users->find(m->getPart(3));
		if ( fter != users->end() ) {
			((Channel*)fter->second)->addUser(m->getPart(7),m->getPart(5),m->getPart(4),m->getPart(8));
		}
		return true;
	}

	bool event005(Message*m,Plugin*p,BotKernel*b)
	{
		int index1 = 0;
		int index2 = 0;
		UsersInfos*obj = (UsersInfos*)p;
		string founded;
		vector<string> parts = m->getSplit();
		for (unsigned int i = 0; i < parts.size() ; i ++ )
		{
			if ( parts[i].find("PREFIX=") != string::npos)
			{
				founded = parts[i].substr(8) ;
				index2 = founded.find(")")+1;
				while (founded[index1]!= ')')
				{
					obj->addPrefixe(founded[index1],founded[index2]);
					index1++;
					index2++;
				}
			}
		}
		return true;
	}
  
   /*bool test (Message*m,Plugin*p,BotKernel*b) 
   {
  	   map<string,Channel*>* users = ((UsersInfos*)p)->getUsers();
		map<string,Channel*>::iterator fter = users->find(m->getSource());
		vector<string*> chanUsers = ((Channel*)fter->second)->getUsers();
      for (unsigned int i= 0;i < chanUsers.size() ; i ++ ) {
         cout << "nick:" << chanUsers[i][0] << " host:" << chanUsers[i][1] << " ident:" << chanUsers[i][2] << " status:" << chanUsers[i][3] << endl;
      }
      return true;
   }*/
}
