/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file socket.cpp
 * @brief Socket implementation file
 */

#include "socket.h"

/**
 * Class constructor
 */
Socket::Socket()
{
   this->mySock = -1 ;
   this->state = false;
}

/**
 * Clas destructor
 */
Socket::~Socket()
{
   this->closeSock();
}

/**
 * Connect the socket to a server
 * The socket can be "block type" and use a dedicated IP adress
 * @param port Server port
 * @param serverName Server IP or hostname
 * @param dedicatedIP Dedicated IP used for connextion. Empty if not used
 * @return True if connection OK, else false
 */
bool Socket::connectSock(int port, string serverName, string dedicatedIP)
{
   struct sockaddr_in sockname;
   struct hostent *hostname;
   hostname = gethostbyname (serverName.c_str());
   if ( hostname == NULL )
      return false;
   if (hostname->h_addrtype != AF_INET)
      return false;
  	if ((this->mySock=socket(AF_INET, SOCK_STREAM, 0)) <0)
    	return false;
   memset ((char *) &sockname, 0, sizeof (struct sockaddr_in));
   memcpy ((char *) &sockname.sin_addr, (char*)hostname->h_addr, hostname->h_length);
   sockname.sin_family = AF_INET;
   sockname.sin_port = htons ((u_short)port);
   if ( dedicatedIP != "" )
   {
      struct sockaddr_in msim;
      unsigned long opt = 1;
      msim.sin_addr.s_addr = inet_addr (dedicatedIP.c_str());
      msim.sin_family = AF_INET;
      msim.sin_port = htons (0);
      setsockopt (this->mySock, SOL_SOCKET, SO_REUSEADDR, (char *) &opt,sizeof (opt));
      if (bind (this->mySock, (struct sockaddr *) &msim, sizeof (msim)) < 0)
      {
         cout << "ERROR : unable to bind socket"<<endl;
         exit(-1);
      }
   }
   if ((connect (this->mySock, (struct sockaddr *) &sockname, sizeof (struct sockaddr))) < 0)
      return false;
   this->state = true;
   return true;
}

/**
 * Close the socket
 * @return true if socket well closed, else false
 */ 
bool Socket::closeSock()
{
   if ( close(this->mySock) < 0 ) {
      return false ;
   }
   else {
      this->state = false;
      return true ;   
   }
}

/**
 * Get connection state
 * @return true if the socket is connected, else false
 */
bool Socket::getState()
{
   return this->state;
}

/**
 * Receive a string from the server
 * @return Received string? Empty if nothing received
 */
string Socket::receive()
{
  const unsigned int MAXDATAS = 1024;
  char buffer[MAXDATAS]; 
  int numbytes ;
  struct timeval tv;
  fd_set readfds;
  tv.tv_sec = 0;
  tv.tv_usec = 050000; // 0.05 seconds
  FD_ZERO(&readfds);
  FD_SET(this->mySock, &readfds);
  if (select(this->mySock+1, &readfds, NULL, NULL, &tv) >=0 ) {
    if (FD_ISSET(this->mySock, &readfds)) {
      if((numbytes=recv(this->mySock,buffer,MAXDATAS, 0))<=0) {
        this->closeSock();
        return "";
      }
      else {
        return ((string)buffer).substr(0,numbytes) ;
      }
    }
    else {
      return "";
    }
  }
  this->state = false;
  return "";
}

/**
 * Send a string to the server
 * @param strData String to send
 * @return true if send OK, else false
 */
bool Socket::sendStr(string strData)
{
   if( send(this->mySock, strData.c_str(), strData.length(),MSG_NOSIGNAL) < 0 ) {
      return false ;
   }
   else {
      return true ;
   }
}
