/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file tools.cpp
 * @brief Tools implementation file
 */

#include "tools.h"
#include <iostream>
using namespace std;

/**
 * Constructor
 */
Tools::Tools()
{

}

/**
 * Destructor
 */
Tools::~Tools()
{

}

/**
 * Convert an ascii string to an hexadecimal string
 * @param asciiStr String to convert
 * @return Hexadecimal string
 */
string Tools::asciiToHexa(string asciiStr)
{
   ostringstream oss;
   for (unsigned int i = 0 ; i < asciiStr.length();i++) {
      oss << hex <<(int) asciiStr[i];
   }
   return oss.str();
}

/**
 * Convert an hexadecimal string to an ascii string
 * @param hexaStr String to convert
 * @return Ascii string
 */
string Tools::hexaToAscii(string hexaStr)
{
   ostringstream oss;
   for (unsigned int i = 0 ; i < hexaStr.length();i++) {
      oss << dec <<(char) hexaStr[i];
   }
   return oss.str();
}

/**
 * Check if a string is in a vector string
 * @param v vector to check
 * @param str string to check
 * @return true if present, else false
 */
bool Tools::isInVector(vector<string> v,string str)
{
   bool ret (false);
   for (unsigned int i = 0 ; i < v.size() ; i ++ )
   {
      if(v.at(i) == str)
      {
         ret = true;
         break;
      }   
   }
   return (ret);
}

/**
 * Delete a string from a vector
 * @param v Vector<string> pointer witch must be delete a string
 * @param str string to delete
 */ 
void Tools::delStrFromVector(vector<string>*v,string str)
{
   vector<string>::iterator it = v->begin();
   while ( (it != v->end()) && (v->size() > 0) )
   {
      if ( (*it) == str )
      {
         v->erase(it);
         break;
      }
      else
         it ++ ;
   }
}

/**
 * Convert a int to a string
 * @param number int number to convert
 * @return String convertion result
 */
string Tools::intToStr(int number)
{
   ostringstream oss;
   oss << number;
   return oss.str();
}

/**
 * Convert a double to a string
 * @param number double number to convert
 * @return String convertion result
 */
string Tools::doubleToStr(double number)
{
   ostringstream oss;
   oss << number;
   return oss.str();
}

/**
 * Convert a string to a double
 * if operation fails, result is 0.0
 * @param str string to convert
 * @return Double convertion result
 */
double Tools::strToDouble(string str)
{
   istringstream iss(str);
   double number;
   if( (iss >> number) && ( iss.eof() ) )
   {
      return number;   
   }
   else
   {
      return 0.0 ;
   }
}

/**
 * Convert a string to a int
 * if operation fails, result is 0
 * @param str string to convert
 * @return Int convertion result
 */
int Tools::strToInt(string str)
{
   istringstream iss(str);
   int number;
   if( (iss >> number) && ( iss.eof() ) )
   {
      return number;   
   }
   else
   {
      return 0 ;
   }
}

/**
 * Convert a string to an unsigned int
 * if operation fails, result is 0
 * @param str string to convert
 * @return Int convertion result
 */
unsigned int Tools::strToUnsignedInt(string str)
{
   istringstream iss(str);
   int number;
   if( (iss >> number) && ( iss.eof() ) )
   {
      return number;   
   }
   else
   {
      return 0 ;
   }
}

/**
 * Convert a 'string time' to seconds.
 * Exemple : 2d6h2m1s = 194521 seconds
 * d for 'days'
 * h for 'hours'
 * m for 'minuts'
 * s for 'seconds'
 * @param strtime String time to convert
 * @return seconds convertion
 */
unsigned int Tools::strtimeToSeconds(string strtime) 
{
   unsigned int days=0,hours=0,minuts=0,seconds=0;
   string currentValue = "";
   for(unsigned int i=0;i < strtime.length() ; i ++ ) {
      switch(tolower(strtime[i])) {
         case 'd': days += Tools::strToUnsignedInt(currentValue); currentValue = "" ;
         break;
         case 'h': hours += Tools::strToUnsignedInt(currentValue); currentValue = "" ;
         break;
         case 'm': minuts += Tools::strToUnsignedInt(currentValue); currentValue = "" ;
         break;
         case 's': seconds += Tools::strToUnsignedInt(currentValue); currentValue = "" ;
         break;
         default : currentValue += strtime[i] ;
      }
   }
   seconds += Tools::strToUnsignedInt(currentValue);
   return (seconds+(minuts*60)+(hours*3600)+(days*86400));
}

/**
 * Put an entire string to lower case
 * @param str string to convert
 * @return string converted to lower case
 */
string Tools::to_lower(string str)
{
   string ret = str;
   for ( unsigned int i = 0 ; i < ret.length() ; i ++ )
   {
      ret[i] = tolower(ret[i]);
   }
   return ret;
}

/**
 * Put an entire string to upper case
 * @param str string to convert
 * @return string converted to upper case
 */
string Tools::to_upper(string str)
{
   string ret = str;
   for ( unsigned int i = 0 ; i < ret.length() ; i ++ )
   {
      ret[i] = toupper(ret[i]);
   }
   return ret;
}

/**
 * Generate a random int between a min and a max value
 * @param min Minimum random value
 * @param max Maximum random value
 * @return radom int
 */
int Tools::random(int min,int max)
{
   return (min+(int) (double(max-(min-1))*rand()/(RAND_MAX+1.0))) ;
}

/**
 * Convert a vector<string> to a string
 * @param vec Vector to convert
 * @param separator Elements separator in generated string
 * @param start Indice for starting conversion
 * @return A string containing vector values
 */
string Tools::vectorToString(vector<string> vec,string separator,unsigned int start )
{
   string ret = "";
   for (unsigned int i = 0 ; i < vec.size() ; i ++ )
   {
      if ( i >= start )
      {
         ret += vec[i];
         if ( i != (vec.size() -1) )
         {
            ret += separator;
         }   
      }
   }
   return ret;
}

/**
 * Split a string in elements inserted in a vector
 * @param str String to split
 * @param separator Delimitor for string
 * @param start Index of the first element to split
 * @return A vector containing strings
 */
vector<string> Tools::stringToVector(string str,string separator,unsigned int start)
{
   vector<string> ret;
   ret.clear();
   string::size_type i = 0 ;
   unsigned int nbWords = 0;
   while ( i < str.length() )
   {
		i = str.find(separator) ;
		if ( i == string::npos ) {
			ret.push_back(str);
			i = str.length();
		}
		else {
			if ( nbWords >= start) {
				ret.push_back(str.substr(0,i));
			}
			str = str.substr(i+separator.length());
			nbWords++;
			i = 0;
		}
   }
   return ret ;
}

/** 
 * Gather vector's elements to make a vector with less elements
 * @param v vector for witch gather elements
 * @param separator separator for elements
 * @param length vector's element length
 * @return The new vector with gathered elements
 */
vector<string> Tools::gatherVectorElements(vector<string> v,string separator,unsigned int length)
{
   vector<string> newVector;
   string buffer ;
   for (unsigned int i = 0 ; i < v.size() ; i+=length )
   {
      buffer = "";
      for ( unsigned int j = 0; (j < length) && (i+j<v.size()) ; j++ )
      {
         buffer+=(v[i+j]+separator);
      }
      newVector.push_back(buffer);
   }
   return newVector;
}

/**
 * Escape a char in a string
 * @param str String containing chars
 * @param c Char to escape in the string
 * @return Initial string with escaped chars
 */
string Tools::escapeChar(string str,char c)
{
   unsigned int i = 0 ;
   while ( i < str.length() )
   {
      if ( str[i] == c )
      {
         str.insert(i,"\\");
         i += 2;
      }
      else
      {
         i += 1;
      }
   }
   return str ;
}

/**
 * Log an event in a file, with timestamp
 * @param fileName Log file
 * @param str Log event
 * @param timestamp tell if a timestamp must appear in the log line
 * @param truncate tell if the file must be truncated
 */
void Tools::log(string fileName,string str,bool timestamp,bool truncate)
{
   time_t now ;
   time(&now);
   char timeFormat[17];
   ofstream file;
   if ( truncate ) {
      file.open(fileName.c_str(), std::ios_base::trunc );
   }
   else {
      file.open(fileName.c_str(), std::ios_base::app );
   }
   if(file.is_open()) {
      if (timestamp) {
	      strftime(timeFormat,18,"%y-%m-%d %X",localtime(&now));
         string time(timeFormat);
         str = time +" "+str ;
      }
      file << str << endl ;
      file.close();
   }
}

/**
 * Convert special chars from a string to be compatible to URL format
 * @param str String to encode
 * @return String compatible to URL format
 */
string Tools::urlencode(string str)
{
   string::size_type pos ;
   string signs[44] = {"+", " ", "&", "é", "\"", "è", "ç", "à", "=", "°", ",", ";", ":", "ù", "^", "$", "µ", "¬", "¹", "#", "{", "[", "|", "`", "\\", "^", "@", "]", "}", "?", "/", "§", "£", "š", "â", "ê", "ô", "î", "û", "ä", "ï", "ö", "ë","ü"};
   string codes[44] = {"%2B", "+", "%26", "%E9", "%22", "%E8", "%E7", "%E0", "%3D", "%B0", "%2C", "%3B", "%3A", "%F9", "%5E", "%24", "%B5", "%AC", "%B9", "%23", "%7B", "%5B", "%7C", "%60", "%5C", "%5E", "%40", "%5D", "%7D", "%3F", "%2F", "%A7", "%A3", "%A8", "%E2", "%EA", "%F4", "%EE", "%FB", "%E4", "%EF", "%F6", "%EB", "%FC"};
   for ( unsigned int i = 0 ; i < 44 ; i ++ )
   {
      pos = str.find(signs[i]) ;
      while ( pos != string::npos )
      {
         str.replace(pos,signs[i].length(),codes[i]);
         pos = str.find(signs[i]) ;
      }
   }
   return str ;
}

/** 
 * Clear accents from a sentence
 * @param str String to clear
 * @return String with no accents
 */
string Tools::clearAccents(string str) 
{
   string::size_type pos ;
   string accents[16] = {"é","è","ê","ë","ç","à","â","ä","ù","û","ü","ù","ô","ö","î","ï"} ;
   string clean[16]   = {"e","e","e","e","c","a","a","a","u","u","u","u","o","o","i","i"} ;
   for ( unsigned int i = 0 ; i < 16 ; i ++ )
   {
      pos = str.find(accents[i]) ;
      while ( pos != string::npos )
      {
         str.replace(pos,accents[i].length(),clean[i]);
         pos = str.find(accents[i]) ;
      }
   }
   return str ;
}

/**
 * Clean all HTML chars in a string :
 * Replace accents marks and delete font marks
 * @param str String with no html tags
 */
string Tools::cleanHTML(string str)
{
   const unsigned int NB_DEL = 7;
   const unsigned int NB_REPLACE = 30;
   string::size_type pos ;
   string del[NB_DEL] = {"&middot;","<b>","</b>","<i>","</i>","<br>","<br />"} ;
   string codes[NB_REPLACE] = {"&nbsp;","&#39;","&#039;","&quot;","&eacute;","&egrave;","&#233;","&#232;","&gt;","&lt;","&#234;","&#224;","&agrave;","&ccedil;","&icirc;","&ecirc;","&ouml;","&ocirc;","&iuml;","&ugrave;","&Ccedil;","&acirc;","&laquo;","&raquo;","&ucirc;","&sup2;","&copy;","&#64;","&amp;","&deg;"};
   string signs[NB_REPLACE] = {" ","'","'","\"","é","è","é","è",">","<","ê","à","à","ç","î","ê","ö","ô","ï","ù","ç","â","«","»","û","²","©","@","&","°"};
   for ( unsigned int i = 0 ; i < NB_REPLACE ; i ++ )
   {
      pos = str.find(codes[i]) ;
      while ( pos != string::npos )
      {
         str.replace(pos,codes[i].length(),signs[i]);
         pos = str.find(codes[i]) ;
      }
   }
   for ( unsigned int i = 0 ; i < NB_DEL ; i++ )
   {
      pos = str.find(del[i]) ;
      while ( pos != string::npos )
      {
         str.erase(pos,del[i].length());
         pos = str.find(del[i]) ;
      }
   }
   return str ;
}

/**
 * Strip all Quake III Arena color codes from a string
 * @param raw String with q3 color codes
 * @return String with no q3 colors
 */
string Tools::parseQ3Colors(string raw) 
{
	string parsedString = "";
	unsigned int i = 0;
	while (i < raw.length() )
	{
		if ( raw[i] != '^' ) {
			parsedString += raw[i];
			i++;
		}
		else {
			if ((i<(raw.length()-1)) && (raw[i+1] !='x') && (raw[i+1] !='X')) {
				i += 2;
			}
			else if ((i<(raw.length()-7)) && ((raw[i+1]=='x')||(raw[i+1]=='X'))) {
				i += 8;
			}
			else { //possible?
				i++;
			}
		}
		
	}
	return parsedString;
}

/**
 * Check if an IRC host match to a mask
 * @param request Irc host
 * @param mask mask
 * @return true if host match to regex, else false
 */
bool Tools::ircMaskMatch(string request,string mask) {
   if (fnmatch(mask.c_str(),request.c_str(),0)==0 )
      return true;
   else
      return false;
   
}

/**
 * Check if two masks match
 * Thanks to "BigBourin" (fr) for this function
 * @param str1 first mask
 * @param str2 second mask
 * @return 1 if match, else 0
 */
int Tools::masksMatch(char*str1,char*str2)
{
  if (*str1 == 0 && *str2 == 0)
    return (1);
  if (*str1 == '*')
    {
      if (*str2 == 0)
      return (masksMatch(str1 + 1, str2));
      else
      return (masksMatch(str1 + 1, str2) || masksMatch(str1, str2 + 1));
    }
  if (*str2 == '*')
    {
      if (*str1 == 0)
      return (masksMatch(str1, str2 + 1));
      else
      return (masksMatch(str1 + 1, str2) || masksMatch(str1, str2 + 1));
    }
  if (*str1 == *str2)
    return (masksMatch(str1 + 1, str2 + 1));
  return (0);
}

/** 
 * Copy a file from a source to a destination
 * @param source Source file (to copy)
 * @param destination Destination file
 * @return true if copy ok, else false
 */
bool copyFile(string source,string destination) {
   char tmp;
   ifstream fi(source.c_str(), ios::in|ios::binary);
   if (!fi) return false;
   ofstream fo(destination.c_str(), ios::out|ios::binary);
   if (!fo) return false;
    
   while ( fo && fi.get(tmp) )
      fo.put(tmp);
   return fo.good() && fi.eof();
}
